%                               Example1_24.m
%
%                    Reliability problem in Example 1.24
%
% Input data:
%    X1  - yield strength   (uniform)    - 1st Input RV
%    X2  - section modulus  (lognormal)  - 2nd Input RV
%   mean(X1)=0.1, sigma(X1)=0.03, mean(X2)=100, sigma(X2)=10
%   
% Also given that X1 and X2 are corelated with correlation coefficient =0.4
%
% 
%                       Nataf transformation 
%      ( transforms correlated non-normal random variables X={X1,X2} to 
%           correlated standard normal random variables Z= {Z1,Z2} 
%                   - see Chapter 1,section 1.10.4)
%
%      This program outputs the correlation coefficient of the Z vector,
%           given 0.4 for the  X vector-(X1, X2)
%----------------------------------------------------------------------------
clc
clear all
close all
%
n=2;            % size of the vector RV X or Z
%
nrho=7;         % number of points in the range (-1, +1) of the correlation
                %         coefficient 
%                
mean=zeros(n);
cov=zeros(n);
sigma=zeros(n);
%
% means and coefficient of variation of input non-normal Rvs x 
mean(1)=0.1;
mean(2)=100;
sigma(1)=0.03;
sigma(2)=10;
%
% coefficients of variation
cov(1)=sigma(1)/mean(1);        % for x1
cov(2)=sigma(2)/mean(2);        % for x2
%
% form the input 'rho - 2 x 2 matrix of RV x
rho(1,1)=1.0;
rho(1,2)=0.4;
rho(2,1)=0.4;
rho(2,2)=1.0;
rho_12_Input=rho(1,2);
%
%
% Simulaton of Equation (1.171)
% nx, ny - no. of divisions on x1 & x2 variables (axes)  
% xmin,xmax,ymin & ymax - Extents of x1 & x2 variables (axes)
nx=100;   
ny=100;        
xmin=-5;
xmax=5;
ymin=-5;
ymax=5;
%
%
param=zeros(n,2);    % initialization of the parameters 
                         %  of the input RVs x1 and x2
%
% 1st variable x1 - Uniform  (see, for example, Ang and Tang 1984)
param(1,1)=mean(1)-2*sigma(1);
param(1,2)=mean(1)+2*sigma(1);
%
%2nd variable x2 - log Normal
zetax2=log(1+cov(2)*cov(2));    % See, for example, Nowak & Collins 2000
logmu=log(mean(2))-0.5*zetax2;  %   (Relability of structures, McGraw-Hill)
%
logsig=sqrt(zetax2);
param(2,1)=logmu;
param(2,2)=logsig;
%
% Recomputation of the mean 'm', variance 'varn' of the input RVs x1 and x2 
%               corresponding to the computed 'param' vector
m=zeros(n);
varn=zeros(n);
stdev=zeros(n);
 [m(1),varn(1)]=unifstat(param(1,1),param(1,2));
 [m(2),varn(2)]=lognstat(param(2,1),param(2,2));
%
 stdev=sqrt(varn);
%
%____________________________________________________________________
% Let  'rho_12'   - correlation coefficient of RV x vector (given)
% Let  'rhod_12'  - correlation coefficient of RV z vector - to be found
%
% Finding 'rhod_12' - of the standard Gaussian correlated random variables 
   %  executed here by reverse way of assuming values for 'rhod_12' on the 
      %    RHS of Equation (1.171) and coputing 'rho_12' on the LHS
                % and then interpolating the value of 'rhod_12' 
                              %   when rho_12 =0.4
                              %
drho=linspace(-0.90,0.90,nrho);    % increment of rhod_12 
%
% rhoNat=0;   % Initialization of the correlation coefficient matrix
%                      %  of the input RVs 'x'
%
rho_12=zeros(nrho,1);    % initialization of rho(1,2) of x vector RV
%
%
% start of numerical integration of the doubleintegral (Equation 1.171)
%
           dx=(xmax-xmin)/nx;
           dy=(ymax-ymin)/ny;
%           
% 
           for k=1:nrho;
             rhod_12(k)=drho(k)      % assumed rhod_12 of z vector RV
             k     % counter for rhod_12
%           
             sum1=0;
             F1=0;
             sum=0;
%             
             for i=1:nx+1;
                 x=xmin+(i-1)*dx;
                 sum2=0;
%
               for j=1:ny+1;
                     ii=i;
                     jj=j;
                     y=ymin+(j-1)*dy;
%
                   std1=stdev(1);
                   std2=stdev(2);
%
                   rho1=[1  rhod_12(k)
                       rhod_12(k)   1];
                   xx=[x
                       y];
                   mx=[0
                       0];
%
%
                    A=((2*pi)^(0.5*2))*((det(rho1))^0.5);
                    B=0.5*(xx-mx)'*inv(rho1)*(xx-mx);
                    t3=exp(-B)/A;  %    standard multi-normal pdf of RVs z
%
%
                   t11=normcdf(x,0,1);   % marginal pdf of ist RV z
                   t22=normcdf(y,0,1);    % marginal pdf of 2nd RV z
% 
%
                   t1=unifinv(t11,param(1,1),param(1,2))-m(1);
                   X1=t1;
%                
                   t2=logninv(t22,param(2,1),param(2,2))-m(2);
                   X2=t2;
%     
                  if j>1
                   fxy2=t1*t2*t3/(std1*std2);
                   sum2=sum2+0.5*dy*(fxy1+fxy2);
                   fxy1=fxy2;
                  else
                   fxy1=t1*t2*t3/(std1*std2);
                  end
%
               end;
%
               if i>1
                 F2=sum2;
                 sum=sum+0.5*dx*(F1+F2);    % total area under the curve
                 F1=F2;
               else
                 F1=sum2;
               end
%
             end
%
             rho_12(k)=sum
%
           end
%         
%
% rhod_12
% rho_12
% itr=(1:k);
%
% plot of rhod_12 vs. rho_12(k) 
plot(rhod_12,rho_12,'black')
%
%    Interpolation of Rhod_12 when Rho_12=0.4
  rhod_12_Output=interp1(rho_12,drho',rho_12_Input)
