%                           Fig1_2.m
%
%       Coin tossing Experiment - statistical regularity
%
%               The code generates Figure 1.2 
%
% The sample space for the coin tossing experiment is {H,T}
% If the coin is fair, intutively P(H)=P(T)=1/2
% These two outcomes are simulated here by
%    i)   first generating a uniform distribution RV between [0,1]
%    ii)  N = Total number of Uniform RVs generated
%           = Total number of tosses in eah trial
%         if U < 0.5, this interval is mapped to {H} and
%         if U > 0.5, this interval is mapped to {T}
%    iii) As the uniform RV is being generated, the number of 'HEAD's
%         are counted.
%    iv)  relative frequency of HEADs = Number of 'HEAD's / N
%    v)   The experiment is repeated with N varied from 1000 to 10000 in steps of 300%
%__________________________________________________________________________
%
clc
clear all
close all
%
% simulation of a uniform RV
for j=1:34
   N=100+(j-1)*300;
   [j N]    % simlation counter and the number of tosses in jth trial
   isum(j)=0;
   x(j)=N;
   for i=1:N
        u(i)=rand;
        if(u(i)>0.5)
            isum(j)=isum(j)+1;
        end
   end
%
%  
% for each N, probability of "HEAD' is evaluated
   y(j)=isum(j)/N;
%
end
%
% plotting of j Vs. y(j)
figure(1)
plot(x,y,'.black')
xlabel('N - total no. of tosses','fontsize',12)
ylabel('no of heads / N','fontsize',12)
ylim([0 1])
%
%
% End of the program
%-------------------------------------------------------------------------
