%                    Fig10_1.m
%
%               Lorenz oscillator ( original Example 9.3 revisited)
%
%      This program contains two seperate runs with NP=10
%          -  one with coalescence based on PSO in innovation 
%          -  second one based on pseudo code 2 without this PSO type of
%                                                innovation
%
%  First run (with PSO) uses Pbest only (Equation 10.1 and 10.2) 
%                       in innovation 
%
%  Second run according to pseudo code 2 (i.e. according to
%      Equation 9.40 with coalescence of Xvector & cost function also in innovation)
%
%
% NOTE:  Since it is a stochastic search algorithm, 
%               better convergence may be tried over a number of independent MC runs >1
%__________________________________________________________________________
clc
clear all
close all
%
%
global sigma r b
sigma = 10;
r = 28;
b = 8/3;
%
gen_max=300;
%
% reference (true) values for plotting
x1_ref(1:gen_max+1)=sigma;
x2_ref(1:gen_max+1)=r;
x3_ref(1:gen_max+1)=b;
%
[z]=lorenz_states();
%
Np=10;   % number of particles (this is less than those used in Figure 9.9)
D=3;
%
% initial sample of particles
 xl=-[10;10;0];xu=[51;60;40];
 x1=repmat(xl,1,Np)+diag(xu-xl)*rand(D,Np);
%
% 
 x1_init=x1;
%
x1=x1_init;
%
% function evaluation at the zeroth iteration 
for i=1:Np
    cost(i)=fun_chao(z,x1(:,i));
end
%
%__________________________________________________________________________
%          Modification based on PSO to pseudo code 2 
% To add G_best & P_best in innovation   (Equation 10.1 and 10.2)
%                   (G_best is not utilized here)
%
P_best = x1
[val_min,G_loc] = min(cost)
G_best = x1(:,G_loc)  
%__________________________________________________________________________
%
min_cost=min(cost);
cost_plot(1)=mean(cost);
%
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
x3_plot(1)=mean(x1(3,:));
%
count=0;
% 
%
tic
while count<gen_max
    count                    % iteration counter
%
    for  ii=1:Np
%
        r1=randi(Np);
%
        r1_st(ii)=r1;
        r2=ii;
        while (r2==r1 || r2==ii)
            r2=randi(Np);
        end
        r2_st(ii)=r2;
        r3=ii;
        while (r3==r1 || r3==r2 || r3==ii)
            r3=randi(Np);
        end
        r3_st(ii)=r3;
%
    end
%    
    G=repmat(G_best,1,Np);
%    
    X=x1;
    X_pr=x1; 
%    
%      h1=(G(:,r3_st)-x1(:,r2_st));   % not used -   m x Np
%
    h2=(P_best(:,r3_st)-x1(:,r2_st));  % modification to innovation (Equation 10.1)
    %                           (P_best only utilized)  - m x Np size
%
    h=[h2];    % -    m x Np size
%
    h_pr=h;
%   
    [gain]=PSO_new_KS_EnKF_gain_matrix(X,h,h_pr,X_pr,Np);
%                       gain is     m x m size
%   
     theta=1-1/exp(count-1);
     F=theta;
%     
    for i=1:Np
%    
%
%         trial(:,1)=x1(:,r1_st(i))-2*F*gain*h(:,i); % (Equation 10.2 - same as 9.40)     
           trial(:,1)=x1(:,i)-2*F*gain*h(:,i); % (Equation 10.2 - same as 9.40)      
        %               factor 2 is for a better convergence
%        
        newpop(:,i)=trial;
        score(i)=fun_chao(z,trial);
%
    end
% 
%__________________________________________________________________________
%               Modification to pseudo code 2 
% To evaluate G_best & P_best at each iteration
   for i=1:Np
       if score(i)<cost(i)
          P_best(:,i) = newpop(:,i);
       end
   end
 %
   [new_val_min,G_loc] = min(score');
    val_max=max(cost);
    if  new_val_min < val_min
       G_best = newpop(:,G_loc);
    end
 %   
    val_min=new_val_min;
%
    index = (score <cost) ;
    cost(index) = score(index);
    x1(:,index) = newpop(:,index);
%__________________________________________________________________________   
    min_cost=min(cost);
%
    count=count+1;
    cost_plot(count+1)=mean(cost);
    x1_plot(count+1)=mean(x1(1,:));
    x2_plot(count+1)=mean(x1(2,:));
    x3_plot(count+1)=mean(x1(3,:));
%    
end
toc
%
iteration=1:(gen_max+1);
figure(1)
plot(iteration,x1_plot,'black'),hold on   % Figure 10.1a
plot(iteration,x1_ref,'--black')
figure(2)
plot(iteration,x2_plot,'black'),hold on   % Figure 10.1b
plot(iteration,x2_ref,'--black')
figure(3)
plot(iteration,x3_plot,'black'),hold on    % Figure 10.1c
plot(iteration,x3_ref,'--black')
figure(4)
plot(iteration,cost_plot,'black'),hold on   % Figure 10.1d
% 
%
%
%
%
%
%__________________________________________________________________________
%            Second rUN by original pseudo code 2  (Equation 9.40)
Np=10;
D=3;
%
%
 xl=-[10;10;0];xu=[51;60;40];
 x1=repmat(xl,1,Np)+diag(xu-xl)*rand(D,Np);
%
%  
 x1_init=x1;
% 
x1=x1_init;
%
%
% function evaluation at the zeroth iteration 
for i=1:Np
    cost(i)=fun_chao(z,x1(:,i));
end
%
%
min_cost=min(cost);
cost_plot(1)=mean(cost);
%
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
x3_plot(1)=mean(x1(3,:));
count=0;
% 
%
tic
while count<gen_max
    count      % counter
%
    for  ii=1:Np
%
        r1=randi(Np);
%
        r1_st(ii)=r1;
        r2=ii;
        while (r2==r1 || r2==ii)
            r2=randi(Np);
        end
        r2_st(ii)=r2;
        r3=ii;
        while (r3==r1 || r3==r2 || r3==ii)
            r3=randi(Np);
        end
        r3_st(ii)=r3;
%
    end
%    
%
    X=x1;
    X_pr=x1; 
    hh=(x1(:,r3_st)-x1(:,r2_st));   % Innovation (original 
%
     h=[min_cost-cost(r2_st); hh]; 
%
    h_pr=h;
    [gain]=PSO_new_KS_EnKF_gain_matrix(X,h,h_pr,X_pr,Np);
% 
%   
     theta=1-1/exp(count-1);
      F=theta;  
    for i=1:Np
%         
%   
        trial(:,1)=x1(:,i)-2*F*gain*h(:,i); % Equation 9.40
%
%        
        newpop(:,i)=trial;
        score(i)=fun_chao(z,trial);
%
    end
%
%
 index = (score <cost);
%
    cost(index) = score(index);
    x1(:,index) = newpop(:,index);
%
    min_cost=min(cost);
%
    count=count+1;
    cost_plot(count+1)=mean(cost);
    x1_plot(count+1)=mean(x1(1,:));
    x2_plot(count+1)=mean(x1(2,:));
    x3_plot(count+1)=mean(x1(3,:));
%  
end
toc
%
iteration=1:(gen_max+1);
figure(1)
plot(iteration,x1_plot,'--black'),hold on    % Figure 10.1a
figure(2)
plot(iteration,x2_plot,'--black'),hold on     % Figure 10.1b
figure(3)
plot(iteration,x3_plot,'--black'),hold on     % Figure 10.1c
figure(4)
plot(iteration,cost_plot,'--black'),hold on   % Figure 10.1d


%------------------------------------------
