%                            Fig10_2.m
%
%               Lorenz oscillator (original Example 9.3 revisited)
%
%      Modified scrambling strategy (Equation 10.4)
%            by utilizing the scrambling of Differential Evolution (DiEv)
%                  +  relaxation approach
%
%
%   This program has two runs with Np=10 (less number of particles
%                                         compared with Figure 9.9)                
%
%        i) First run is with MODIFIED scrambling on parameters 
%                        in a particle - Equation 10.4 
%                        +  relaxation strategy with CR=0.1
%
%       ii) Second run is with original scrambling on particle as a whole
%                       -  Equation 9.40 
%
% NOTE:   Since it is a stochastic search algorithm, 
%               better convergence may be tried for a number of independent MC runs >1
%__________________________________________________________________________
clc
clear all
close all
%
%
global sigma r b
%
sigma = 10;
r = 28;
b = 8/3;
%
 gen_max=300;
%
% reference (true) values for plotting
x1_ref(1:gen_max+1)=sigma;
x2_ref(1:gen_max+1)=r;
x3_ref(1:gen_max+1)=b;
%
[z]=lorenz_states();
%
Np=10;   % number of particles
D=3;    % number of parameters to be estimated
%
%
% initial sample of particles
 xl=-[10;10;0];xu=[51;60;40];
 x1=repmat(xl,1,Np)+diag(xu-xl)*rand(D,Np);
%
% % 
 x1_init=x1;
%
x1=x1_init;
%
% function evaluation at the zeroth iteration
for i=1:Np
    cost(i)=fun_chao(z,x1(:,i));
end
%
min_cost=min(cost);
cost_plot(1)=mean(cost);
%
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
x3_plot(1)=mean(x1(3,:));
%
count=0;
%
Pr=0.9;    %  probability with regards to the relaxation approach 
%                       (section 10.1.2 of Chapter 10) 
CR=1-Pr;  % = 0.1;  
%
%
tic
while count<gen_max
    count    % iteration counter
%
    for  ii=1:Np
%
        r1=randi(Np);
%
        r1_st(ii)=r1;
        r2=ii;
        while (r2==r1 || r2==ii)
            r2=randi(Np);
        end
        r2_st(ii)=r2;
        r3=ii;
        while (r3==r1 || r3==r2 || r3==ii)
            r3=randi(Np);
        end
        r3_st(ii)=r3;
%
    end
%    
    X=x1;
    X_pr=x1;
%    
    hh=(x1(:,r3_st)-x1(:,r2_st));
%    
    h=[min_cost-cost(r2_st); hh];  % innovation matrix -   (m+1 x Np) size
%     
    h_pr=h;
%    
    [gain]=new_KS_EnKF_gain_matrix(X,h,h_pr,X_pr,Np);  % (m x m+1) size
%
         theta=1-1/exp(count-1);
         F=theta;
      innv = gain*h;    % (m x Np) size
%      
    for i=1:Np
%
%
%           below is the modification to scrambling strategy at parameter
%           level of each particle
     j = randi(D);  %  random seletion of parameter for ith particle 
%  
    for k = 1:D  % Modification adopted from Differential Evolution (DiEv) 
        %          for scrambling at the parameter level 
%
       temp = x1(j,i) - 2*F*innv(j,i);   
%                   factor 2 used for better convergence
%
%
%       
      if rand < CR || k == D   %  ralaxation approach - CR =(1-Pr) 
          trial(j,1) = temp;   % update accepted
%
        else
            trial(:,1)=x1(:,i);  % update rejected and instead the earlier result accepted
      end
%
%--------------------------------------------------------------------------
% Modification to scrambling strategy based on Differential Evolution (DiEv)
%                       (see Equation 10.4)
       j = mod(j+1,D);   
        if j == 0
            j = D;
        end
%--------------------------------------------------------------------------        
    end  % end of the parameter loop k=1:D
%
        score=fun_chao(z,trial);
        if score<=cost(i)
            x2(:,i)=trial;
            cost(i)=score;
        else
            x2(:,i)=x1(:,i);
        end
%        
    end % end of the particle loop i=1:Np
%    
    min_cost=min(cost);
    x1=x2;
    count=count+1;
    cost_plot(count+1)=mean(cost);
%
    mx1=mean(x1(1,:));
    mx2=mean(x1(2,:));
    mx3=mean(x1(3,:));
%  
    x1_plot(count+1)=mx1;
    x2_plot(count+1)=mx2;
    x3_plot(count+1)=mx3;
%  
%     
end
%  
toc
%
iteration=1:(gen_max+1);
%
%plotting
figure(1)               % Figure 10.2a
plot(iteration,x1_plot,'black'),hold on
plot(iteration,x1_ref,'--black')
figure(2)               % Figure 10.2b
plot(iteration,x2_plot,'black'),hold on
plot(iteration,x2_ref,'--black')
figure(3)               % Figure 10.2c
plot(iteration,x3_plot,'black'),hold on
plot(iteration,x3_ref,'--black')
figure(4)               % Figure 10.2d
plot(iteration,cost_plot,'black'),hold on
%
%
%
%
%__________________________________________________________________________
%       2nd run of pseudo code 2 (Equation 9.40) with original scramblng 
%       on particles as a whole 
%
Np=10; 
D=3;
%
% initial particles
 xl=-[10;10;0];xu=[51;60;40];
 x1=repmat(xl,1,Np)+diag(xu-xl)*rand(D,Np);
%
%  
 x1_init=x1;
%
x1=x1_init;
%
% function evaluation at xerot iteration
for i=1:Np
    cost(i)=fun_chao(z,x1(:,i));
end
%
min_cost=min(cost);
cost_plot(1)=mean(cost);
%
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
x3_plot(1)=mean(x1(3,:));
%
count=0;
%
tic
while count<gen_max
    count     % counter
    %   
    for  ii=1:Np
%
        r1=randi(Np);
%
        r1_st(ii)=r1;
        r2=ii;
        while (r2==r1 || r2==ii)
            r2=randi(Np);
        end
        r2_st(ii)=r2;
        r3=ii;
        while (r3==r1 || r3==r2 || r3==ii)
            r3=randi(Np);
        end
        r3_st(ii)=r3;
%
    end
%    
    X=x1;X_pr=x1; hh=(x1(:,r2_st)-x1(:,r3_st));
    h=[cost(r2_st)-min_cost; hh];
    h_pr=[cost(r2_st)-min_cost; hh];
    [gain]=new_KS_EnKF_gain_matrix(X,h,h_pr,X_pr,Np);
%
         theta=1-1/exp(count-1);
         F=theta;
%         
    for i=1:Np
        %      
%         if rand(1)<CR
%             F=theta;            
%
 trial(:,1)=x1(:,i)-2*F*gain*h(:,i);
%
%         else
%             trial(:,1)=x1(:,i);
%         end
%
        score=fun_chao(z,trial);
        if score<=cost(i)
            x2(:,i)=trial;
            cost(i)=score;
        else
            x2(:,i)=x1(:,i);
        end
    end
    min_cost=min(cost);
    x1=x2;
    count=count+1;
    cost_plot(count+1)=mean(cost);
    x1_plot(count+1)=mean(x1(1,:));
    x2_plot(count+1)=mean(x1(2,:));
    x3_plot(count+1)=mean(x1(3,:));
%   
end
toc
%
% plotting
figure(1)
plot(iteration,x1_plot,'--black')   % Figure 10.2a
figure(2)
plot(iteration,x2_plot,'--black')   % Figure 10.2b
figure(3)
plot(iteration,x3_plot,'--black')   % Figure 10.2c
figure(4)
plot(iteration,cost_plot,'--black') % Figure 10.2d



%------------------------------------------
