%                   called by Figure10_3.m
%
%                   calls rotor_matrices.m 
%
%          Analysis of the FE model in frequency domain
%
%  i) form stiffness, mass and damping matrices first 
%  ii) assembly
%  iii) inversion
%  iv) store the responses (obtained in frequency domain) at the bearings 
%__________________________________________________________________________
function [X]=rotor_states_full_model(byy,bzz,cyy,czz)
%
dfr=1;
%
frq1=500;
frq2=2500;
%
nFr=(frq2-frq1)/dfr+1;
%
fr=linspace(frq1,frq2,nFr)'; % time variables
%
%
dens=7806.0;
ee=2.075*10^11;
se=ee;
%
%
nodes=19;
nel=18;
%
XEL(1) = 0.0127;
XEL(2) = 0.0381;
XEL(3) = 0.0254;
XEL(4) = 0.0127;
XEL(5) = 0.0127;
XEL(6) = 0.0051;
XEL(7) = 0.0076;
XEL(8) = 0.0127;
XEL(9) = 0.0076;
XEL(10) = 0.0305;
XEL(11) = 0.0254;
XEL(12) = 0.0381;
XEL(13) = 0.0381;
XEL(14) = 0.0203;
XEL(15) = 0.0178;
XEL(16) = 0.0102;
XEL(17) = 0.0304;
XEL(18) = 0.0127;
%
dia(1) = 0.0102;
dia(2) = 0.0204;
dia(3) = 0.0152;
dia(4) = 0.0406;
dia(5) = 0.0406;
dia(6) = 0.0660;
dia(7) = 0.0660;
dia7 = 0.0304;
dia(8) = 0.0508;
dia8 = 0.0356;
dia(9) = 0.0508;
dia(10) = 0.0254;
dia(11) = 0.0254;
dia(12) = 0.0304;
dia(13) = 0.0304;
dia(14) = 0.0254;
dia(15) = 0.0254;
dia(16) = 0.0762;
dia(17) = 0.0406;
dia(18) = 0.0406;
dia18 = 0.0304;
%
ndof = nodes * 4;
%
%
ifreq=0;
%
force_vec=zeros(ndof,1);    
%
for freq=frq1:dfr:frq2
     om = freq;
    ifreq=ifreq+1;
%    
    fr(ifreq)=freq;    
    [ifreq freq]
%    
    ams=zeros(ndof,ndof);
    stf=zeros(ndof,ndof);
    cdp=zeros(ndof,ndof);
%    
    amt=zeros(8,8);
    amr=zeros(8,8);
    aka=zeros(8,8);
    akb=zeros(8,8);
    akc=zeros(8,8);
    ag=zeros(8,8);
%    
    NROW =0;
    NCOL =0;
    
    for i=1:nel
        sl = XEL(i);
        area(i)=0.25*pi*dia(i)*dia(i);
        xmom(i)=pi*(dia(i)^4)/64.0;
        if(i==7)
            area(i)=0.25*pi*(dia(i)*dia(i)-dia7*dia7);
            xmom(i)=pi*(dia(i)^4-dia7^4)/64.0;
        end
        if(i==8)
            area(i)=0.25*pi*(dia(i)*dia(i)-dia8*dia8);
            xmom(i)=pi*(dia(i)^4-dia8^4)/64.0;
        end
        if(i==18)
            area(i)=0.25*pi*(dia(i)*dia(i)-dia18*dia18);
            xmom(i)=pi*(dia(i)^4-dia18^4)/64.0;
        end
%        
        sm = dens*area(i);
        si = xmom(i);
%        
        xmiy=xmom(i);
        sa = dia(i)/2.0;
%        
        [amt,amr,akb,ag]=rotor_matrices(nodes,nel,sl,sm,sa,se,si,dens,xmiy);
%        
        NROW =  (i-1) * 4;
        NCOL =  (i-1) * 4;
%        
        for ii = NROW+1:NROW+8
            for jj =NCOL+1:NCOL+8
%                
                NR = ii-NROW;
                NC = jj-NCOL;
%                
                ams(ii,jj) = ams(ii,jj) + amt(NR,NC) + amr(NR,NC);
                stf(ii,jj) = stf(ii,jj) + akb(NR,NC);
                cdp(ii,jj) = cdp(ii,jj) - om*ag(NR,NC);
%                
            end
        end
%        
    end %assembly  for  NEL Elements over
 %
 %  add masses of disk elements
    m1 = 5;
    idof = (m1-1)*4+1;
    jdof = (m1-1)*4+1;
    ams(idof,jdof) = ams(idof,jdof) + 1.4;
    ams(idof+1,jdof+1) = ams(idof+1,jdof+1) + 1.4;
    ams(idof+2,jdof+2) = ams(idof+2,jdof+2) + 0.00136;
    ams(idof+3,jdof+3) = ams(idof+3,jdof+3) + 0.00136;
    cdp(idof+2,jdof+3) = cdp(idof+2,jdof+3) - 0.00203 *om;
    cdp(idof+3,jdof+2) = cdp(idof+3,jdof+2) + 0.00203 *om;
 %  
%  stiffness values for 1st bearing  
    b1yy=byy;
    b1yz=0;      
    b1zy=0;
    b1zz=bzz;
% 
% stiffness values for 2nd bearing
    b2yy=byy;
    b2yz=0;     
    b2zy=0;
    b2zz=bzz;
% 
%  damping values for 1st bearing 
    c1yy=cyy;
    c1yz=0;
    c1zy=0;
    c1zz=czz;
%    
% damping values for 1st bearing 
    c2yy=cyy;
    c2yz=0;
    c2zy=0;
    c2zz=czz;   
%------------------------------------------------------------------------
%    Frequency response of the FE model with order = ndof ( of full model)
%
%  add the stiffness values of the bearings at the appropriate dofs    
    stf(41,41)=stf(41,41)+b1yy;
    stf(42,42)=stf(42,42)+b1zz;
    stf(57,57)=stf(57,57)+b2yy;
    stf(58,58)=stf(58,58)+b2zz;
    stf(41,42)=stf(41,42)+b1yz;
    stf(42,41)=stf(42,41)+b1zy;
    stf(57,58)=stf(57,58)+b2yz;
    stf(58,57)=stf(58,57)+b2zy;
    %
%  add the damping values of the bearings at the appropriate dofs       
    cdp(41,41)=cdp(41,41)+c1yy;
    cdp(42,42)=cdp(42,42)+c1zz;
    cdp(57,57)=cdp(57,57)+c2yy;
    cdp(58,58)=cdp(58,58)+c2zz;
    cdp(41,42)=cdp(41,42)+c1yz;
    cdp(42,41)=cdp(42,41)+c1zy;
    cdp(57,58)=cdp(57,58)+c2yz;
    cdp(58,57)=cdp(58,57)+c2zy;
%
% imalance forces at the disk locations
     force_vec(17)=1.4*0.001*freq*freq*1e-3;
     force_vec(18)=1.4*0.001*freq*freq*1e-3;
%
% frequency matrix
    D_mat=zeros(ndof,ndof);
    D_mat=complex(stf-freq^2*ams,freq*cdp);    
%  
    frs=zeros(ndof,1);   
%
% 
    frs=inv(D_mat)*force_vec;    
%    
%  store the responses at the bearing locations 
    X(1,ifreq)=abs(frs(41));
    X(2,ifreq)=abs(frs(42));
    X(3,ifreq)=abs(frs(56));
    X(4,ifreq)=abs(frs(57));
%  
end
%
end

