%                   Fig10_4.m
%
%      This program runs COMBEO for rotor dynamics problem
%
%          Minimizing the unbalance response without constraints
%                           on critical speeds
%
%   Calls Rotor_unbalance_response.m   and   ENKF_blend_egt_opt_Nset1.m
%
%  NOTE: The final result on the shaft diameters is stored in pop_init.mat
%          file for use in Fig10_5.m
%__________________________________________________________________________
close all  
clear all;
clc;
%
nx = 15;                 % dimensionality of benchmark functions 1000
ne = 10;                 % population size
D=nx;
Np=ne;
%
% 
%
Lb=0.004;
Ub=0.05; 
max_count=200;
Max_FEs=max_count*ne;    % maximum iterations
%
%
stopfitness = 1e-19;      % stop if fitness < stopfitness (minimization)
% 
%
%  bounds on the design variables (15 shaft diameters)
 xL=[0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004];
 xU=[0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05];
%  
 pop=zeros(D,Np);
%
% initial population of the design vector X
 pop=repmat(xL,1,Np)+diag(xU-xL)*rand(D,Np)
% 
     for j=1:D
       x_plot(j,1)=mean(pop(j,:),2);         
     end
% evaluation of cost function   
[val]=Rotor_unbalance_response(pop,Np)
% 
%
cost_plot(1)=mean(val)
%
%--------------------------------------------------------------------------
FEs = ne;                
count = 1;
itr(1)=count;
%
weights = 1/ne*ones(1,ne);
r1_st = zeros(ne,1);
r2_st = zeros(ne,1);
r3_st = zeros(ne,1);
%
tic;
%
while (FEs <= Max_FEs-ne)
    
    for i = 1:ne
        r1 = i;
        while r1 == i
            r1 = randi(ne);
        end
        r1_st(i) = r1;
        r2 = i;
        while (r2 == r1 || r2 == i)
            r2 = randi(ne);
        end
        r2_st(i) = r2;
        r3 = i;
        while (r3 == r1 || r3 == r2 || r3 == i)
            r3 = randi(ne);
        end
        r3_st(i) = r3;
    end
%
        newpop = pop;
        oldpop=pop;
%  
%         
            CR2 = rand;   % for update by blending
% 
            newpop = ENKF_blend_egt_opt(pop,Np,D,weights,r1_st,r2_st,r3_st,CR2,val);
%    
%
            [newval]=Rotor_unbalance_response(newpop,Np);
%--------------------------------------------------------------------------            
%  
            index = (newval < val);
            val(index) = newval(index);
            pop(:,index) = newpop(:,index);
%
            weights = sum(val'.*weights) - (val'.*weights);
            weights = weights/sum(weights);
%   
%    
    min_val = min(val);
    FEs = FEs + ne;
    count = count + 1;
%    
    % 
    fprintf(1, 'FEs = %d, Iterations = %d: mean(val) = %e \n\n', FEs, count, mean(val));
%    
    mv(count) = mean(val);
    cost_plot(count)=mv(count);
     for j=1:D
       x_plot(j,count)=mean(pop(j,:),2);         
     end
%
%     
    itr(count)=count;
%    
    % Break, if fitness is good enough
     if mean(val) <= stopfitness
%    
      break;
     end
    
     if(count==max_count)
        pop 
     end
%    
end
toc;
save pop_init.mat pop
% 
% 
%
figure(D+1)             %  
plot(itr,cost_plot,'black')   % Figure 10.4
%
%
for j=1:D
    figure(j)                 % optional 
    plot(itr,x_plot(j,:),'black')  % 15 shaft diameters at the end of iterations
%                                           (not shown in Chapter 10)
end
%
