%                   Fig10_6and10_7.m
%
%               This program runs rotor dynamics problem
%
%           Minimizing the unbalance response along with
%           consraints on citical speeds (Equation 10.8)
%
%   Figure 10.6a - mean of f(X)in Equation (10.8)
%   Figure 10.6b - mean of the three components of f(X) (see Equation 10.8)
%   Figure 10.7  - mean of the first two critical speeds
%
%           Calls Rotor_unbalance_response.m   and   ENKF_blend_egt_opt.m
%
%
%  NOTE:  The final solution (at the end of 200 iterations)
%           for the design variables (shaft diameters)
%               is stored by this program in the varaible 'pop(D,Np)' 
%               This solution is the input to Fig10_8.m
%
%
%__________________________________________________________________________
close all  
clear all;
clc;
%
%
% 
nx = 15;            % dimensionality of the problem (15 shaft diameters)
ne = 10;            % population size
D=nx;
Np=ne;
%
%
%
Lb=0.004;
Ub=0.05;   
max_count=200;
Max_FEs=max_count*ne;
stopfitness = 1e-19;      % stop if fitness < stopfitness (minimization)
% 
%
%
 xL=[0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004;0.004];
 xU=[0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05;0.05];
%  
 pop=zeros(D,Np);
%
 pop=repmat(xL,1,Np)+diag(xU-xL)*rand(D,Np);
 
     for j=1:D
       x_plot(j,1)=mean(pop(j,:),2);         
     end
  
[val val1 val2 val3 crspeeds]=Rotor_unbalance_response(pop,Np);
% 
%
cost_plot(1)=mean(val);   %  total function value in Equation 10.8
   crsp_plot(1,1)=mean(crspeeds(1,:),2);    %1st  critical speed  
     crsp_plot(2,1)=mean(crspeeds(2,:),2);   % 2nd  critical speed
      cost1_plot(1)=mean(val1);   %  mean of 1st term in Equation 10.8
      cost2_plot(1)=mean(val2);  %  mean of 2nd term in Equation 10.8
      cost3_plot(1)=mean(val3);  %  mean of 3rd term in Equation 10.8
%--------------------------------------------------------------------------
FEs = ne;             
count = 1;
itr(1)=count;
%
weights = 1/ne*ones(1,ne);
r1_st = zeros(ne,1);
r2_st = zeros(ne,1);
r3_st = zeros(ne,1);
%
tic;
while (FEs <= Max_FEs-ne)
    
    for i = 1:ne
        r1 = i;
        while r1 == i
            r1 = randi(ne);
        end
        r1_st(i) = r1;
        r2 = i;
        while (r2 == r1 || r2 == i)
            r2 = randi(ne);
        end
        r2_st(i) = r2;
        r3 = i;
        while (r3 == r1 || r3 == r2 || r3 == i)
            r3 = randi(ne);
        end
        r3_st(i) = r3;
    end
%
% 
            CR2 = rand;  
%           
      newpop = ENKF_blend_egt_opt(pop,ne,D,weights,r1_st,r2_st,r3_st,CR2,crspeeds);
%
     [newval newval1 newval2 newval3 new_crspeeds]=Rotor_unbalance_response(newpop,Np);
%--------------------------------------------------------------------------            
%   
            index = (newval < val);
            val(index) = newval(index);
            pop(:,index) = newpop(:,index);
            crspeeds(:,index)=new_crspeeds(:,index);
            weights = sum(val'.*weights) - (val'.*weights);
            weights = weights/sum(weights);
            %
            val1(index) = newval1(index);
            val2(index) = newval2(index);
                val3(index) = newval3(index);
%     
    min_val = min(val);
    FEs = FEs + ne;
    count = count + 1;
     cost1_plot(count)=mean(val1);
      cost2_plot(count)=mean(val2);
      cost3_plot(count)=mean(val3);
    % 
  %
    fprintf(1, 'FEs = %d, Iterations = %d: mean(val) = %e \n\n', FEs, count, mean(val));
%    
    mv(count) = mean(val);
    cost_plot(count)=mv(count);
     for j=1:D
       x_plot(j,count)=mean(pop(j,:),2);         
     end
     crsp_plot(1,count)=mean(crspeeds(1,:),2);
     crsp_plot(2,count)=mean(crspeeds(2,:),2);
     
    itr(count)=count;
%    
    % Break, if fitness is good enough
     if mean(val) <= stopfitness
% 
        break;
    end
%    
end
toc;
save pop_init.mat pop
% load pop_init.mat
% 
% 
%
figure(D+1)             %  
plot(itr,cost_plot,'black')   % Figure 10.6a
%
%
% plotting diameters - optional
% for j=1:D
%     figure(j)
%     plot(itr,x_plot(j,:),'black')     % not used in the Chapter 10
% end
%
%
figure(D+2)             %  
plot(itr,crsp_plot(1,:),'black'),hold on   %   Figure 10.7     %  
plot(itr,crsp_plot(2,:),'black')
%
%
% 
% Convett the Y-axis scale Fig10_6 
figure(D+3)             %  
semilogy(itr,cost1_plot,'black'),hold on  % Figure 10.6b 
%                                        - 1st term in (RHS of) Equation 10.8   
%             %  
semilogy(itr,cost2_plot,'black'),hold on  % Figure 10.6b
%                                        - 2nd term in (RHS of) Equation 10.8
%             %  
semilogy(itr,cost3_plot,'black')  % Figure 10.6b
%                                       - 3rd term in (RHS of) Equation 10.8