%                   Table10_4_COMBEO.m
%
%    This program runs with COMBEO algorithm
%
%
%    For a run with COMBEO open "method=2" 
%                            or "method=4"
%                        
%        "method=2" means COMBEO with Equation (10.4)
%        "method=4" means COMBEO with Equation (10.5) (blending included) 
%
% NOTE:  (Since it is a stochastic search) try repeated MC runs > 1 
%           for a result 
%__________________________________________________________________________
close all;
clear all;
clc;
%
%--------------------------------------------------------------------------
%
method = 2;  %  COMBEO  - for F1,F2,F3,F5,F7,F9,F10,F13,F14,F15,F16,F18,F19,F20
% method = 4;  %  COMBEO -  uncomment this for F4, F6, F8, F11, F12, F17  
%--------------------------------------------------------------------------
%
%
nx = 40;                 % dimensionality of benchmark functions 1000
%
np=10;
% np=50;    %  uncomment for F6 only
% np=100;   %  uncomment  for  F11 only
%
ne = np;                 % population size
%
%
Max_FEs = 4*1e6*ne      % maximal number of FEs, should be set to 3e+06
%
stopfitness = 1e-5;      % stop if fitness < stopfitness (minimization)
%
global initial_flag;    % the global flag used in test suite
initial_flag = 0;       % should set the flag to 0 for each run, each function
%
%--------------------------------------------------------------------------
func_num = 19;  % Enter the objective function  ( to be changed from 1 to 20)
%
%--------------------------------------------------------------------------
%
%
% set the lower and upper bound for each function
if (func_num == 1 || func_num == 4 || func_num == 7 || func_num == 8 || func_num == 9 || func_num == 12 || func_num == 13 || func_num == 14 || func_num == 17 || func_num == 18 || func_num == 19 || func_num == 20)
    lb = -100;
    ub = 100;
end
if (func_num == 2 || func_num == 5 || func_num == 10 || func_num == 15)
    lb = -5;
    ub = 5;
end
if (func_num == 3 || func_num == 6 || func_num == 11 || func_num == 16)
    lb = -32;
    ub = 32;
end
%
%
pop = lb + (ub - lb)*rand(nx,ne);                % initial population
val = benchmark_func(pop',func_num);             % fitness evaluation
% 
%
%
FEs = ne;                                       % count the number of function evaluations
%
tic;
icount=0;
while (FEs <= Max_FEs-ne)
    icount=icount+1;
    % 
%
    if method == 2
        newpop = ENKF_opt(pop,ne,nx);    %  for result by COMBEO (Equation 10.4)
    elseif method == 4
        newpop = ENKF_blend_opt(pop,ne,nx);   %  for result by COMBEO with blending 
%                                                     (Equation 10.5)
    end
%    
    newval = benchmark_func(newpop', func_num);
    FEs = FEs + ne;
    index = (newval < val);
%    
    val(index) = newval(index);
    pop(:,index) = newpop(:,index);
%    
    min_val = min(val);
    mean_val = mean(val);
    count = FEs/ne;
%    
    % demo output, you should save your results to some files
    fprintf(1, 'FEs = %d, Iterations = %d: mean(val) = %f \n\n', FEs, count, mean_val);
%    
    mv(count) = mean(val);
%    
    % Break, if fitness is good enough
    if mean_val <= stopfitness
        break;
    end
%    
end
toc;
mean(newpop,2)';