%                       Example2_10_Fig2_13.m
%
%       MC simulation with partial averaging - Example 2.10
%
%    Ist run:     Standard simulation using uniform distribution
%    II nd run:   Simulation using partial averaging
%
%                   Figure 2.13a & b
%______________________________________________________________________
clc
clear all
close all
%
%
Nruns=10;       % no. of independent MC runs
pi=3.14159265;
n=50001         % no. of smaples in each MC run
%
%________________________________________________________________________
%                                   Ist run 
%        Standard simulation using uniform distribution
%              ten simulation runs with 1001 realizations from
%               uniform distribution%
%
for j=1:Nruns       % each simulation run
numb(j)=j;           % MC run counter

for i=1:n
  u1(i)=rand;
  u2(i)=rand;
end
%
sumgx=0.0;
for i=1:n
  a1=(u1(i)-0.5)*(u1(i)-0.5);
  b1=(u2(i)-0.5)*(u2(i)-0.5);
  gx(i)=exp(-0.5*(a1+b1));
  sumgx=sumgx+gx(i);
end
%
standardMCs(j)=sumgx/n    %  value of the integral in Example 2.10
                            %  for each MC run  by standard simulation
%   
sigmastandardMCs(j)=sqrt(var(gx))
%
end
%```                    end of 1st run -standard simulation
%
%_______________________________________________________________________
%%               IIrd Run -  by partial averaging
%
%               Here I = Idash + Irem
%
     idash = 0.4156    % known apriori (may be an analytical solution) 
                                    %    over a sector with radius =0.75
%
%     Irem = balance area under r(x) is computed using UD Rv
            %    Here the RV must always fall within this balance area
               %       Hence a need to check u1^2+u2^2 > 0.75^2 =0.5625
                     % where u1 & u2 are UD RVs   
% 
% c can be computed explicitly  (see Equations 2.106-107)
c=(pi/4)*0.75*0.75;     %  =0.4418  (area of the sector)
%
% partial avergaing over the remaining portion above the sector -
%                           Ten simulation runs
for i=1:n

  u1(i)=rand;
  u2(i)=rand;

end
%
idash=0.4156;
%
for j=1:Nruns

for i=1:n
  u1(i)=rand;
  u2(i)=rand;
end
%
% 1/(1-c) is the pdf (uniform) over remaining area outside the quarter sector
%
sumgx=0.0;
for i=1:n
  a1=(u1(i)-0.5)*(u1(i)-0.5);
  b1=(u2(i)-0.5)*(u2(i)-0.5);
  c1=u1(i)*u1(i)+u2(i)*u2(i);
  if(c1>0.5625)
     gx(i)=exp(-0.5*(a1+b1));
  else
     gx(i)=0.0;
  end
   sumgx=sumgx+gx(i);
end
%
irem=sumgx/n     
partialaveMCs(j)=idash+irem %  required value of the integral I=idash +irem
sumgx=0.0;
%
for i=1:n
  sumgx=sumgx+(1-c)*gx(i)*gx(i);
 end
%
sigsq=sumgx/n-irem*irem    % see Equation (2.110)
sigmapartialaveMCs(j)=sqrt(sigsq)
%
end
%                       end of partial averaging
%--------------------------------------------------------------------------
% plotting
figure(1)
plot(numb,standardMCs,'+black')       % Figure 2_13a  - standard simulation
hold
plot(numb,partialaveMCs,'*black')        %Figure2_13a  - partial averaging
ylim([0.91,0.93])
%
figure(2)
plot(numb,sigmastandardMCs,'+black')      %Figure2.13b - standard simulation
hold
plot(numb,sigmapartialaveMCs,'*black')     %Figure2.13b - partial averaging
%
%
%
