%                   Example2_12_Fig2_15.m
%
%               Important sampling - Example 2.12 
%
%                  Figure 2.15a  & 2.15b
% 
%         The given integral is obtained 
%       1) first by finding as expectation integral
%               with respect to (wrt)  f(x) - a normal pdf with mu=0 
%  i.e. assuming g(x(i)) =x(i)^4*exp(x(i)^2/4)*I(x(i)<=2))
%                    and sampling x(i) as a normal rv wrt f(x)
%                       I(.) is an indicator function
%
%       2) second with an important samling density h(x)
%           i.e.   by finding as expectation integral
%               with respect to h(x)- a normal pdf with mu=2
% i.e. assuming g(x(i)) =x(i)^4*exp(-x(i)^2/4)*exp(-mu*X(i)+mu*mu/2)*I(x(i)<=2)) 
%                   (see Equation 2.121)
%
%_________________________________________________________________
%
clc
clear all
close all
%
n=20001;           % samples of RV in each independent MC run 
pi=3.14159265;
%
%-----------------------------------------------------------------------
% Ist Run    - Standard MC simulation - f(x) with mu=0
cnst=sqrt(2*pi);
%
% 10 independent MC runs
Nruns=10;
for j=1:Nruns
%
for i=1:n
%   Normal variable by Box-Muller method
  u1(i)=rand;
  u2(i)=rand;
  R(i)=sqrt(-2*log(u1(i)));
  Th(i)=2*pi*u2(i);
end

% X & Y are normal   
for i=1:n
  X(i)=R(i)*cos(Th(i));  % zero mean normal RV (corresponding to f(x))
end
%
%            _____________________ 1st run starts ___________________

for i=1:n
  if(X(i)>=2)                   %beacause of the Indicator function
  a1=X(i)*X(i)*X(i)*X(i);
  b1=exp(X(i)*X(i)/4);
  gx(i)=a1*b1;
  else
      gx(i)=0;
  end
end
% 
standardMCs(j)=mean(gx)%
sigmastandard(j)=sqrt(var(gx))
%                               End of 1st run
%
%
%
%
%                   ____________2nd run starts____________
% IInd run - wrt an important samling density h(x) with mu=2
mu=2;
for i=1:n
 X(i)=X(i)+mu;    % normal RV with mean =2 (corresponding to h(x)) 
end
%
for i=1:n
  if(X(i)>=2)           %beacause of the Indicator function
  a1=X(i)*X(i)*X(i)*X(i);
  b1=exp(X(i)*X(i)/4);
  c1=exp(-mu*X(i)+mu*mu/2);
  gxfxbyhx(i)=a1*b1*c1;          %   gxfx/hx   Equation (2.121)
  else
      gxfxbyhx(i)=0;
  end
end
%
impsamp(j)=mean(gxfxbyhx)
sigmaimpsamp(j)=sqrt(var(gxfxbyhx))
%
numb(j)=j;
%                        END of 2nd run
%----------------------------------------------------------------------
end
%
%
% Plotting the figures
figure(1)                              %  Figure 2.15a
plot(numb,standardMCs,'+black','MarkerSize',10)         % by standard MC simulation
% 
hold
plot(numb,impsamp,'*black','MarkerSize',10)           % by important sampling
%

figure(2)                              %  Figure 2.15b
plot(numb,sigmastandard,'+black','MarkerSize',10)       % by standard MC simulation
% 
hold
plot(numb,sigmaimpsamp,'*black','MarkerSize',10)        % by important sampling  
% 
%----------------------------------------------------------------

