%              Table2_1.m
%
%    Reliability problem in Example 2.5
%
%
%    Sy  - yield strength (lognormal)
%    Sm  - section modulus  (lognormal)
%    Bm  - bending moment   (Type I asymptotic distribution)
% 
%    Sy and Sm are corelated with correlation coefficient =0.4
%
%  Thus, given correlated non-normal vector {Sy, Sm} 
%     this program first transforms this RV vector 
%        to uncorrelated standard Normal RVs
%            by Rosenblaut transformation
%
%   Using these uncorrelated information, the program uses
%           Equation (2.42) and (2.45) to get the simulated samples of
%                      Sy and Sm
%   The program uses Equation (2.46) to get samples of Bm
%   
%
%     The program outputs results on reliability index 
%          & probability of failure given in Table 2.1 
%----------------------------------------------------------------------------
clc;
clear all;
close all;
%
       pi=3.14159265;
       pi2=2*pi;  
%
%  Input data on the mean and standard deviations of Sy, Sm and Bm
      mY=40.0;    
      mU=50.0;
      mT=1000;
      sigY=5.0;
      sigU=2.5;
      sigT=200;  
%
% correlation coefficient between Sy and Sm is 0.4
  r=0.4;
%
% parameters (in the respective distribution) of the three random variables ( see Ang and Tang 1984)
% parametrs of the lognormal variable Sy 
      zetaY=0.125;
      lamdaY=3.681;
% parametrs of the lognormal variable Sm      
      zetaU=0.05;   
      lamdaU=3.911;
% parametrs of the lognormal variable Bm
      alpha=0.006413;
      u=910.02;     
% 
% Start of Hohenbichler and Rackwitz iteration (1981)
      itr=5;   % no. of iterations
      tol=0.0001;
%
% Initial x ={Sy, Sm, Bm} vector  
     x(1)=40;
     x(2)=50;
     x(3)=1000;       
%  Corresponding z-vector - standard normal (Equations 2.42, 2.45 and 2.46)   
     z(1)=0.063;
     z(2)=-0.005;
     z(3)=0.176;   
%     
% Corresponding beta - index    
'initial iteration = 1'
       beta=sqrt( z(1)*z(1) + z(2)*z(2) + z(3)*z(3) )
       fprintf(1, 'Sy = %f, Sm = %f, Bm = %f \n\n', x(1), x(2), x(3));    
%
%
      for iteration=2:itr     
% 
% to find the Jacobian matrix - 3 x 3 
         rjac=jacobi(x,z(3),zetaY,zetaU,r,alpha,u);
%
%
gx0=x(1)*x(2)-x(3);    % limit state function
%
Gx0=[x(2) x(1) -1];      % gradient vector of gx0
%
Gu0=rjac'*Gx0';          % gradient vector at z  
%
% obtain the new failure point in terms of z
B1=Gu0'*Gu0;
B2=1/B1;
B3=Gu0'*z';
B3-gx0;
znew=B2*(B3-gx0)*Gu0;
%
%
% obtain the new failure point in terms of original x
%         from substitution of z in Equations 2.42, 2.45 and 2.46
     A1=znew(1)*zetaY+lamdaY;
     xnew(1)=exp(A1);
     A2=znew(2)*zetaU*sqrt(1-r*r)+lamdaU;
     A3=A2+r*(zetaU/zetaY)*(A1-lamdaY);
     xnew(2)=exp(A3);
     A4=normcdf(znew(3),0,1);
     A5=-log(A4);
     A6=-log(A5);
     xnew(3)=A6/alpha+u;
% 
%
x=xnew;
z=znew';
%  beta at the end of the iteration  
iteration    % iteration number
beta=sqrt( z(1)*z(1) + z(2)*z(2) + z(3)*z(3) )
fprintf(1, 'Sy = %f, Sm = %f, Bm = %f \n\n', x(1), x(2), x(3));    
%        
      end
%
% compute the probability of failure      
      pf=normcdf(-beta)
%
fprintf(1, 'relaibility index = %f, prob of failure = %f \n\n', beta, pf);   



       



