%               Jacobi.m
%
%           Called by Fig2_3.m
%
%     Computes the Jacobian (see Equation 2.39)
%               (step 2 of Table 2.1)
%
%  Jacobian matrix [rjac] is obtained by differemntiating 
%     Equations (2.42), (2.45) and (2.46)with respect to 
%                 sy, sm and Bm
%
%   Input  is the vector x = {sy  sm  Bm]
%           and the expressions of Jacobian matrix 
%               in terms of sy, sm and Bm
%_________________________________________________
function [rjac]=jacobi(x,Z3,zetaY,zetaU,r,alpha,u)
%
%
    rjac=zeros(3,3);
%  
      A1=x(1)*zetaY;
      rjac(1,1)=1/A1;   
%      
      rjac(2,1)=-r/(A1*sqrt(1-r*r));
%      
      A2=x(2)*zetaU;
      rjac(2,2)=1/(A2*sqrt(1-r*r));       
%     
      A3=normpdf(Z3);
      A4=-alpha*(x(3)-u);
      rjac(3,3)=alpha*(1/A3)*exp(A4-exp(A4));
%
      for i=1:3
      for j=1:3
        xx(i,j)=rjac(i,j);
      end
      end
%
%
% Inversion of Jacobian matrix 
      a1=xx(2,2)*xx(3,3) - xx(3,2)*xx(2,3);
      a2=xx(2,1)*xx(3,3) - xx(3,1)*xx(2,3);
      a3=xx(2,1)*xx(3,2) - xx(3,1)*xx(2,2);
      d=xx(1,1)*a1-xx(1,2)*a2+xx(1,3)*a3;  % determinant
%      write(6,*)'a1,a2,a3,d =',a1,a2,a3,d
%
      b1=xx(1,2)*xx(3,3) - xx(3,2)*xx(1,3);
      b2=xx(1,1)*xx(3,3) - xx(3,1)*xx(1,3);
      b3=xx(1,1)*xx(3,2) - xx(3,1)*xx(1,2);
%
      c1=xx(1,2)*xx(2,3) - xx(2,2)*xx(1,3);
      c2=xx(1,1)*xx(2,3) - xx(2,1)*xx(1,3);
      c3=xx(1,1)*xx(2,2) - xx(2,1)*xx(1,2);
%
      xx(1,1)=a1/d;
      xx(1,2)=-b1/d;
      xx(1,3)=c1/d;
      xx(2,1)=-a2/d;
      xx(2,2)=b2/d;
      xx(2,3)=-c2/d;
      xx(3,1)=a3/d;
      xx(3,2)=-b3/d;
      xx(3,3)=c3/d;
%
%________________________________________
% To check the inversion
%        for i=1:3
%        for j=1:3
%         sum=0.0;
%        for k=1:3
%         sum = sum + rjac(i,k)*xx(k,j);
%        end
% end
% end
%________________________________________
%
% 
      for i=1:3
       for j=1:3
        rjac(i,j)=xx(i,j);   % Inverted matrix
       end
      end
%      
end


      