 %                      Example2_7_Fig2_11.m
 %
 %                           Example 2.7
 %
 % Generation of correlated non-normal RVs by Rosenblatt transformation
 %          (given the joint distribution)
 %
 %    MC Simulated results
 %  Figure 2.11a -   pdf of X
 %  Figure 2.11b -   CDF of X
 %  Figure 2.11c -   pdf of Y
 %  Figure 2.11d -   CDF of Y
 %
%------------------------------------------------------------------------

clc
close all
clear all
%
n=5000;
dx2=0.05;
%
% Initalization
xysum=0.0;
%
% 
for i=1:n
  u1=rand;
  x(i)=-log(u1);   % samples of the 1st exponential RV 
 %                         in Equation 2.94a(see Equation 2.85 also)
%
% To generate samples of the second exponential Y
% (see Equation 2.94b)
  u2=rand;
  %
  %range of Y
  x2s=0.0;      %   starting value
  x2f=10.0;     % end value
  %
  %
  % for a given x, we need to find y such that Equation 2.94b is satisfied
  % i.e., u2 = 1-(1+y)*exp(-y(1+x))  
  %  i.e. g(y)=1-u2-(1+y)*exp(-y*(1+x)) = 0 
  % To solve this nonlinear equation for y by bisection method
    fu1=1-u2-(1+x2s)*exp(-x2s*(1+x(i))); 
    fu2=1-u2-(1+x2f)*exp(-x2f*(1+x(i)));
    xsq=(x2f-x2s)*(x2f-x2s);  
    fsq=(fu2-fu1)*(fu2-fu1);
    dif=sqrt(xsq+fsq);
%
    while(dif>0.001)
      fu1;
      fu2;
      dif;
      x2s;
      x2f;
  %
        x2=x2s+0.5*(x2f-x2s);
        fu2=1-u2-(1+x2)*exp(-x2*(1+x(i)));
        fu1;
        if(fu1*fu2<0.0)
           x2f=x2;
           x2s;
  %    
        else
           x2s=x2;
           x2f;
        end
        fu1=1-u2-(1+x2s)*exp(-x2s*(1+x(i)));
        fu2=1-u2-(1+x2f)*exp(-x2f*(1+x(i)));
        xsq=(x2f-x2s)*(x2f-x2s);
        fsq=(fu2-fu1)*(fu2-fu1);
        dif=sqrt(xsq+fsq);
        continue
 %  
    end
%
     y(i)=x2f;
     xysum=xysum+x(i)*y(i);   % This gives the correlation between x and Y
%
  end
%----------------------------------------------------------------------
%
  mx=mean(x);
  my=mean(y);
  stdx=sqrt(var(x));
  stdy=sqrt(var(y));
%
  corn=(xysum/n-mx*my)/(stdx*stdy)  % This gives the correlation 
  %                                     coefficient from simulations 
m=length(x);
%----------------------------------------------------------------------
%
% to find pdf and CDF of the exponential x variable
y1=sort(x);
nint=100;     % no. of intervals for generating histogram
du=(y1(m)-y1(1))/nint;
sum=0;
for i=2:nint+1
    ny(i-1)=0;
    yend=y1(1)+(i-1)*du;
    yint(i-1)=yend-du/2;
    for j=1:m
        if((y1(j)>(yend-du)) && (y1(j)<=yend))
          ny(i-1)=ny(i-1)+1;
        end
    end
    pny(i-1)=ny(i-1)/m;
    sum=sum+pny(i-1);
    cny(i-1)=sum;     % CDF of x

    pny(i-1)=ny(i-1)/(m*du);    % pdf of x
end
%
%
%------------------------------------------------------------------------
% plotting the pdf and CDF of X
figure(1)                  % pdf of X -  Figure 2.11a
plot(yint,pny,'black')

figure(2)                   % CDF of X - Figure 2.11b
plot(yint,cny,'black')
%_______________________________________________________________________
%
% to find pdf and CDF of the exponential Y variable
  m=length(y)
  mn1=mean(y)
  std1=sqrt(var(y))

  y1=sort(y);
  nint=100;   % no. of intervals to generate the histogram

du=(y1(m)-y1(1))/nint;
sum=0;
for i=2:nint+1
    ny(i-1)=0;
    yend=y1(1)+(i-1)*du;
    yint(i-1)=yend-du/2;
    for j=1:m
        if((y1(j)>(yend-du)) && (y1(j)<=yend))
          ny(i-1)=ny(i-1)+1;
        end
    end
    pny(i-1)=ny(i-1)/m;
    sum=sum+pny(i-1);
    cny(i-1)=sum;     % CDF of Y 

    pny(i-1)=ny(i-1)/(m*du);    % pdf of Y
end
%
%plotting the pdf and CDF of Y
figure(3)              % pdf of Y -   Figure 2.11c
plot(yint,pny,'black')
%
figure(4)              % CDF of Y -   Figure 2.11d
plot(yint,cny,'black')
%----------------------------------------------------------------------

