%                   Example2_8_Fig2_12.m
%
%                    Ising model in  Example 2.8
%
%       A 2 dimensional Ising model [Ising 1925, Tobochnik and Gould
%             2008] for identifying the thermo-dynamical 
%                   phase transitions in a ferromagnet 
%                       using MC simulation.
%
%             The code plots Figure 2.12 a-f
%
%    NOTE:
%    Set N=5  for Figures 2.12a and Figure 2.12b   
%    Set N=20 for Figures 2.12c and Figure 2.12d   
%    Set N=50 for Figures 2.12e and Figure 2.12f   
%--------------------------------------------------------------
clc
close all
clear all
%--------------------------------------------------------------
%  for Figures 2.12a and Figure 2.12b  
N=5    % lattice size   
%
%--------------------------------------------------------------
%  UNCOMMENT for Figures 2.12c and Figure 2.12d 
% N=20    % lattice size
%
%--------------------------------------------------------------
%  UNCOMMENT for Figures 2.12e and Figure 2.12f
% N=50    % lattice size
%
%--------------------------------------------------------------
J = 1; % +1 for Ferromagnetic and change sign for antiferromagnetic coupling
%
%
% start temparature count
for ii=1:100    %   Number of temparatures - 100
    %
                    ii    %  Temparature count
kT = rand()*5+1e-10;
temp(ii)=kT;      % start temparature randomly
%
  kT              % temparature value
%
%
% Generate a random initial configuration. 
p=.5; %  +1 spins in the initial configuration on an average
%
grid = sign(p-rand(N)); % Choose initial configuration randomly
%
% 
%   
%
% Run the Metropolis algorithm 
Nt =100*N^2;%  Nt update steps at each temparature 
%
%--------------------------------------------------------------------
M = sum(sum(grid))/numel(grid);   % average magnetization  
E = isingenergy(grid,J);            %  average energy 
figure(1);
isingplot(grid,N,J,kT,M,E);     % 2D plot of the grid 
%
% precompute the indicies adjacent to each spin index
s=1:N^2;
s = s-1; % index by zero
adj = zeros(length(s),4);
%
% 
r = floor(s/N);
c = rem(s,N);
%
% see Figure B.2  (in Appendix B)
adj(:,1) = mod(r+1,N)*N+c;  %down 
adj(:,2) = mod(r-1,N)*N+c ; %up
adj(:,3) = r*N+mod(c+1,N);  %right
adj(:,4) = r*N+mod(c-1,N);  %left
%
adj = adj+1; % index by one again
%
%
%
% Pick a sequence of random spins (with a linear index)
spin = randi(N^2,Nt,1);    % selection probability of a site (location)
                                                            % in the grid)
%
for i=1:Nt,
%     i
% location of ith spin
    s = spin(i);
%
    % Calculate the change in energy of flipping s
    dE = 2*J*grid(s)*sum(grid(adj(s,:)));
    % Calculate the transition probability
    p = exp(-dE/kT);
    % Decide if a transition will occur
    if rand <= p, grid(s) = -1*grid(s); end
    % Refresh display of current spin configuration every N^2 trials
    if mod(i,N^2)==0,
        % Sum up our variables of interest and plot
        M = sum(sum(grid))/numel(grid);    % average magnetization 
        E = isingenergy(grid,J);           % average energy 
        isingplot(grid,N,J,kT,M,E);
%
    end
%  
end
%
% Compute final magnetization density and energy density
Mag(ii) = sum(sum(grid))/numel(grid);
Energ(ii) = isingenergy(grid,J);
%
%
end   % loop of kT (ii=1:100)
%
% 
% 
figure(1)
plot(temp,Mag,'oblack')
figure(2)
plot(temp,Energ,'oblack')

