%                           Example2_9_Table2_2.m 
%
%               Reliability of the system for which the input
%                    data is given in Example 2.5
%      
%                 Computation by direct MC simulation
%               of the multiple integral in Equation (2.100)
%                  to get the probability of failure
%
%
%    Sy  - yield strength (lognormal)
%    Sm  - section modulus  (lognormal)
%    Bm  - bending moment   (Type I asymptotic distribution)
% 
%    Sy and Sm are corelated with correlation coefficient =0.4
%
%   This program generates the samples of the random variables
%               Sy, Sm and Bm from Equations 2.42, 2.45 and 2.46  %   
%----------------------------------------------------------------------------
clc;
clear all;
close all;
%
       pi=3.14159265;
       pi2=2*pi;
%
% input data
%  Given data on the mean and standard deviations of Sy, Sm and Bm
      mY=40.0;    
      mU=50.0;
      mT=1000;
      sigY=5.0;
      sigU=2.5;
      sigT=200;  
%
% Specified correlation coefficient between Sy and Sm is 0.4
  r=0.4;     
%
% parameters of the respective distributions of the three random variables ( see Ang and Tang 1984)
% parametrs of the lognormal variable Sy 
      zetaY=0.125;
      lamdaY=3.681;
% parametrs of the lognormal variable Sm      
      zetaU=0.05;   
      lamdaU=3.911;
% parametrs of the Type I asymptotic distribution of Bm
      alpha=0.006413;
      u=910.02;     
%   
%_____________________________________________________________________ 
Nsim=10;       %  no. of independent MC runs
n=500000;   %no. of simulated samples of the random variables
%
%Start of MC simulation
for j=1:Nsim
% 
j   %simulation counter
for i=1:n
%
  Rn1(i)=randn;    % normal randomm variable
  Rn2(i)=randn;    % normal randomm variable
end
%
  for i=1:n
     A1=Rn1(i)*zetaY+lamdaY;
     X(i)=exp(A1);     % lognormal random variable Sy
     A2=Rn2(i)*zetaU*sqrt(1-r*r)+lamdaU;
     A3=A2+r*(zetaU/zetaY)*(A1-lamdaY);
     Y(i)=exp(A3);     % lognormal random variable Sm
  end
%_________________________________________
% Check for mean and variance of Sm and Sy 
%
  mx=mean(X);
  my=mean(Y);
  sigx=sqrt(var(X));
  sigy=sqrt(var(Y));
%
sum=0;
  for i=1:n
    sum=sum+(X(i)-my)*(Y(i)-mU);
  end
%   correln=sum/n - mx*my
covariance=sum/n;
rr=covariance/(sigx*sigy);     % simulated value of the correlation coefficient
                                      % between Sy and Sm
rsim=rr;
%_________________________________________________________________________
% Bending moment (Bm) random variable (Type I asymptotic distribution)
alpha=0.006413;
u=910.02;
z=randn(n,1);
% 
for i=1:n
    zz=z(i);
%  
 A4=normcdf(zz,0,1);
     A5=-log(A4);
     A6=-log(A5);
     Z(i)=A6/alpha+u;   % random variable Bm (Type I asymptotic distribution)
end
%
%_____________________________________________________________________
%Check for mean and variance of Bm
mM=mean(Z);
sigM=std(Z);
%______________________________________________________________________
%  Simulation of the integral in Equation (2.100)  
% Limit state function Sy*Sm - Bm = X(i)*Y(i)-Z(i)
sumi=0;
for i=1:n
    pfn=X(i)*Y(i)-Z(i);
   if pfn<0
        ind(i)=1;
        sumi=sumi+1;
    else
        ind(i)=0;
    end 
end
pfi=sumi/n;
%
%
     fprintf(1, 'prob of failure = %f, rsim = %f, \n\n', pfi,rr);   
%
end
%


