%                       Fig2_10.m
%
%
   %                      MC simulation
   %                   BOX-Muller method
   %        Figure 2.10a - uncorrelated normal bivariate
   %        Figure 2.10b - positively correlated normal bivariate
   %        Figure 2.10c - negatively correlated normal bivariate
%__________________________________________________________________________
clc
close all
clear all
%
%
n=10000;
pi=3.14159265;
%
%           Box Muller method
for i=1:n
%
  u1(i)=rand;
  u2(i)=rand;
%
 % Radius vector - Rayleigh distribution by inversion method
  R(i)=sqrt(-2*log(u1(i)));
%
 % Theta - Uniformly distributed over (0,2PI) by inversion method
  Th(i)=2*pi*u2(i);
%
end
%
% X & Y are the normal rvs
for i=1:n
  X(i)=R(i)*cos(Th(i));
  Y(i)=R(i)*sin(Th(i));
end
%
%----------------------------------------
%expectation & sigma of X & Y
  mx=mean(X);
  my=mean(Y);
  sigx=sqrt(var(X));
  sigy=sqrt(var(Y));
  %
  sum=0.0;
  for i=1:n
    sum=sum+X(i)*Y(i);
  end
  correln=sum/n - mx*my      % computing the actual correlation
%
%
% Plotting uncorrelated normal RVs
figure(1)                           %    Figure 2.10a
plot(X,Y,'.black','LineStyle','none')
%
%-----------------------------------------------------------------------
%
%
%  generating +vely correlated RVs   with rho=0.9
for i=1:n
  X1(i)=X(i);
  Y1(i)=0.9*X(i)+ Y(i)*sqrt(1-0.81);
 end
%
  mx=mean(X1);
  my=mean(Y1);
  sigx=sqrt(var(X1));
  sigy=sqrt(var(Y1));
%  
  sum=0.0;
  for i=1:n
    sum=sum+X1(i)*Y1(i);
  end
  corpositive=sum/n - mx*my    % computing the actual correlation
%
%Plotting +vely correlated normal RVs
figure(2)                             % Figure 2.10b
plot(X1,Y1,'.black','LineStyle','none')
%
%----------------------------------------------------------------------
%
%
% generating -vely correlated RVs
 for i=1:n
  X2(i)=X(i);
  Y2(i)=-0.9*X(i)+ Y(i)*sqrt(1-0.81);
 end
%
  mx=mean(X2);
  my=mean(Y2);
  sigx=sqrt(var(X2));
  sigy=sqrt(var(Y2));

  sum=0.0;
  for i=1:n
    sum=sum+X2(i)*Y2(i);
  end
  cornegative=sum/n - mx*my  % computing the actual correlation
%
%Plotting -vely correlated normal RVs
figure(3)                                 %  Figure 2.10c
plot(X2,Y2,'.black','LineStyle','none')
%

%-------------------------------------------------------------------------


