%                       Fig2_6.m
%
%                   MC simulation of normal random variables
%   
%            generation of normal RVS X & Y by Box-Muller method
%
%                        pdf and CDF of X and Y
%
%                       Figure 2.6 (a,b,c and d)
%----------------------------------------------------------------
clc
clear all
close all
%
n=10000;
pi=3.14159265;
%
for i=1:n
%
%  two uniformly distributed random variables     
  u1(i)=rand;     
  u2(i)=rand;
%
 % Radius vector - Rayleigh distribution by inverse method 
 %          
  R(i)=sqrt(-2*log(u1(i)));
%
 % Theta - Uniformly distributed over (0,2PI) by inverse method
 %           
  Th(i)=2*pi*u2(i);
%
end
%   
%
% X & Y are the geneerated (cartesian) RVs from R and Th (Equation 2.86)
for i=1:n
  X(i)=R(i)*cos(Th(i));
  Y(i)=R(i)*sin(Th(i));
end
%__________________________________________________________________________
% simulated pdfs & CDFs of X & Y
%
%
% First histogram generation on X
m=length(X);
y1=sort(X);
nint=100;
du=(y1(m)-y1(1))/nint;
sum=0;
for i=2:nint+1
    ny(i-1)=0;
    yend=y1(1)+(i-1)*du;
    yint(i-1)=yend-du/2;
    for j=1:m
        if((y1(j)>(yend-du)) && (y1(j)<=yend))
          ny(i-1)=ny(i-1)+1;
        end
    end
    pnydx(i-1)=ny(i-1)/m;
    sum=sum+pnydx(i-1);
    cny(i-1)=sum;           % CDF of X normal
    pny(i-1)=ny(i-1)/(m*du);   %  pdf of X normal
end
%
figure(1)
plot(yint,pny,'black')    % pdf of X (Figure 2_6a)
%
%
figure(2)
plot(yint,cny,'black')    % cdf of X normal (Figure 2_6b)
%
%______________________________________________________________________________
%
% Second histogram generation on Y normal
y2=sort(Y);
nint=100;
du=(y2(m)-y2(1))/nint;
sum=0;
for i=2:nint+1
    ny(i-1)=0;
    yend=y2(1)+(i-1)*du;
    yint(i-1)=yend-du/2;
    for j=1:m
        if((y2(j)>(yend-du)) && (y2(j)<=yend))
          ny(i-1)=ny(i-1)+1;
        end
    end
    pnydx(i-1)=ny(i-1)/m;
    sum=sum+pnydx(i-1);
    cny(i-1)=sum;                       % CDF of Y normal
    pny(i-1)=ny(i-1)/(m*du);          % pdf of Y normal
end
%
figure(3)
plot(yint,pny,'black')    % pdf of Y (Figure 2_6c)
%
%
figure(4)
plot(yint,cny,'black')    % CDF of Y (Figure 2_6d)
%
%
%---------------------------------------------------------------------------
