%                              Appendix_D_FigD1.m
%
%           Non-stationary mean square response of an SDOF oscillator
%                   under white noise input
%
%                           Figure D.1 (in Appendix D)
%__________________________________________________________________________
clc
close all
clear all
%
%parameters of the SDOF oscillator
w=10;       % natural frequency in rad/sec.
eta=0.02;       % damping ratio
wd=w*sqrt(1-eta*eta);    % damped frequency  in rad/sec.
%
I=1.0;          % strength of the white noise input
%
%
%start of computation
T=20.0;     % total time 
dt=0.01;   % time step
t=0:dt:T;      %time variable
Nt=numel(t);     % total time steps
%
cst=I/(4*eta*w^3);
A1=eta*eta/(1-eta*eta);
A2=eta/sqrt(1-eta*eta);
A3=1/(1-eta*eta);
%
for i=1:Nt
    sd=sin(2*wd*t(i));
    cd=cos(2*wd*t(i));    
    Ed=exp(-2*eta*w*t(i));    
    Esq(i)=cst*(1+Ed*(A1*cd-A2*sd-A3));    % Equation (D.18) of Appendix D
end

figure(1)   % Figure D.1
plot(t,Esq,'black')