%                       Appendix_D_FigD2.m
%
%           Stationary correlation of an SDOF oscillator
%                   under white noise input
%
%                           Figure D.2 (in Appendix D)
%__________________________________________________________________________
clc
close all
clear all
%
%parameters of the SDOF oscillator
w=10;       % natural frequency in rad/sec.
eta=0.02;       % damping ratio
wd=w*sqrt(1-eta*eta);    % damped frequency  in rad/sec.
%
I=1.0;          % strength of the white noise input
%
%
%start of computation
T=6.0;     % total time 
dt=0.01;   % time step
t=-T:dt:T;      %time variable
Nt=numel(t);     % total time steps
%
cst=I/(4*eta*w^3);
A1=eta/sqrt(1-eta*eta);
%
for i=1:Nt
    sd=sin(wd*abs(t(i)))
    cd=cos(wd*t(i));    
    Ed=exp(-eta*w*abs(t(i)))  ;  
    cd+A1*sd;
    Esq(i)=cst*Ed*(cd+A1*sd);    % Equation (D.18) of Appendix D
end

figure(1)
plot(t,Esq,'black')  % Figure D.2
ylim([-0.02,0.02])
