%                              Appendix_D_FigD_3.m  
%
%    
%
%    Figure D.3 contains 1)  solution for the unconditioned X(t)
%                        2)  solution for the conditioned X(t)
%     
%   Note: Run the program three times to get the required result 
%                        in Figure D.3
%__________________________________________________________________________
% Another Note: comment the following three lines after the 1st run is over
%           so that the paths are plotted in the same Figure
clc
clear all
close all
%
%--------------------------------------------------------------------------
% parameters of Brownian motion
sigma=1.0;  % noise intensity
%
%
xzero=0.1;    % initial condition 
%
%
%__________________________________________________________________________
%           
%
T=10;   % total time of integration
dt=0.001;   % time step
t=0:dt:T;   % time variable t
%
% at starting time step - simulation of Brownian motion 
dB(1)=sqrt(dt)*randn;   % Brownian increment 
% B(1)=dB(1);
%
%
% at first time step  
xold(1)=xzero + sigma*dB(1);  % solution to the orginal SDE dX=dB
% 
% yNew(1)=xzero+sigma*dB(1)+cot(xzero)*dt;  % solution to the new SDE (Equation D.42)
    e1=exp(-T);
    e3=exp(-9*T);
    c1=cos(xzero);
    c3=cos(3*xzero);
    s1=sin(xzero);
    s3=sin(3*xzero);
    yNew(1)=xzero+sigma*dB(1)+(e1*c1+e3*c3)/(e1*s1+(1/3)*e3*s3)*dt;
                                  % solution to the new SDE (Equation D.42)
%
%
% solution at all other time steps
for k=2:numel(t)
    k  % time step 
  dB(k) = sqrt(dt)*randn;
%   
% 
    xold(k)=xold(k-1) + sigma*dB(k);  %  % solution to the orginal SDE dX=dB
%
%
% 
%   yNew(k)=yNew(k-1)+sigma*dB(k)+cot(yNew(k-1))*dt;
    e1=exp(-(T-t(k)));
    e3=exp(-9*(T-t(k)));
    c1=cos(yNew(k-1));
    c3=cos(3*yNew(k-1));
    s1=sin(yNew(k-1));
    s3=sin(3*yNew(k-1));
    yNew(k)=yNew(k-1)+sigma*dB(k)+(e1*c1+e3*c3)/(e1*s1+(1/3)*e3*s3)*dt;
%
end
% plotting 
figure(1)
plot(t,xold,'black'),hold on   % plotting X(t)
% 
figure(2)
plot(t,yNew,'black'),hold on     % plotting the conditioned X(t)
%_____________________________________________________________________________
%
%
