%                        Example4_18_FIG4_4and5.m  (Example 4.18)
%
%  Program for response moments of an MDOF system by Ito's formula
%                        (Example 4.18)
%
%            The MDOF system is shown in Figure 4.3 .
%    It is a 2 dof system with X1(t) and X2(t) representing the two
%    displacement dof contained in the MDOF system. The velocity dof
%    are given by Y1(t) and Y2(t).
%
%                   vector X(t)={X1(t),Y1(t),X2(t),Y2(t)}^T
%
%  The response moments include the autocorrelations and cross-correlations
%    of the displacement abd velocity dof. This program outputs the
%    time histories of the moments:
%       E[X1^2], E[Y1^2], E[X1 Y1], E[X2^2], E[Y2^2],   
%       E[X2 Y2],  E[X1 X2], E[Y1 Y2], E[X1 Y2] & E[X2 Y1]
%
%
%              This program calls RK program namely rkFig4_4and5.m
%______________________________________________________________________
%      
     clc
     clear all
     close all
%
      T=5;   % total time of integration in sec.
      dt=0.001; % time step in sec.  
      t=0:dt:T;   %  time variable
      Nt =numel(t);     % no. of time steps    
%    	
	D=10;    % no. of response moments
%
   y0=zeros(D,1);    
   X=zeros(D,Nt);    % initialization
% 
   X(:,1)=y0;          % zero initial condition 
%    
% Time integration by RK method starts
for ijk=1:Nt-1
%     t(ijk)
  dxdt=rk4Fig4_4and5(t(ijk),X(:,ijk),dt);
  X(:,ijk+1)=dxdt;
end
%
 figure(1)   % Plotted results  in Fgure 4.4
 plot(t,X(1,:),'black'); hold on     % E[X1^2]
 plot(t,X(3,:),'black'); hold on     % E[X1Y1]
 plot(t,X(4,:),'black'); hold on     % E[X2^2]
 plot(t,X(6,:),'black'); hold on     % E[X2Y2]
 plot(t,X(7,:),'black'); hold on     % E[X1X2]
  
figure(2)       % Plotted results in  Fgure 4.5
 plot(t,X(2,:),'black'); hold on    % E[Y1^2]
 plot(t,X(5,:),'black'); hold on    % E[Y2^2]
 plot(t,X(8,:),'black'); hold on    % E[Y1Y2]
 %plot(t,X(9,:),'black'); hold on   % E[X1Y2]
 %plot(t,X(10,:),'black'); hold on   % E[X2Y1]

