%                                Example4_20_Fig4_6b.m
%
%
%      Example 4.20: Non-stationary gust response of an SDOF oscillator
%                     representing an aeroplane
%
%                     Mean square acceleration response history    
%
%                              Figure 4.6(b)
%
%__________________________________________________________________________    
%
clc
clear all
close all
%
T=5; dt=0.05; N=101;    %   total time, time step and No. of time steps
M=101;                  % frequency step in rad/sec.
%
dlamda=0.1;           % frequency step in rad/sec.
lamdamax=10;          % last frequency in rad/sec.
M=101;    % no. of freqency steps;  lamda(1)=0 and lamda(M)=lamdamax
%
%Initalization
lamda=zeros(1,M);  % frequency 
SXX=zeros(1,M);    % response PSD of acceleration
RXX=zeros(1,N);    % autocorrelation of acceleration
tim=zeros(1,N);
%
pi=3.14159265;
%
% Input Data (Equation 4.182)
L=500;    
U=200;    
sig=1.0;    % standard deviation of gust   
gama=1.11;   
alpha=0.0;    % Equation 4.181  -  (only beta is non-zero)
ratio=L/U;
c=1.0;      % Equation (4.181)
%
cnst=sig*sig*L/(2*pi*U);
%
% To find RXX of acceleration at 5 different time instatnts
for j=1:5          % response curves for different beta in Equation (4.181)
    if(j==1)beta=0.5;
    elseif(j==2)beta=1.0;
    elseif(j==3)beta=2.5;
    elseif(j==4)beta=5.0;
    elseif(j==5)beta=10.0;
    end
%
figure(1)
%
for k=1:N
  t=(k-1)*dt;
  sum=0.0;
  tim(k)=t;
%
 % For each time instant 't', find RXX from Equation(4.178a) 
 % This involves integration over lamda 
 % Lamda varies from lamda(1)=0 zero to lamda(M)=lamdamax
 % Trapezoidal rule for integration
    for i=2:M-1      
        lamda(i)=(i-1)*dlamda;
%
% SGG(lamda) - input (Dryden) spectrum (PSD)  --  Equation (4.182)
        a1=1+(3*lamda(i)*ratio)*(3*lamda(i)*ratio);
        a2=1+(lamda(i)*ratio)*(lamda(i)*ratio);
        SGG=cnst*a1/a2;
%
% h(t,lamda)h*(t,lamda)
%        b1=alpha-beta;
        b2=beta-gama;
        b1=alpha-gama;
        c1=complex(b1,-lamda(i));
        c2=complex(b2,-lamda(i));
        c3=complex(gama,lamda(i));
%
        e3=exp(-c3*t);
        e1=exp(-alpha*t);
        e2=exp(-beta*t);
%
        H=-gama*gama*(e1/c1 + e2/c2) +gama*gama*e3*(1/c1+1/c2);   %Equation 4.188
       H=H-gama*(e1-e2);
        Hc=conj(H);
%
        SXX(i)=H*Hc*SGG;   % h(t,lamda)*h*(t,lamda)*SGG   Equation 4.178b
        sum=sum+SXX(i)*dlamda;   % integration in Equations 4.178a
    end
%
   % SGG(at first lamda_value=0)  i.e.,   lamda(1)=0;
        a1=1+(3*lamda(1)*ratio)*(3*lamda(1)*ratio);
        a2=1+(lamda(1)*ratio)*(lamda(1)*ratio);
        SGG=cnst*a1/a2;
%
        c1=complex(b1,-lamda(1));
        c2=complex(b2,-lamda(1));
        c3=complex(gama,lamda(1));
%
        e3=exp(-c3*t);
        e1=exp(-alpha*t);
        e2=exp(-beta*t);
%
        H=-gama*gama*(e1/c1 + e2/c2) +gama*gama*e3*(1/c1+1/c2);
       H=H-gama*(e1-e2);
        Hc=conj(H);
%
        SXX(1)=H*Hc*SGG;
%
        sum=sum+0.5*SXX(1)*dlamda;  %   at first lamda_value=0
%   
%
% SGG(last lamda_value = lamdamax)
        lamda(M)=lamdamax;
        a1=1+(3*lamda(M)*ratio)*(3*lamda(M)*ratio);
        a2=1+(lamda(M)*ratio)*(lamda(M)*ratio);
        SGG=cnst*a1/a2;
%
        c1=complex(b1,-lamda(M));
        c2=complex(b2,-lamda(M));
        c3=complex(gama,lamda(M));
%
        e3=exp(-c3*t);
        e1=exp(-alpha*t);
        e2=exp(-beta*t);
%
        H=-gama*gama*(e1/c1 + e2/c2) +gama*gama*e3*(1/c1+1/c2);
       H=H-gama*(e1-e2);
        Hc=conj(H);
%
        SXX(M)=H*Hc*SGG;        % at last lamda_value= lamdamax
%
        sum=sum+0.5*SXX(M)*dlamda;  
%
       RXX(k)=sum;    % mean square acceleration 
    end

    plot(tim,RXX,'--black')   % Figure 4_6b
    hold on
end
