%                       Example4_21_Fig4_8.m
%
%       Example 4.21:  Non-stationary response of an SDOF oscillator
%                 under support excitation
%               
%               Mean square response history by
%                  Spectral representation approach
%
%
%                       Figure 4.8               
%-------------------------------------------------------------------------
T=30; dt=0.05;  N=601;   % total time, time step and No. of time steps
%
%
dlamda=0.1;     % frequency step in rad/sec.
lamdamax=50;      % last frequency in rad/sec.  
M=501;  % no. of freqency steps;  lamda(1)=0 and lamda(M)=lamdamax
%
% Initalization
lamda=zeros(1,M);   % frequency
SZZ=zeros(1,M);       % response PSD  of displacement  
RZZ=zeros(1,N);     % autocorrelation of displacement
tim=zeros(1,N);
%
pi=3.14159265;
%
%parameters of the SDOF oscillator
eta=0.02;       % damping
w=10.0;     % frequency  in rad/sec.
wd=w*sqrt(1-eta*eta);   % damped frequency
%
%
% parameters of the filter - Equation 4.191
etag=0.6;
wg=5*pi;
%
sig=1.0;   % S0=1 in Equation (4.190)  
%
%ground acceleration - modulating function - Equation (4.181)
      alpha=0.13;      
      beta=0.45;
      c=1;
%
%
figure(1)
%  start of integration
for k=1:N
  t=(k-1)*dt;  % time step
  k, t  % counter
  sum=0.0;
  tim(k)=t;
%
 % For each time instant 't', find RXX from Equation (4.178a) 
 % This involves integration over frequency, lamda 
 % Lamda varies from lamda(1)=0 zero to lamda(M)=lamdamax
 % Trapezoidal rule for integration
    for i=2:M-1     
        lamda(i)=(i-1)*dlamda;  % frequency 
%
% SGG(lamda)     % input (Kanai-Tajimi spectrum ) spectrum (PSD)  
                        % Equation (4.190) with S0=1.0
        ratio=lamda(i)/wg;
        a1=1+4*etag*etag*ratio*ratio;
        a2=(1-ratio*ratio)*(1-ratio*ratio)+4*etag*etag*ratio*ratio;
        SGG=sig*a1/a2;
%
% h(t,lamda)h*(t,lamda)
        a1=complex(alpha-eta*w,-lamda(i));
        a2=complex(beta-eta*w,-lamda(i));
%
        denom1=wd*wd+a1*a1;
        denom2=wd*wd+a2*a2;
%
        b1=exp(a1*t);
        b2=exp(a2*t);
%
        c1=cos(wd*t);
        s1=sin(wd*t);
%
        I1=(wd*(1-b1*c1)+a1*b1*s1)/denom1;
        I2=(wd*(1-b2*c1)+a2*b2*s1)/denom2;
%
        e1=exp(-alpha*t);
        e2=exp(-beta*t);
%
        H=(e1*I1-e2*I2)/wd;     % h(t,lamda)in Equation (4.194)
        Hc=conj(H);             % h*(t,lamda)
%
        SZZ(i)=H*Hc*SGG;    %h(t,lamda) h*(t,lamda) %SGG  - Equation 4.178b

% multiply SZZ(i) by 2 to account for the negative frequencies (-Lamda(i))
        SZZ(i)=2*SZZ(i);         % PSD of displacement  
        sum=sum+SZZ(i)*dlamda;      % mean square response of the displacement-
                                         % integration in Equations 4.178a
    end
%
%
% SGG(first lamda_value=0)
        lamda(1)=0;
        ratio=lamda(1)/wg;
        a1=1+4*etag*etag*ratio*ratio;
        a2=(1-ratio*ratio)*(1-ratio*ratio)+4*etag*etag*ratio*ratio;
        SGG=sig*a1/a2;
%
        a1=complex(alpha-eta*w,-lamda(1));
        a2=complex(beta-eta*w,-lamda(1));
%
        denom1=wd*wd+a1*a1;
        denom2=wd*wd+a2*a2;
%
        b1=exp(a1*t);
        b2=exp(a2*t);
%
        c1=cos(wd*t);
        s1=sin(wd*t);
%
        I1=(wd*(1-b1*c1)+a1*b1*s1)/denom1;
        I2=(wd*(1-b2*c1)+a2*b2*s1)/denom2;
%
        e1=exp(-alpha*t);
        e2=exp(-beta*t);
%
        H=(e1*I1-e2*I2)/wd;
        Hc=conj(H);
%
        SZZ(1)=H*Hc*SGG;
        sum=sum+0.5*SZZ(1)*dlamda;     % first lamda_value=0
%
        lamda(M)=lamdamax;
%
%  SGG(last lamda value, lamda(M)= lamdamax)
        ratio=lamda(M)/wg;
        a1=1+4*etag*etag*ratio*ratio;
        a2=(1-ratio*ratio)*(1-ratio*ratio)+4*etag*etag*ratio*ratio;
        SGG=sig*a1/a2;
%
%
        a1=complex(alpha-eta*w,-lamda(M));
        a2=complex(beta-eta*w,-lamda(M));
%
        denom1=wd*wd+a1*a1;
        denom2=wd*wd+a2*a2;
%
        b1=exp(a1*t);
        b2=exp(a2*t);
%
        c1=cos(wd*t);
        s1=sin(wd*t);
%
        I1=(wd*(1-b1*c1)+a1*b1*s1)/denom1;
        I2=(wd*(1-b2*c1)+a2*b2*s1)/denom2;
%
        e1=exp(-alpha*t);
        e2=exp(-beta*t);
%
        H=(e1*I1-e2*I2)/wd;
        Hc=conj(H);
%
        SZZ(M)=H*Hc*SGG;
% multiply SZZ(M) by 2 to account for the negative frequency -Lamda(M)
        SZZ(M)=2*SZZ(M);
%
        sum=sum+0.5*SZZ(M)*dlamda;  % at last lamda value
%
       RZZ(k)=sum;
    end
%
    plot(tim,RZZ,'black')  % Figure4.8

