%                           Example4_29_Figure 4.10
%
%
%        Example 4.29:   Solution of PDE (4.295) 
%
%                         a) Closed form Solution         (Figure 4.10a)
%                         b) Solution  by MC simulation   (Figure 4.10b)
%                        
%
%
%__________________________________________________________________________
clc
close all
clear all
%
% 
%
dt=0.01;                 % time step for integration of SDE
T=10;                    % Total time of integration
t=0:dt:T;                % range of time varible
%
%
mu=0.75;               % parameter 'a' in the PDE (4.295)
sig=0.3;               % parameter 'sigma' in the PDE (4.295)
h=0.5;                 % parameter 'c' in the PDE (4.295)
%
%
%___________________________________________________________________________
%           Closed form solution to PDE Equation (4.300)
%
figure (1)
%
V=zeros(20,20);
x=zeros(20);              % x - variable
tm=zeros(20);             % t - variable
%
%
for k=1:20       % loop on x- variable  %range of x is chosen to be 0-1.0
%
x(k)=0.05+(k-1)*0.05;     % x variable
%
u=zeros(20);             % initialization of u(x,t)
%
for i = 1:20          % loop on t - variable   %range of tm (time) 
                            % is chosen to be 0-T  =10.0
%
tm(i) = 0.5 + (i-1)*0.5;             % time variable
cnst=exp((2*mu+sig*sig-h)*(T-tm(i)));    
u(i)=x(k)*x(k)*cnst;                    % Equation (4.300)
V(k,i)=u(i);

end                      % end of loop on t- variable

plot(tm,u,'black');    % Figure 4.10a
hold on
end                      % end of loop on x- variable
% pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%          Numerical Solution to PDE by MC simulation (Equation 4.298)
%
N=10000;                % Number of Monte Carlo simulations
%
figure(2)
for k=1:20                      % loop on x- variable
k                         % each trajectory u(x,t) for a fixed x variable
x(k)=0.05+(k-1)*0.05;
%
u=zeros(20);                    % required solution to PDE
%
for i = 1:20                    % loop on t-variable
%
tm(i) = 0.5 + (i-1)*0.5;
%
m=tm(i)/dt;
cnst=exp(-h*(T-tm(i))) ;     % multiplication factor  (See Equation 4.298)
%
sumzz=0;    % initialization for finding Expectation  (see Equation 4.298)
%
for j=1:N                  % loop on Monte Carlo simulations
y=zeros(1,1001);             
%
y(m)=x(k);
%
for ii=m+1:numel(t)       % time loop on finding numerical solution to SDE
%
tim=ii*dt;
dw=sqrt(dt)*randn;
y(ii)=y(ii-1)+mu*y(ii-1)*dt+sig*y(ii-1)*dw;    % numerical solution to SDE (4.296) 
                                                   % by Euler-Maruyama
                                                   % method (see Chapter 5)
%
end                        % end of time loop for SDE solution
%
yT=y(numel(t));                 % boundary condition y(T) 
yTsq=yT*yT;
sumzz=sumzz+yTsq;    

end                        % end of simulations
%
Ey=sumzz/N;                     % mean E[x^2(T)]
%
u(i)=cnst*Ey;   % solution to PDE, u(t)  at each x-variable  via Equation (4.298)
%
end                        % end of loop on t-variable
%
plot(tm,u,'black');             % Figure 4.10b
hold on

end                        % end of loop on x-variable

% end of MC simulation
%

