%                            Example4_30_Figure 4.12
%
%
%   Example 4.30:  solution of Dirichlet-Poisson problem in 2 dimensions
%                               by probabilistic approach (using MC
%                               simulation)
%__________________________________________________________________________
clc
close all
clear all
%
% step sizes for x and y-variables 
% 100 x 100 grid in x-y plane (-1,+1) x (-1,+1)
dx=0.02;                 
dy=0.02;
%
% time step for integration of SDE%              
dt=0.001;
%
% Total time of integration%                 
T=5.0;
%
%  System data (2- dimensional membrane)
K=1.0;    % uniform load on the system 
w=1.0;    % frequency parameter
a=1.0;                   %  X-limit
b=1.0;                   %  Y-limit
x=-1:dx:a                % range of x variable
y=-1:dy:b  
Nx=numel(x)
Ny=numel(y)
%
% 
t=0:dt:T;                 % time variable
%
% x-y grid size
numel(x)   %  no. of x points
numel(y)   %  no. of y points
%
%
m=numel(t);  % No. of time steps
%
%
%    Solution by MC simulation
N=5000;    % no. of simulations for each grid point(x,y)
%
%
%
figure(1)
u=zeros(Nx,Ny);
for i=2:Nx-1     % edge at x(1) and x(51) is clamped
i         % x-variable counter
%
for j=2:Ny-1          % edge at y(1) and y(51) is clamped
j         % y-variable counter
%
sumx=0.0;
sumy=0.0;
sum=0.0;
%
%
% MC simulation starts
for k=1:N
[i j k]           % x & y variable and MC run counter
xx=zeros(1,m);     % initialization for x
yy=zeros(1,m);     % initialization for y
%
%
% fix first point (x,y)
xx(1)=x(i);
yy(1)=y(j);
%
%
for ii=2:numel(t)    % time integration starts
%
dw1=sqrt(dt)*randn;
xx(ii)=xx(ii-1)+dw1;
%
%
% Find the exit time 'toux' on the x variable
if (abs(xx(ii-1)) < a) && (abs(xx(ii)) > a)
toux=t(ii-1)+(a-abs(xx(ii-1)))*(t(ii)-t(ii-1))/abs(xx(ii)-xx(ii-1));
break
end

end

for jj=2:numel(t)

dw2=sqrt(dt)*randn;
yy(jj)=yy(jj-1)+dw2;
%
% Find the exit time 'touy' on the y variable
if (abs(yy(jj-1)) < b && abs(yy(jj)) > b)
touy=t(jj-1)+(b-abs(yy(jj-1)))*(t(jj)-t(jj-1))/abs(yy(jj)-yy(jj-1));
break
end

end
%
%
% whichever of 'toux' and 'touy' is less, it gives directly 'tou=tou-t'
%                                         in Equation (4.309)
%                                         Here t=0, the initial time  
if(toux<touy)
   tou=toux;
else
   tou=touy;
end
%
sum=sum+(K/(w*w))*(1-exp(w*w*tou));      %  Equation (4.309)
if(toux==0.0)
    sum=0.0;
end
if(touy==0.0)
    sum=0.0;
end

end

u(i,j)=sum/N;
end

end

surf(x,y,u);   % Figure 4_12
xlabel('x')
ylabel('y')
