%                               Example5_1_Fig5_1.m 
%
%    Approximate ANALYTICAL solution of Black-Scholes SDE by Equation 5.37   
%                             ( which may not converge)
%
%    Figure 5.1 contains 1) true solution (Equation 5.35)
%                        2) approximate EM solution with dt=2^-5 (divergent)- Equation 5.37
%                        3) approximate EM solution with dt=2^-9 (divergent)- Equation 5.37
%__________________________________________________________________________
clc
clear all
close all
%
% parameters of Black-Scholes SDE (Equation 5.34)
lambda=2.0;    
mu=1.0;
cnst=lambda-mu*mu/2.0;
% 
%
xzero=1.0;    % initial condition 
%
%__________________________________________________________________________
%           To obtain true solution and 1st approximate EM solution with dt=2^-9
%
T=1;   % total time of integration
dt1=2^-9;   % time step
t1=0:dt1:T;   % time variable to get true solution and 1st approximate EM solution
%
%
% at starting time step - simulation of Brownian motion 
dB(1)=sqrt(dt1)*randn;   % Brownian increment 
B(1)=dB(1);
%
%
% at first time step  
xtrue(1)=xzero*exp(cnst*t1(1)+mu*B(1));  % true solution to Black-Scholes SDE - Equation 5.35
xapprox1(1)=xzero*exp(dt1*(lambda+mu*dB(1)/dt1));  %1st approx. EM solution - Equation 5.37
%
% at all other time steps

% 
fprintf(1, 'true solution and approximate EM solution with dt=2^-9');
%
for k=2:numel(t1)
    k  % time step 
  dB(k) = sqrt(dt1)*randn;
  B(k) = B(k-1) + dB(k);
  xtrue(k)=xzero*exp(cnst*t1(k)+mu*B(k));  % true solution to Black-Scholes SDE - Equation 5.35
  %
  %
  xapprox1(k)=xzero*exp(dt1*(lambda+mu*dB(k)/dt1));  % 1st approximate solution  - Equation 5.37
end
% plotting 
figure(1)
plot(t1,xtrue,'black'),hold on   % plotting the true solution
plot(t1,xapprox1,'--black')      % plotting the 1st approximate solution with  dt1=2^-9
%_____________________________________________________________________________
%
%
%
%
%           To obtain the 2nd approximate EM solution with  dt=2^-5
%
dt2=2^-5;
t2=0:dt2:T;
%
% at first time step  
dB(1)=sqrt(dt2)*randn;
xapprox2(1)=xzero*exp(dt2*(lambda+mu*dB(1)/dt2));% 2nd approximate solution -  Equation 5.37
%
% at all other time steps

fprintf(1, 'true solution and approximate EM solution with dt=2^-5');
for k=2:numel(t2)
    k % time step
  dB(k) = sqrt(dt2)*randn;
  xapprox2(k)=xzero*exp(dt2*(lambda+mu*dB(k)/dt2));% 2nd approximate solution with  dt2=2^-5   
end
plot(t2,xapprox2,'-.black') % plotting the 2nd approximate solution