%                           Example5_1_Fig5_2.m  
%                          (Example 5.1 contd...)
%
%    NUMERICAL solution of Black-Scholes SDE by Explicit EM method  
%                                 (Equation 5.36)   
%
%     Figure 5.2 contains 1) true solution  (Equation 5.35)
%                         2) numerical solution by EM method 
%                           (Equation 5.36)with dt=2^-8 and 2^-9
%__________________________________________________________________________
%
%
%
clc
clear all
close all
%
% parameters of Black-Scholes SDE (Equation 5.34)
lambda=2.0;    
mu=1.0;
cnst=lambda-mu*mu/2.0;
% 
%
xzero=1.0;    % initial condition 
%
T=1;   % total time of integration
%__________________________________________________________________________
%           To obtain i) true solution
%                    ii) and EM solution with dt=2^-9
%
dt1=2^-9;
t1=0:dt1:T;

xold(1)=xzero;   % initial condition
xnew(1)=xzero;  % extra variable needed for iterative process
%
% at starting time step - simulation of Brownian motion 
dB(1)=sqrt(dt1)*randn;
B(1)=dB(1);
%
xtrue(1)=xzero*exp(cnst*t1(1)+mu*B(1)) ; % true solution at initial time
%
%
fprintf(1, 'true solution and approximate EM solution with dt1=2^-9');
%
for k=2:numel(t1)
        k  % time step
  dB(k) = sqrt(dt1)*randn;
  B(k) = B(k-1) + dB(k);
  xtrue(k)=xzero*exp(cnst*t1(k)+mu*B(k));  % true solution at each time step
%
  xnew(k)=xold(k-1)+ lambda*xold(k-1)*dt1 + mu*xold(k-1)*dB(k);  % EM solution with dt=2^-9
  xold(k)=xnew(k);
end
%
figure(1)
plot(t1,xtrue,'black'),hold on    % plotting the true solution
%
%
plot(t1,xnew,'--black')        % plotting the EM solution with dt1=2^-9
% pause
%
%__________________________________________________________________________
%
%           To obtain EM solution a with dt=2^-8
%
dt2=2^-8;
t2=0:dt2:T;
%
xo(1)=xzero;  % initial condition
xn(1)=xzero;   % extra variable needed for iterative process
%
% at starting time step - simulation of Brownian motion 
dB(1)=sqrt(dt2)*randn;
B(1)=dB(1);
%
% at all other time steps
fprintf(1, 'true solution and approximate EM solution with dt1=2^-8');
%
for k=2:numel(t2)
    k  % time step
  dB(k) = sqrt(dt2)*randn;
  B(k) = B(k-1) + dB(k);
%
  xn(k)=xo(k-1)+ lambda*xo(k-1)*dt2 + mu*xo(k-1)*dB(k);  % EM solution with dt2=2^-8
  xo(k)=xn(k);
end
plot(t2,xn,'-.black')  % plotting the EM solution with dt2=2^-8