%                        Example5_1_Fig5_4.m  
%                       (Example 5.1 Contd....)
%
%       weak order of convergence (Equation 5.39b) of Explicit EM method  
%                                 
%
%                   solution of Black-Scholes SDE - (5.34)    
%
%             Figure 5.4 contains the log-log graphs obtained from
%                   three independent MC runs for weak order of
%                             convergence 
%__________________________________________________________________________
%
%
clc
clear all
close all
%
T=1;    % total time of integration
%
N=3000;   % samples in each MC run
%
% parameters of Black-Scholes SDE - (5.34)
lambda=2.0;
mu=1.0;
cnst=lambda-mu*mu/2.0;
%
xzero=1.0;   % initial condition
%
%
for k=1:3
%
k      % independent MC run counter
%
for i=1:N    %  over 3000 samples
%
for j=1:5    % time step sizes
%
%
% choosing a time step
if(j==1)
dt=2^-9;
elseif(j==2)
dt=2*2^-9;
elseif(j==3)
dt=4*2^-9;
elseif(j==4)
dt=8*2^-9;
elseif(j==5)
dt=16*2^-9;
end
dt;
t=0:dt:T;
%
%
xold(1)=xzero;   % initial condition
xnew(1)=xzero;    % extra variable for iterative EM solution
%
% at the first time step
dB(1)=sqrt(dt)*randn;
B(1)=dB(1);
xtrue(1)=xzero*exp(cnst*t(1)+mu*B(1));   %  true solution - Equation 5.35
%
%
% at all other time steps
for k=2:numel(t)
  dB(k) = sqrt(dt)*randn;
  B(k) = B(k-1) + dB(k);
  xtrue(k)=xzero*exp(cnst*t(k)+mu*B(k));   % true solution - Equation 5.35
%
%
% EM solution - Equation (5.36)
  xnew(k)=xold(k-1)+ lambda*xold(k-1)*dt + mu*xold(k-1)*dB(k);
  xold(k)=xnew(k);
end
%
nlast=numel(t);
xtr(i,j)=xtrue(nlast);  % true solution  
xEM(i,j)=xnew(nlast);   % EM solution
%
end
end
%
%
for j=1:5
sum1=0.0;
sum2=0.0;
for i=1:N
sum1=sum1+xtr(i,j)/N;   % for true solution
sum2=sum2+xEM(i,j)/N;   % for EM solution
end
%
err(j)=abs(sum1-sum2);   % error  - weak order of convergence
end
%
%
% time step values
dtvals(1)=2^-9;
dtvals(2)=2*2^-9;
dtvals(3)=4*2^-9;
dtvals(4)=8*2^-9;
dtvals(5)=16*2^-9;
%
figure(1)
loglog(dtvals,err,'-blacks','markersize',5), hold on  % log-log graphs (Equation 5.39b)
%
%
%__________________________________________________________________________
%  to find the slope of the three log-log graphs (after curve-fitting)
C1=log(dtvals);
C2=log(err);
xbar=mean(C1);
ybar=mean(C2);

sum1=0.0;
sum2=0.0;
for j=1:5
  sum1=sum1+C1(j)*C2(j);
  sum2=sum2+C1(j)*C1(j);
end
sum1=sum1-5*xbar*ybar;
sum2=sum2-5*xbar*xbar;
slope=sum1/sum2
c=ybar-slope*xbar;
end
