%                               Example5_2_Fig5_5.m  
%
%                            Duffing oscillator (Equation 5.40)
%
%                     solution by the Explicit EM method (Equation 5.36)
%
%                         Figure 5.5a - E[x(t)]  - displacement
%                     and Figure 5.5b - E[dx(t)] - velocity
%
%           Each figure contains 1) solution with dt=0.05
%                                2) solution with dt=0.001
%                                3) solution with dt=0.0001
%__________________________________________________________________________
%
clc
clear all
close all
%
%
pi=3.14159265;
%
%parameters of the Duffing oscillator - Equation (5.40)
c=4.0;
k=100.0;
alpha=100.0;
P=4.0;
sigma=0.01;
omega=2*pi;  % frequency of the harmonic force
%
T=4.0;  % total time of integration
%
N=100;   % no. of samples in the MC run
%
%
%
%
%__________________________________________________________________________
%                       Explicit EM Run with dt=0.05
%
dt=0.05;
t=0:dt:T;
%
%
%
% initial conditions
x0=0.0;
dx0=0.0;
%
%
for j=1:N
    j  %   sample counter
  x1old=x0;
  x2old=dx0;
  x(1)=x1old;
  dx(1)=x2old;
%
%
% numerical solution by Explicit EM method for Duffing oscillator (5.40) 
  for i=1:numel(t)
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
    x1new=x1old+x2old*dt;
    x2new=x2old+(-c*x2old-k*x1old-alpha*x1old^3+force)*dt+sigma*dB;
    dx(i+1,j)=x2new;
    x(i+1,j)=x1new;
    x1old=x1new;
    x2old=x2new;
  end
%
%
end
%
for i=1:numel(t)
  meanx(i)=mean(x(i,:));
  meandx(i)=mean(dx(i,:));
end
%
%plotting for dt=0.05
figure(1)
plot(t,meanx,'black'), hold on   % E[x(t)]
figure(2)
plot(t,meandx,'black'), hold on  % E[dx(t)]
%
%
%
%
%
%__________________________________________________________________________
%                 Explicit EM Run wth dt=0.001
%
dt=0.001;
t=0:dt:T;
%
%
%
%
x0=0.0;
dx0=0.0;
%
%
for j=1:N
  y1old=x0;
  y2old=dx0;
  y(1)=y1old;
  dy(1)=y2old;
%
% numerical solution by EM method for Duffing oscillator (5.40) 
  for i=1:numel(t)
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
    y1new=y1old+y2old*dt;
    y2new=y2old+(-c*y2old-k*y1old-alpha*y1old^3+force)*dt+sigma*dB;
    dy(i+1,j)=y2new;
    y(i+1,j)=y1new;
    y1old=y1new;
    y2old=y2new;
  end
%
%
end
%
for i=1:numel(t)
  meany(i)=mean(y(i,:));
  meandy(i)=mean(dy(i,:));
end
%
%plotting for dt=0.001
figure(1)
plot(t,meany,'-.black')  % E[x(t)]
figure(2)
plot(t,meandy,'-.black') % E[dx(t)]
%
%
%
%__________________________________________________________________________
%                 Explicit EM Run wth dt=0.0001
%
dt=0.0001;
t=0:dt:T;
%
%
%
%  initial conditions
x0=0.0;
dx0=0.0;
%
%
for j=1:N
  z1old=x0;
  z2old=dx0;
  z(1)=z1old;
  dz(1)=z2old;
%
% numerical solution by EM method for Duffing oscillator (5.40) 
  for i=1:numel(t)
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
    z1new=z1old+z2old*dt;
    z2new=z2old+(-c*z2old-k*z1old-alpha*z1old^3+force)*dt+sigma*dB;
    dz(i+1,j)=z2new;
    z(i+1,j)=z1new;
    z1old=z1new;
    z2old=z2new;
  end
%
%
end
%
for i=1:numel(t)
  meanz(i)=mean(z(i,:));
  meandz(i)=mean(dz(i,:));
end
%
% plotting for dt=0.0001
figure(1)
plot(t,meanz,'--black')   % E[x(t)]
%
figure(2)
plot(t,meandz,'--black')   % E[dx(t)]
 