%                              Example5_4_Fig_5.7  
%
%                    same Duffing oscillator as in Example 5.2
%
%
%                    Higher order scheme (Equation 5.70d) - by Ito-Taylor expansion 
%                      (as represented by Equation 5.90)
%
%    Figure 5.7 contains:  1) solution by explicit EM method
%                                (Equation 5.36) with dt=0.0001 (assumed
%                                to be the true solution)
%                          2) solution by higher order scheme in Eqation (5.70d)  with dt=0.05                                      
%_____________________________________________________________________________________________
%
clc
clear all
close all
%
pi=3.14159265;
%
T=5;    % total time of integration
%
%
% parameters of Duffing oscillator in Equation (5.40)
  c=4.0;
  k=100.0;
  alpha=100.0;
  sigma=0.01;
  omega=2*pi;
  P=4.0;
%
%
N=1000;   % no. of samples in the MC run
%
%
%--------------------------------------------------------------------------
%         Solution by Equation (5.70d) - higher order accuracy
%
dt1=0.05;     % time step
tm=0:dt1:T;   % time variable
N1=numel(tm);
%
%
%
% initial conditions
x0=0.0;
dx0=0.0;
%
%
fprintf(1, 'solution by higher order scheme with dt1=0.05');
%
for j=1:N
    j   % sample counter
  y1old=x0;
  y2old=dx0;
  y(1)=y1old;
  dy(1)=y2old;
%
% start of integration
  for i=1:N1
%
  u1=randn;
  u2=randn;
  u3=randn;
%
%  Brownian motions
  db1=sqrt(dt1)*u2;    
  db2=sqrt(dt1)*u3;
%
% brownian motion
  db=sqrt(dt1)*u1;   
%
% 
  I10=0.5*dt1*(db1+db2/sqrt(3));   %  I10 in Equation 5.90b
  I01=0.5*dt1*(db1-db2/sqrt(3));   %  I01 in Equation 5.90b
%
    a2=-c*y2old-k*y1old-alpha*y1old*y1old*y1old+P*cos(omega*tm(i));   % Equation 5.89a
%
    y1new=y1old+y2old*dt1+sigma*I10+0.5*dt1*dt1*a2;   % Equation 5.90a
%    
    y2new=y2old + sigma*db + dt1*a2 - c*sigma*I10;   
    y2new=y2new - 0.5*dt1*dt1*(c*a2+k*y2old+3*alpha*y2old*y1old*y1old);  % Equation 5.90b
%
    dy(i+1,j)=y2new;
    y(i+1,j)=y1new;
%
    y1old=y1new;
    y2old=y2new;
%
  end                 %  end of integration over tm=0:dt1:T
%
%
end                %   end of  MC  run
%
for i=1:N1
  meany(i)=mean(y(i,:));
  meandy(i)=mean(dy(i,:));
end
%
%plotting
figure(1)     % Figure 5.8a
plot(tm,meany,'--black'),hold on       % E[x(t)]
figure(2)     % Figure 5.8b
plot(tm,meandy,'--black'),hold on      % E[dx(t)]
%
%                   
%__________________________________________________________________________
%                 solution by  Explicit EM Run with dt=0.0001 
%                           (supposed to be the true solution)
%
%initial conditions
x0=0.0;
dx0=0.0;
%
%
%     
dt=0.0001;
t=0:dt:T;
%
%
%
%
fprintf(1, 'true solution by EM method with small time step dt1=0.0001');
%
for j=1:N
    j  % sample counter
  x1old=x0;
  x2old=dx0;
  x(1)=x1old;
  dx(1)=x2old;
%
% numerical solution by Explicit EM method for Duffing oscillator  
  for i=1:numel(t)-1
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
    x1new=x1old+x2old*dt;
    x2new=x2old+(-c*x2old-k*x1old-alpha*x1old^3+force)*dt+sigma*dB;
    dx(i+1,j)=x2new;
    x(i+1,j)=x1new;
    x1old=x1new;
    x2old=x2new;
  end
%
end
%
%
for i=1:numel(t)
  meanx(i)=mean(x(i,:));
  meandx(i)=mean(dx(i,:));
end
%plotting 
figure(1)    % Figure 5.8a
plot(t,meanx,'black'), hold on   % E[x(t)]
figure(2)    % Figure 5.8b
plot(t,meandx,'black'), hold on   % E[dx(t)]
