%                          Example5_4_Fig5_8a.m 
%
%                     Duffing-Van der pol oscillator (Equation 5.91)
%
%
%                    Higher order scheme (Equation 5.70d) by Ito-Taylor expansion 
%                      (as represented by Equation 5.92)
%
% Figures 5.8a contains: solution by higher order scheme in Eqation (5.70d) 
%                         for the deterministic case of sigma =0 with dt=0.001     
%
%_____________________________________________________________________________________________
clc
clear all
close all
%
%
T=10;    % total time of integration
%
N=1;   % no. of samples in the MC run
%
%--------------------------------------------------------------------------
%parameters of the Duffing-Van der Pol oscillator 
alpha =1;
        sigma=0;   % deterministic case
        figure(1)
        y(1)=-4;
y(2)=4;
z(1)=0;
z(2)=0;
plot(y,z,'black'),hold on

%__________________________________________________________________________
% solution by  higher order scheme  in Equation (5.70)d 
% 
%     
dt=0.001;
h=dt;
t=0:dt:T;
Nlast=numel(t);
%
%initial conditions
%x0 varied from -0.2 to -4 in 10 steps and from 0.2 to 4 in another 10 steps
%
for k=1:20
    if(k==1)
        x0=-2;
    elseif(k==2)
        x0=-2.2;
    elseif(k==3)
        x0=-2.4;
    elseif(k==4)
        x0=-2.6;
    elseif(k==5)
        x0=-2.8;
    elseif(k==6)
        x0=-3.2;
    elseif(k==7)
        x0=-3.4;
    elseif(k==8)
        x0=-3.6;
    elseif(k==9)
        x0=-3.8;
    elseif(k==10)
        x0=-4;  
    elseif(k==11)
        x0=2;
    elseif(k==12)
        x0=2.2;
    elseif(k==13)
        x0=2.4;
    elseif(k==14)
        x0=2.6;
    elseif(k==15)
        x0=2.8;
    elseif(k==16)
        x0=3.2;
    elseif(k==17)
        x0=3.4;
    elseif(k==18)
        x0=3.6;
    elseif(k==19)
        x0=3.8;
    elseif(k==20)
        x0=4;      
    end
%    
dx0=0.0;
%
x=zeros(numel(t),1);
dx=zeros(numel(t),1); 
%
for i=1:N
%

x=zeros(numel(t),1);
dx=zeros(numel(t),1); 

  x1old=x0;
  x2old=dx0;
  x(1)=x0;
  dx(1)=dx0;
%
% start of integration
  for kk=1:numel(t)-1
%       
% k, i, kk % initial condition counter, sample counter and time counter
%      
      a1=x2old;
      a2=x1old*(alpha-x1old^2)-x2old;
%
  u1=randn;
  u2=randn;
  u3=randn;
%
  db1=sqrt(dt)*u2;
  db2=sqrt(dt)*u3;
  db=sqrt(dt)*u1;
%
  I10=0.5*dt*(db1+db2/sqrt(3));
  I01=0.5*dt*(db1-db2/sqrt(3));
  I00=0.5*h*h;
%  
    x1new=x1old + a1*dt;
    x1new=x1new + sigma*x1old*I10 ;
%
    x1new=x1new + a2*I00; 
%
%        
    x1new=x1new + a2*I00;
%    
    x2new=x2old + a2*dt + sigma*x1old*db;    
    x2new=x2new - sigma*x1old*I10 + sigma*x2old*I01;
%
    x2new=x2new + (a1*(alpha-3*x1old*x1old)-a2)*I00; 
%                                                    
%
    dx(kk+1)=x2new;
    x(kk+1)=x1new;
%    
    x1old=x1new;
    x2old=x2new;
  end
%
% 
plot(x,dx,'black'),hold on
end
%
end
