%                        Example5_4_Fig5_8fandg.m  
%
%                     Duffing-Van der pol oscillator (Equation 5.91)
%
%
%                    Higher order scheme (Equation 5.70d by Ito-Taylor expansion 
%                      (as represented by Equation 5.92)
%
%
% Figures 5.8f -  containing two time histories by higher order scheme in Equation (5.70d) 
%                             with dt=0.001 and different ICs -0.2 and -0.4
%                             and with sigma = 0.2
% Figures 5.8g -  containing two time histories by higher order scheme in Equation (5.70d) 
%                             with dt=0.001 and different ICs -0.2 and -0.4
%                             and with sigma = 0.5
%
% 
%  NOTE: For Figure 5.8g,try the program repeatedly in order to get a clear switching of the two steady states,
%__________________________________________________________________________
clc
clear all
close all
%
%
T=200;    % total time of integration
%
N=1;   % no. of samples in the MC run
%
%--------------------------------------------------------------------------
%parameters of the Duffing-Van der Pol oscillator 
alpha =1;
% sigma   % varied from 0, 0.2 and 0.05 
for ip=1:2
%   
    if(ip==1)
        sigma=0.2;   % low multiplicative noise
%
    elseif(ip==2)
        sigma=0.5;   % high multiplicative noise
%
    end
%__________________________________________________________________________
%                solution by  higher order scheme  in Equation (5.70)d
% 
%     
dt=0.001;
h=dt;
t=0:dt:T;
Nlast=numel(t);
%
%initial conditions
%x0 varied  -0.2 or -4 
%
for k=1:2
    if(k==1)
        x0=-2;
    elseif(k==2)
        x0=-4;  
    end
%    
dx0=0.0;
%
x=zeros(numel(t),1);
dx=zeros(numel(t),1); 
%
for i=1:N
    i % sample counter
%
x=zeros(numel(t),1);
dx=zeros(numel(t),1); 

  x1old=x0;
  x2old=dx0;
  x(1)=x0;
  dx(1)=dx0;
%
% start of integration
for kk=1:numel(t)-1
%  
% k, i, kk % initial condition counter, sample counter and time counter
%
%
      a1=x2old;
      a2=x1old*(alpha-x1old^2)-x2old;
%
  u1=randn;
  u2=randn;
  u3=randn;
%
  db1=sqrt(dt)*u2;
  db2=sqrt(dt)*u3;
  db=sqrt(dt)*u1;
%
  I10=0.5*dt*(db1+db2/sqrt(3));
  I01=0.5*dt*(db1-db2/sqrt(3));
  I00=0.5*h*h;
%  
    x1new=x1old + a1*dt;
    x1new=x1new + sigma*x1old*I10 ;
    %------------------------------------------------------------------------------------
    x1new=x1new + a2*I00;   
%    
    x2new=x2old + a2*dt + sigma*x1old*db;    
    x2new=x2new - sigma*x1old*I10 + sigma*x2old*I01;
    x2new=x2new + (a1*(alpha-3*x1old*x1old)-a2)*I00;   
%
    dx(kk+1)=x2new;
    x(kk+1)=x1new;
    
    x1old=x1new;
    x2old=x2new;
end                % end of time integration
%
figure(ip)
%
% 
%
    plot(t,x,'black'),hold on  
%
end   % end of samples

end   % end of the two initial conditions for each 'sigma'
%
%
end     % end of the two 'sigma' values
