%                             Example5_7_Fig5_12.m  
%
%                   weak order of convergence  -  global error plot
%
%         Higher order numerical integration scheme (Equation(5.70)d) 
%                              
%
%              Black-Scholes SDE (Equation 5.34) - as applied
%                         according to Equation (5.158)a) with MSIs
%              
%      Figure 5.12 contains two figures:
%           a) with parameters lamda=2, mu=0.01      - Figure 5.12a
%           b) with parameters lamda=2, mu=0.1       - Figure 5.12b    
%
%   Both Figure 5.12a and Figure 5.12b contain:
%        global error plot - weak order of convergence defined in Equation 5.2b
%
%   NOTE: It requires two independent runs to obtain Figures 5.12a and 5.12b
%         To obtain Figure 5.12a, keep mu =0.01 and run the code
%                   Figure 5.12b, keep mu =0.1  and run the code
%__________________________________________________________________________
%
clc
clear all
close all 
%
T=1;    % total time of integration
%
N=100;   % no. of samples in each (independent) MC run 
%
NMC=20;   % no. of independent MC runs
%
%----------------------------------------------------------------
%parameters of the Black-Scholes SDE (Equation 5.34)
  lambda=2.0; 
  mu=0.01;  %   Figure 5.12a
%   mu=0.1;   %   Figure 5.12b   % UNCOMMENT this statement to get 
                                    % the result for Figure 5.12b
  cnst=lambda-mu*mu/2.0;                                    
%----------------------------------------------------------------
% initial condition
x0=1.0;
% initialization of the variable indocating global error
 for i=1:5
   errm(i)=0.0;
 end
%
%
%
%  start of MC runs
for L=1:NMC
% L    % MC counter
%
%
for i=1:5   % integration with 5 different time steps
%
if(i==1)
 dt=2^-6;
 elseif(i==2)
 dt=2*2^-6;
%
 elseif(i==3)
 dt=4*2^-6;
%
 elseif(i==4)
 dt=8*2^-6;
%
 elseif(i==5)
 dt=16*2^-6;
end
%
t=0:dt:T;   % time variable
Nt=numel(t);
%
% 
xtr=0.0;   % initializaton for true solution variable
xwMSI=0.0;  % initializaton for the solution variable of higher order scheme
                                        % with MSIs
%
% 
for j=1:N
% j  % sample counter for each MC run
db(1)=sqrt(dt)*randn;
%
B(1)=db(1);
xtrue(1)=x0*exp(cnst*t(1)+mu*B(1));    % true solution at t(1)
%
xold(1)=x0;
%
for k=2:numel(t)
%
  u1=randn;
  u3=randn;
  u4=randn;
%
  db1(k)=sqrt(dt)*u3;
  db2(k)=sqrt(dt)*u4;
%
  db(k)=sqrt(dt)*u1;
  % MSIs involved in Equation (5.158)a
  I1=db(k);
  I10=0.5*dt*(db1(k)+db2(k)/sqrt(3));
  I01=0.5*dt*(db1(k)-db2(k)/sqrt(3));
  I11=0.5*(I1*I1-dt);
  I111=(1/6.0)*(I1*I1-3*dt)*I1;
%
  B(k) = B(k-1) + db(k);
  xtrue(k)=x0*exp(cnst*t(k)+mu*B(k));   % true solution at t(k)
%
% equation (5.70d)
  xnew(k)=xold(k-1) + lambda*xold(k-1)*dt + mu*xold(k-1)*db(k);
  xnew(k)=xnew(k) + lambda*mu*xold(k-1)*(I10+I01);
  xnew(k)=xnew(k) + 0.5*lambda*lambda*xold(k-1)*dt*dt;
  xnew(k)=xnew(k) + mu*mu*xold(k-1)*I11;
  xnew(k)=xnew(k) + mu*mu*mu*xold(k-1)*I11;
%
  xold(k)=xnew(k);
%
end          %  end of integration for true &  higher order solutions
%
xtr=xtr+xtrue(numel(t))/N;
xwMSI=xwMSI+xnew(numel(t))/N;
%
end     %   end of N samples
%
err(i)=abs(xtr-xwMSI);  % global error according to Equation 5.2b per each MC run
errm(i)=errm(i)+err(i)/NMC; % average global error over all MC runs
%
xdt(i)=log(dt)/log(2);% logarithm of time step wrt the base '2'
%
end      %    end of i=5 time steps
%
end      %   end of NMC independent Monte Carlo runc
%
for i=1:5
  errm(i)=log2(errm(i)); % logarithm of global error wrt the base '2'
end
%
figure(1)% Figure 5.12a or Figure 5.12b 
plot(xdt,errm),hold on  % log-log error plot
%
%--------------------------------------------------------------------------
%To find 'gama' in Equation (5.2)b from a linear fit by regression 
xbar=mean(xdt)
ybar=mean(errm)
%
sum1=0.0;
sum2=0.0;
for i=1:5
  sum1=sum1 + xdt(i)*errm(i);
  sum2=sum2 + xdt(i)*xdt(i);
end
num=5*sum1-25*xbar*ybar;
denom=5*sum2-25*xbar*xbar;
%
'gama of weak order of convergence in Equation (5.2)b'
slope=num/denom             % 'gama' in Equation (5.2)b
c=(ybar-slope*xbar)/5;
%--------------------------------------------------------------------------

