%                          Example5_8_Fig5_13.m  
%
%         weak solutions for Black-Scholes SDE (Equation 5.34)
%              
% 
%      Figure 5.13 contains two figures:
%           a) with parameters lamda=2, mu=1.0       - Figure 5.13a
%           b) with parameters lamda=2, mu=0.5       - Figure 5.13b    
%
%      Both Figures 5.13a and 5.13b contain:
%          i)  weak solution by higher order scheme in Equation (5.158a) with MSIs
%          ii) weak solution by the same higher order scheme 
%                       replaced by RVs - Equation (5.158b) 
%          iii) true solution by Equation (5.35)
%
%__________________________________________________________________________
clc
clear all
close all
%
%
N=100;  % no. 0f samples in the MC run 
%
% initial condition
xzero=1.0;
%
%
T=1;      % total time of integration
dt=2^-7;  % time step
t=0:dt:T; % time variable
M=numel(t)
%
%
%__________________________________________________________________________                       
%      weak solution by Equation (5.158a) - higher order scheme with MSIs
%
%          The true solutin (Equation 5.35) is also computed below 
%
for icase =1:2
%
xMSI=zeros(M,N);
xRVs=zeros(M,N);
xtrue=zeros(M,N);
%
% parameters of the Black-Scholes SDE (Equation 5.34)
if icase==1
  lambda=2.0;
  mu=1.0;
elseif icase==2
  lambda=2.0;
  mu=0.5;  
end
%
%
cnst=lambda-mu*mu/2.0;
for j=1:N
%     j   % sample counter
  xold=xzero;
  xMSI(1,j)=xold;
  xRVs(1,j)=xold;
  xtrue(1,j)=xold;
dB(1)=sqrt(dt)*randn;   % Brownian increment 
B(1)=dB(1);%
%
%at all other time instants 
x1old=xzero;
x2old=xzero;
  for i=1:M-1
%
  u2=randn;
  u3=randn;
%
  db1=sqrt(dt)*u2;
  db2=sqrt(dt)*u3;
%
%
  I1=db1;
  I10=0.5*dt*sqrt(dt)*(db1+db2/sqrt(3));
  I01=0.5*dt*sqrt(dt)*(db1-db2/sqrt(3));
  I11=0.5*(I1*I1-dt);
  I111=(1/6)*(I1*I1-3*dt)*I1;
%
    x1new=x1old + mu*x1old*I1 + lambda*x1old*dt;
    x1new=x1new + mu*mu*x1old*I11 + lambda*mu*x1old*I01;
    x1new=x1new + lambda*mu*x1old*I10;
    x1new=x1new + 0.5*lambda*lambda*x1old*dt*dt;
%
    xMSI(i+1,j)=x1new; %solution by higher order scheme in equation (5.158a)
    x1old=x1new;
 %
  dB(i+1) = sqrt(dt)*randn;
  B(i+1) = B(i) + dB(i+1);
  xtrue(i+1,j)=xzero*exp(cnst*t(i+1)+mu*B(i+1));  % true solution to Black-Scholes SDE - Equation 5.35
%    
%
  end                 %  end of integration over tm=0:dt:T
%
%
end                %   end of  MC  simulations
%
for i=1:M
  meanxtrue(i)=mean(xtrue(i,:)); 
  meanxMSI(i)=mean(xMSI(i,:));
end
%
if (icase==1)
  figure(1)              % Figure 5.13a   for lamda =2 , mu =1.0
  plot(t,meanxMSI,'--black'),hold on  % solution with MSIs 
  plot(t,meanxtrue,'black'),hold on   % supposed to be the true solution
end
if (icase==2)
  figure(2)              % Figure 5.13b   for lamda =2 , mu =0.5
  plot(t,meanxMSI,'--black'),hold on   % solution with MSIs 
  plot(t,meanxtrue,'black'),hold on    % supposed to be the true solution
end
%
%  
%
%__________________________________________________________________________         
%      weak solution by Equation (5.158b) - same higher order scheme 
%                               but now with RVs
% Choice of RVs by way of their probabilities 
% for xi
xi1=-sqrt(3);
xi2=0;
xi3=sqrt(3);
pxi1=1/6;
pxi2=2/3;
pxi3=1/6;
%
% for sigma
sig1=-1;
sig2=1;
psig1=1/2;
psig2=1/2;
%
%
for j=1:N
%     j  % sample counter
   xRVs(1,1)=xzero;  
  x1old=xzero;
%
  for i=1:M-1
%
% for xi RV - Equation (5.155)a
    u=rand;
    if u<=pxi1
        xi(i)=xi1;
    elseif u>pxi1 && u<=pxi1+pxi2
        xi(i)=xi2;
    elseif u>pxi1+pxi2
        xi(i)=xi3;
    end
% 
% for sig RV  - Equation (5.155)b
    if u<=psig1
        sig(i)=sig1;
    elseif u>psig1
        sig(i)=sig2;
    end
%
zeta(i)=xi(i)/2;  % zeta RV  in Equation (5.156)a
xijk(i)=0.5*(xi(i)*xi(i)-sig(i)*sig(i));  % Equation (5.156)b
%    
    x1new=x1old + mu*x1old*sqrt(dt)*xi(i) + lambda*x1old*dt;
    x1new=x1new + mu*mu*x1old*dt*xijk(i) + lambda*mu*x1old*dt*sqrt(dt)*(xi(i)-zeta(i));
    x1new=x1new + lambda*mu*x1old*dt*sqrt(dt)*zeta(i) ;
    x1new=x1new + 0.5*lambda*lambda*x1old*dt*dt;
%
    xRVs(i+1,j)=x1new; %solution by higher order scheme in equation (5.158b)with RVs
    x1old=x1new;
%
  end                 %  end of integration over tm=0:dt:T
%
%
end                %   end of  MC  simulations
%
for i=1:M
  meanxRVs(i)=mean(xRVs(i,:));
end
%
if (icase==1)
  figure(1)              % Figure 5.13a   for lamda =2 , mu =1.0
  plot(t,meanxRVs,'-.black'),hold on   % solution with RVs
end
if (icase==2)
  figure(2)              % Figure 5.13b   for lamda =2 , mu =0.5
  plot(t,meanxRVs,'-.black'),hold on   % solution with RVs
end

end
