%                             Example6_1_Fig6_3.m  
%
%                   Non-linear filtering by LTL based-EKF 
%
%               Non-linear  Duffing oscillator (Ghosh et al. - 
%               Comput. Methods Appl. Mech. Engrg. 196, 2007)
%
%                                 
%                   Responses:  E[X(t)] & E(dx(t)]                     
%
%                Figure 6.3a,b by LTL based-EKF 
%	--------------------------------------------------------	
%
clc
close all
clear all
%
%
pi=3.14159265;
pi2=2*pi;
%--------------------------------------------------------------------
% data of Duffing oscillator (Equation 6.56)
c=0.05;
k=4;
alpha=7;
eta=c;  
w=sqrt(k);     % system frequency in rad/sec.
%
Lamda=1.5*w;   % excitation frequency in rad/sec.
A=3;           % amplitude of the excitation (harmonic) force
%
sigx=0.05;      % process noise intensity  (Equation (6.56)  
Q=[0 0.01;0.01 1.0];   % initial process noise covariance matrix;
%
%
sigy=0.03;    % measurement noise intensity
R = 0.009;    % measurement noise variance (in general a matrix)
%_________________________________________________________________
%  intermediate variables
  c1=c;
  c2=k;
  c3=A;
  c4=sigx;  
  c5=0.0;
  g0=c4;
  c6=alpha;
%----------------------------------------------------------------
%
N=1;      % no. of simulations
%
% initial state vector
x0=0.0;
dx0=0.0;
P=[0.01 0; 0 0.01];   % Initial prediction for error covariance matrix 
%
Qpre=Q;   % initialization of the Q matrix in equation 6.48
%
%
T=30;   % total time of integraion in sec.
dt1=0.01;   % time step
t=0:dt1:T;   % time variable
N1=numel(t);
h=dt1;
%_______________________________________________________________________
%                   Case 1. measurement on displacement 
H=[1 0];  % observation matrix  on velocity for Figure 6.3a
%               -----------------------------------------------
% 
%
%
Qpre=[0 0.01;0.01 1.0];     % initial assumption
%
for j=1:N
j
xest(1,j)=x0;
dxext(1,j)=dx0;
%
%
  y1old=x0;
  y2old=dx0;
% 
   y1aold=x0;
  y2aold=dx0;
%
%
  for k=2:N1      
  t(k);
      
%-------------------------------------------------------------      
%  Step 1.  to find xpre form process equation without noise     
    
    y1a=y1old;
    y2a=y2old;

    err=1;

    while err>1e-4
      y1a_1=y1a;
      y2a_1=y2a;

      phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1old - phi12*y2old;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);
 
      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);
    
      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1old + phi22*y2old;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
     
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;
  
       err=(y1a_2-y1a_1)^2;
        err=abs(y1a_2-y1a_1);

     y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop

y1a;
y2a;
xpre=[y1a; y2a];
 %-------------------------------------------------------------------
 
 % Step 2.       True solution x & dx 
 % 
  y1a=y1aold;
  y2a=y2aold;

    err=1;

    while err>1e-4
        
      y1a_1=y1a;
      y2a_1=y2a;

    phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1aold - phi12*y2aold;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);

      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);

      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1aold + phi22*y2aold;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;     

      err=(y1a_2-y1a_1)^2;
 
      err=abs(y1a_2-y1a_1);
      
      y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop  
   
 xTR=[y1a; y2a];
%-------------------------------------------------------------------  
 % Step 3. to find predicted covariance matrix Ppre at current t
 %       first compute the state transition matrix 'phi' in Equation 6.47
 %       
tmin=t(k-1);
tmax=t(k);

y1a=y1old;
         
%  
A1=c6*(1+y1a^2);
phi11=1-0.5*h^2*A1+(1/6)*h^3*c1*A1;
phi12=h-0.5*h^2*c1+(1/6)*h^3*(c1^2-A1);
phi21=-h*A1+0.5*h^2*c1*A1+(1/6)*h^3*(A1^2-c1^2*A1);
phi22=1-h*c1+0.5*h^2*(c1^2-A1)+(1/6)*h^3*(2*c1*c6-c1^3);    
      phi=zeros(2,2);
        
      phi(1,1)=phi11;
      phi(1,2)=phi12;
      phi(2,1)=phi21;
      phi(2,2)=phi22; 
 
 % to find Ppre matrix
      Ppre=phi*P*phi'+ Qpre;
     
 % Step 4 to find Kalman gain matrix 
 AA=H*Ppre*H'+R;
 Kgain=Ppre*H'*inv(AA);
 
 % Step 5.  Observations 
 u1=sqrt(dt1*sigy)*randn;    
  Yobs=H*xTR+u1;  
 yy(k,j)=Yobs;     
 
 % Step 6. Udate the states 
 xUp=xpre+Kgain*(Yobs-H*xpre);
 
 xest(k,j)=xUp(1);
 dxest(k,j)=xUp(2);

   y1old=xUp(1);
   y2old=xUp(2);  
   
   y1aold=xTR(1);
   y2aold=xTR(2); 

%Step 7 Find opdated error covariance matrix   
Im=[1 0;0 1];
S1=[Im-Kgain*H]*Ppre*[Im-Kgain*H]';
PUp=S1+Kgain*R*Kgain';
P=PUp;
%
%
%  find and update Q matrix (Equation 6.48) - which is the 
%                 process error covariance matrix
   
  A1=1+y1a^2;
hj=tmax-tmin;     
Iphi12j = hj^2/2 - c1*hj^3/6 + (c1^2-c2*A1)*hj^4/24;

hk=tmax-tmin;     
Iphi12k = hk^2/2 - c1*hk^3/6 + (c1^2-c2*A1)*hk^4/24;

Iphi22k = hk-c1*hk^2/2 + hk^3/6*(c1^2-c2*A1) + hk^4/24*(2*c1*c2*A1-c1^3);

Iphi22j = hj-c1*hj^2/2 + hj^3/6*(c1^2-c2*A1) + hj^4/24*(2*c1*c2*A1-c1^3);

Qpre(1,1)=g0^2*Iphi12j*Iphi12k;
Qpre(1,2)=g0^2*Iphi12j*Iphi22k;
Qpre(2,1)=Qpre(1,2);
Qpre(2,2)=g0^2*Iphi22j*Iphi22k;    
%
%
  end    % end of integration     k=1:N1
%
end    % of N samples
%
%
%
for k=1:N1    
    xwM(k)=0.0;
    for j=1:N
    xwM(k)= xwM(k)+xest(k,j)/N;
    end    
end
%
for k=1:N1   
       yobs(k)=0.0;
    for j=1:N
       yobs(k)= yobs(k)+yy(k,j)/N;
    end    
end
%
figure(1)                  %     Figure 6.3a
plot(t,xwM,'--black'),hold on    % estimated displacement
%
plot(t,yobs,'black')      % observation on displacement%
%
%__________________________________________________________________________
%                   Case 2. measurement on velocity 
H=[0 1];  % observation matrix  on velocity for Figure 5.3b
%           -----------------------------------------------
%
%
% initial state vector
x0=0.0;
dx0=0.0;
P=[0.01 0; 0 0.01];   % Initial prediction for error covariance matrix 
Qpre=[0 0.01;0.01 1.0];     % initial assumption
for j=1:N
j
xest(1,j)=x0;
dxext(1,j)=dx0;
%
%
  y1old=x0;
  y2old=dx0;
% 
   y1aold=x0;
  y2aold=dx0;
%
%
  for k=2:N1      
  t(k);
      
%-------------------------------------------------------------      
%  Step 1.  to find xpre form process equation without noise     
    
    y1a=y1old;
    y2a=y2old;

    err=1;

    while err>1e-4
      y1a_1=y1a;
      y2a_1=y2a;

      phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1old - phi12*y2old;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);
 
      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);
    
      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1old + phi22*y2old;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
     
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;
  
       err=(y1a_2-y1a_1)^2;
        err=abs(y1a_2-y1a_1);

     y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop

y1a;
y2a;
xpre=[y1a; y2a];
 %-------------------------------------------------------------------
 
 % Step 2.       True solution x & dx with noise

  u1=randn;
  u2=randn; 
%
  I1 = sqrt(dt1)*u1;
  I2 = sqrt(dt1)*u2;

  y1a=y1aold;
  y2a=y2aold;

    err=1;

    while err>1e-4
        
      y1a_1=y1a;
      y2a_1=y2a;

    phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1aold - phi12*y2aold;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);

      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);

      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1aold + phi22*y2aold;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;     

      err=(y1a_2-y1a_1)^2;
 
      err=abs(y1a_2-y1a_1);
      
      y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop  
   
 xTR=[y1a; y2a];
%-------------------------------------------------------------------  
 % Step 3. to find predicted covariance matrix Ppre at current t
 %       first compute the state transition matrix 'phi' in Equation 6.47
 %       
tmin=t(k-1);
tmax=t(k);

y1a=y1old;
         
%  
A1=c6*(1+y1a^2);
phi11=1-0.5*h^2*A1+(1/6)*h^3*c1*A1;
phi12=h-0.5*h^2*c1+(1/6)*h^3*(c1^2-A1);
phi21=-h*A1+0.5*h^2*c1*A1+(1/6)*h^3*(A1^2-c1^2*A1);
phi22=1-h*c1+0.5*h^2*(c1^2-A1)+(1/6)*h^3*(2*c1*c6-c1^3);    
      phi=zeros(2,2);
        
      phi(1,1)=phi11;
      phi(1,2)=phi12;
      phi(2,1)=phi21;
      phi(2,2)=phi22; 
 
      % to find Ppre matrix
      Ppre=phi*P*phi'+ Qpre;
     
 % Step 4 to find Kalman gain matrix 
 AA=H*Ppre*H'+R;
 Kgain=Ppre*H'*inv(AA);
 
 % Step 5.  Observations 
 u1=sqrt(dt1*sigy)*randn;    
  Yobs=H*xTR+u1;  
 dyy(k,j)=Yobs;     
 
 % Step 6. Udate the states 
 xUp=xpre+Kgain*(Yobs-H*xpre);
 
 xest(k,j)=xUp(1);
 dxest(k,j)=xUp(2);

   y1old=xUp(1);
   y2old=xUp(2);  
   
   y1aold=xTR(1);
   y2aold=xTR(2); 

%Step 7 Find opdated error covariance matrix   
Im=[1 0;0 1];
S1=[Im-Kgain*H]*Ppre*[Im-Kgain*H]';
PUp=S1+Kgain*R*Kgain';
P=PUp;
%
%
%  find and update Q matrix (Equation 6.48) - which is the 
%                 process error covariance matrix   
A1=1+y1a^2;
hj=tmax-tmin;     
Iphi12j = hj^2/2 - c1*hj^3/6 + (c1^2-c2*A1)*hj^4/24;

hk=tmax-tmin;     
Iphi12k = hk^2/2 - c1*hk^3/6 + (c1^2-c2*A1)*hk^4/24;

Iphi22k = hk-c1*hk^2/2 + hk^3/6*(c1^2-c2*A1) + hk^4/24*(2*c1*c2*A1-c1^3);

Iphi22j = hj-c1*hj^2/2 + hj^3/6*(c1^2-c2*A1) + hj^4/24*(2*c1*c2*A1-c1^3);

Qpre(1,1)=g0^2*Iphi12j*Iphi12k;
Qpre(1,2)=g0^2*Iphi12j*Iphi22k;
Qpre(2,1)=Qpre(1,2);
Qpre(2,2)=g0^2*Iphi22j*Iphi22k;    
%
  end    % end of integration     k=1:N1
%
end    % of N samples
%---------------------------------------------------------------
%
%
for k=1:N1    
    dxwM(k)=0.0;
    for j=1:N
    dxwM(k)= dxwM(k)+dxest(k,j)/N;
    end    
end
%
%
for k=1:N1 
      dyobs(k)=0.0;
    for j=1:N  
      dyobs(k)= dyobs(k)+dyy(k,j)/N;
    end    
end
%
figure(2)                   % Figure 6.3b
plot(t,dxwM,'--black'),hold on       % estimated velocity
%
plot(t,dyobs,'black')   % Observation on velocity
