%                       Example6_2_Fig6_4and5.m
%
%                   Non-linear filtering problem
%
%		        Parameter estimation by Extended Kalman filter (EKF)
%          
%          Case 1:             Figure 6.4 for displacement measurement
%                           &
%          Case 2:             Figure 6.5  for velocity measurement
%           
                     
%
%                     Non-linear  Duffing oscillator
%    
%________________________________________________________________________
clc
clear all
close all
%
%
% data of Duffing oscillator (Equation 6.64)
c=0.9;   %Ghosh et al. 2007 (Comput. Methods Appl. Mech. Engrg, V 196)
k=7;
w=sqrt(k);
eta=c/(2*w);
alpha=3.0;
A=4;
Lamda=3.5;
%
sigx=0.001;
sigy=0.03;
g0=sigx;
%
%
T=10;
dt=0.001;
t=0:dt:T;
h=dt;

length(t);
%
R = 0.009; % measurement error covariance
%
c7=0.00002;
sig=c7;
%
N=10;   % No. of simulations
%
M=numel(t);
%
%______________________________________________________________________
%                   Case 1:   observation on displacement
%
H=[1 0 0 0 0];  % observation matrix (only displacement is observed)
%           -------------------------------------------------
%
%
for j=1:N
  j      % simulation counter
% initializaion of states
x0=0;
dx0=0;
%
Qpre=zeros(5,5);
phi=zeros(5,5);
%
% set up the size of the innovations vector
Inn = zeros(size(R));
%
%error covariance marix  P - initialization
P=zeros(5,5);
P(1,1)=0.01;
P(2,2)=0.01;
P(3,3)=1;
P(4,4)=1;
P(5,5)=1;
xp=zeros(M,N);
dxp=zeros(M,N);
%
for i=1:M
    xstiff(i,j)=0;
    xeta(i,j)=0;
    xalpha(i,j)=0;
end
%
 y1old=x0;
  y2old=dx0;
  y3old=2;     % c
  y4old=5; % K
  y5old=5;  % alpha   % these five for getting the predicted state 
  
  y1aold=x0;
  y2aold=dx0;  
  y3aold=0.9;     % c
  y4aold=7; % K
  y5aold=3;  % alpha  % these five for getting the true state 

for i=2:M
%    t(i) 

u3=randn;
u4=randn;
u5=randn;
  
I3 = sqrt(dt*c7)*u3;
I4 = sqrt(dt*c7)*u4;
I5 = sqrt(dt*c7)*u5;
 
 y3aold= y3aold+I3;
y4aold= y4aold+I4;
 y5aold= y5aold+I5;   

% 
 %  Step 1.  to find xpre form process equation without noise 
 %   by LTL method (Ghosh et al. 2007 (Comput. Methods Appl. Mech. Engrg, V 196) 
% 
  % intermediate variables
  c1=y3old;
  c2=y4old;
  c3=A;
  c4=sigx;  
  c5=0.0;
  g0=c4;
  c6=y5old;

    y1a=y1old;
    y2a=y2old;

    err=1;

    while err>1e-4
      y1a_1=y1a;
      y2a_1=y2a;

      phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1old - phi12*y2old;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);
 
      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);
    
      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1old + phi22*y2old;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
     
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;
  
       err=(y1a_2-y1a_1)^2;
        err=abs(y1a_2-y1a_1);

     y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop

y1a;
y2a;

 xpre = [y1a; y2a; c1; c2; c6];

 xp(i,j)=y1a;
 dxp(i,j)=y2a;
%
%
% Step 3  To find true solution (without noise) by LTL method

%  intermediate variables
  c1=y3aold;
  c2=y4aold;
  c3=A;
  c4=sigx;  
  c5=0.0;
  g0=c4;
  c6=y5aold;
%
 y1a=y1aold;
  y2a=y2aold;

    err=1;

    while err>1e-4
        
      y1a_1=y1a;
      y2a_1=y2a;

    phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1aold - phi12*y2aold;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);

      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);

      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1aold + phi22*y2aold;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;     

      err=(y1a_2-y1a_1)^2;
 
      err=abs(y1a_2-y1a_1);
      
      y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop  
   
 xTr=[y1a; y2a; c1; c2; c6];
 
 y1aold=xTr(1);
 y2aold=xTr(2);
% 
    y3aold=xTr(3);   %c
    y4aold=xTr(4);    %K
    y5aold=xTr(5);    % alpha
%
      y1a=y1old;
      y2a=y2old;
      y3a=y3old;
      y4a=y4old;
      y5a=y5old;

%
 % Step 3. to find predicted covariance matrix P at current t
 %       for this to first find Q matrix (Equation 6.48) - which is the 
 %                 process error covariance matrix

 tmin=t(i-1);
 tmax=t(i);

hj=tmax-tmin;  
hk=hj;

Qpre=zeros(5,5);


Qpre(1,1)=0.25*sigx^2*hj^2*hk*2  ;

Qpre(1,2)=sigx^2*(0.5*hj^2*hk-0.25*y3a*hj^2*hk^2)  ;
Qpre(2,2)=sigx^2*(hj-0.5*y3a*hj^2)*(hk-0.5*y3a*hk^2) ;
Qpre(2,2)=Qpre(2,2)+0.25*y2a^2*sig^2*hj^2*hk^2;
Qpre(2,2)=Qpre(2,2)+0.25*y1a^2*sig^2*hj^2*hk^2;
Qpre(2,2)=Qpre(2,2)+0.25*y1a^6*sig^2*hj^2*hk^2;


Qpre(2,3)=-0.5*sig^2*y2a*hj^2*hk;
Qpre(2,4)=-0.5*sig^2*y1a*hj^2*hk;
Qpre(2,5)=-0.5*sig^2*y1a^3*hj^2*hk;

Qpre(3,3)=sig^2*hj*hk;
Qpre(4,4)=sig^2*hj*hk;
Qpre(5,5)=sig^2*hj*hk;

Qpre(2,1)=Qpre(1,2);

Qpre(3,2)=Qpre(2,3);

Qpre(4,2)=Qpre(2,4);

Qpre(5,2)=Qpre(2,5);

Qpre;
  
      
     % to find Ppre matrix
     
     phi=zeros(5,5);
     
     phi(1,1)=1;
     phi(1,2)=h;

     phi(2,1)=-h*(y4a+3*y1a^2*y5a);
     phi(2,2)=1-h*y3a;
     phi(2,3)=-h*y2a;
     phi(2,4)=-h*y1a;
     phi(2,5)=-h*y1a^3;
     phi(3,3)=1;
     phi(4,4)=1;
     phi(5,5)=1;    

     Ppre=phi*P*phi'+ Qpre;
     
 % Step 4 to find Kalman gain matrix 
 AA=H*Ppre*H'+R;
 Kgain=Ppre*H'*inv(AA);
 
 % Step 5.  Observations
 u1=sqrt(dt*sigy)*randn; 
 Yobs=H*xTr+u1;
 
 % Step 6. Udate the state
 xUp=xpre+Kgain*(Yobs-H*xpre);
 
 y1a_2=xUp(1);
 y2a_2=xUp(2);
y3a_2=xUp(3);
 y4a_2=xUp(4);
y5a_2=xUp(5); 

      y1a=xUp(1);
      y2a=xUp(2);
      y3a=xUp(3);
      y4a=xUp(4);
      y5a=xUp(5);      
    
    Im=[1 0 0 0 0;0 1 0 0 0;0 0 1 0 0;0 0 0 1 0;0 0 0 0 1];

S1=[Im-Kgain*H]*Ppre*[Im-Kgain*H]';
PUp=S1+Kgain*R*Kgain';

  P=PUp ;   

   y1old=y1a_2;
   y2old=y2a_2;  
   y3old=y3a_2;
   y4old=y4a_2;  
   y5old=y5a_2;     

    stiff(i,j)=xUp(4);
    eta(i,j)=xUp(3);
    gama(i,j)=xUp(5);
end

end
%
for k=1:M
%
    xeta(k)=0.0;
    xstiff(k)=0.0;
    xalpha(k)=0.0;
%
    for j=1:N
%
    xeta(k)= xeta(k)+eta(k,j)/N;
    xstiff(k)= xstiff(k)+stiff(k,j)/N;
    xalpha(k)= xalpha(k)+gama(k,j)/N;
%     
    end
%    
end
%
figure(3)          % to plot Figure 6.4
plot(t,xeta,'black'),hold on
%
plot(t,xstiff,'black'), hold on
%
plot(t,xalpha,'black')
%
%
%
%______________________________________________________________________
%
%
%                   Case 2:   obseration on velocity
%
H=[0 1 0 0 0];  % observation matrix (only velocity is observed)
%           -------------------------------------------------
%
% data of Duffing oscillator (Equation 6.64)
c=0.9;   %Ghosh et al. 2007 (Comput. Methods Appl. Mech. Engrg, V 196)
k=7;
w=sqrt(k);
eta=c/(2*w);
alpha=3.0;
A=4;
Lamda=3.5;
%
sigx=0.001;
sigy=0.03;
g0=sigx;
%
%
R = 0.009; % measurement error covariance
Qpre=zeros(5,5);
phi=zeros(5,5);
% set up the size of the innovations vector
Inn = zeros(size(R));
%
%error covariance marix  P - initialization
P=zeros(5,5);
P(1,1)=0.01;
P(2,2)=0.01;
P(3,3)=1;
P(4,4)=1;
P(5,5)=1;
%
c7=0.00002;
sig=c7;
%
M=numel(t);
xp=zeros(M,N);
dxp=zeros(M,N);
%
%
for j=1:N
  j      % simulation counter
% initializaion of states
x0=0;
dx0=0;
%
Qpre=zeros(5,5);
phi=zeros(5,5);
%
% set up the size of the innovations vector
Inn = zeros(size(R));
%
%error covariance marix  P - initialization
P=zeros(5,5);
P(1,1)=0.01;
P(2,2)=0.01;
P(3,3)=1;
P(4,4)=1;
P(5,5)=1;
xp=zeros(M,N);
dxp=zeros(M,N);
for i=1:M
    xest(i,j)=0.0;
    dxest(i,j)=0.0;
    xestr(i,j)=0.0;
    dxestr(i,j)=0.0;
    yOBS(i,j)=0.0;
    xstiff(i,j)=0;
    xeta(i,j)=0;
    xalpha(i,j)=0;
end
%
 y1old=x0;
  y2old=dx0;
  y3old=2;     % c
  y4old=5; % K
  y5old=5;  % alpha   % these five for getting the predicted state 
  
  y1aold=x0;
  y2aold=dx0;  
  y3aold=0.9;     % c
  y4aold=7; % K
  y5aold=3;  % alpha  % these five for getting the true state 

for i=2:M
%    t(i) 

u3=randn;
u4=randn;
u5=randn;
  
I3 = sqrt(dt*c7)*u3;
I4 = sqrt(dt*c7)*u4;
I5 = sqrt(dt*c7)*u5;
 
 y3aold= y3aold+I3;
y4aold= y4aold+I4;
 y5aold= y5aold+I5;   
%
 
  %  intermediate variables
  c1=y3old;
  c2=y4old;
  c3=A;
  c4=sigx;  
  c5=0.0;
  g0=c4;
  c6=y5old;
%
  
%  Step 1.  to find xpre form process equation without noise   
        y1a=y1old;
    y2a=y2old;

    err=1;

    while err>1e-4
      y1a_1=y1a;
      y2a_1=y2a;

      phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1old - phi12*y2old;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);
 
      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);
    
      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1old + phi22*y2old;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
     
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;
  
       err=(y1a_2-y1a_1)^2;
        err=abs(y1a_2-y1a_1);

     y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop

 xpre = [y1a; y2a; c1; c2; c6];

 xp(i,j)=y1a;
 dxp(i,j)=y2a;
%
%  Step 2.  to find true solution from process equation without noise 
                     % LTL method
%  intermediate variables
  c1=y3aold;
  c2=y4aold;
  c3=A;
  c4=sigx;  
  c5=0.0;
  g0=c4;
  c6=y5aold;
  
 y1a=y1aold;
  y2a=y2aold;

    err=1;

    while err>1e-4
        
      y1a_1=y1a;
      y2a_1=y2a;

    phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;

      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);

      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;

      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);

      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);

      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);

      fun1 = y1a - phi11*y1aold - phi12*y2aold;
      f1 = (1/Lamda)*(sin(Lamda*t(k)) - sin(Lamda*t(k-1)));
      f2 = c6*y1a^3*h;
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);

      df2=3*c6*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);

      y1new=y1a-fun1/df1;

      ff2 = c6*y1new^3*h;
      fun2 = phi21*y1aold + phi22*y2aold;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
      y2new=fun2;

      y1a_2=y1new;
      y2a_2=y2new;     

      err=(y1a_2-y1a_1)^2;
 
      err=abs(y1a_2-y1a_1);
      
      y1a=y1a_2;
      y2a=y2a_2;
      
  end                   % end of while err loop  
   
 xTr=[y1a; y2a; c1; c2; c6];
 
 y1aold=xTr(1);
 y2aold=xTr(2);
 
    y3aold=xTr(3);   %c
    y4aold=xTr(4);    %K
    y5aold=xTr(5);    % alpha
    
%
% 
      y1a=y1old;
      y2a=y2old;
      y3a=y3old;
      y4a=y4old;
      y5a=y5old;
%
 % Step 3. to find predicted covariance matrix P at current t
 %       for this to first find Q matrix (Equation 6.48) - which is the 
 %                 process error covariance matrix

 tmin=t(i-1);
 tmax=t(i);
hj=tmax-tmin;  
hk=hj;

Qpre=zeros(5,5);


Qpre(1,1)=0.25*sigx^2*hj^2*hk*2  ;

Qpre(1,2)=sigx^2*(0.5*hj^2*hk-0.25*y3a*hj^2*hk^2)  ;
Qpre(2,2)=sigx^2*(hj-0.5*y3a*hj^2)*(hk-0.5*y3a*hk^2) ;
Qpre(2,2)=Qpre(2,2)+0.25*y2a^2*sig^2*hj^2*hk^2;
Qpre(2,2)=Qpre(2,2)+0.25*y1a^2*sig^2*hj^2*hk^2;
Qpre(2,2)=Qpre(2,2)+0.25*y1a^6*sig^2*hj^2*hk^2;


Qpre(2,3)=-0.5*sig^2*y2a*hj^2*hk;
Qpre(2,4)=-0.5*sig^2*y1a*hj^2*hk;
Qpre(2,5)=-0.5*sig^2*y1a^3*hj^2*hk;

Qpre(3,3)=sig^2*hj*hk;
Qpre(4,4)=sig^2*hj*hk;
Qpre(5,5)=sig^2*hj*hk;

Qpre(2,1)=Qpre(1,2);

Qpre(3,2)=Qpre(2,3);

Qpre(4,2)=Qpre(2,4);

Qpre(5,2)=Qpre(2,5);
     
     % to find Ppre matrix     
     phi=zeros(5,5);     
     phi(1,1)=1;
     phi(1,2)=h;   
     
     phi(2,1)=-h*(y4a+3*y1a^2*y5a);
     phi(2,2)=1-h*y3a;
     phi(2,3)=-h*y2a;
     phi(2,4)=-h*y1a;
     phi(2,5)=-h*y1a^3;
     phi(3,3)=1;
     phi(4,4)=1;
     phi(5,5)=1;
 
     Ppre=phi*P*phi'+ Qpre;
     
 % Step 4 to find Kalman gain matrix 
 AA=H*Ppre*H'+R;
 Kgain=Ppre*H'*inv(AA);
 
 % Step 5.  Observations
 u1=sqrt(dt*sigy)*randn; 
 Yobs=H*xTr+u1;
 
 % Step 6. Udate the state
 xUp=xpre+Kgain*(Yobs-H*xpre);
 
 y1a_2=xUp(1);
 y2a_2=xUp(2);
y3a_2=xUp(3);
 y4a_2=xUp(4);
y5a_2=xUp(5); 

      y1a=xUp(1);
      y2a=xUp(2);
      y3a=xUp(3);
      y4a=xUp(4);
      y5a=xUp(5);     
    
    Im=[1 0 0 0 0;0 1 0 0 0;0 0 1 0 0;0 0 0 1 0;0 0 0 0 1];

S1=[Im-Kgain*H]*Ppre*[Im-Kgain*H]';
PUp=S1+Kgain*R*Kgain';

  P=PUp ; 
   
   y1old=y1a_2;
   y2old=y2a_2;  
   y3old=y3a_2;
   y4old=y4a_2;  
   y5old=y5a_2;
     
%     xest(i,j)=xUp(1);
%     dxest(i,j)=xUp(2);
%     
%  xestr(i,j)=xTr(1);
%     dxestr(i,j)=xTr(2);
%     
%     yOBS(i,j)=Yobs(1);
    stiff(i,j)=xUp(4);
    eta(i,j)=xUp(3);
    gama(i,j)=xUp(5);
end

end



%
for k=1:M
%
    xeta(k)=0.0;
    xstiff(k)=0.0;
    xalpha(k)=0.0;
%
    for j=1:N
%
    xeta(k)= xeta(k)+eta(k,j)/N;
    xstiff(k)= xstiff(k)+stiff(k,j)/N;
    xalpha(k)= xalpha(k)+gama(k,j)/N;
%     
    end
%    
end
%
figure(4)    % to plot Figure 6.5
plot(t,xeta,'black'),hold on
%
plot(t,xstiff,'black'), hold on
%
plot(t,xalpha,'black')
%
