%                         Example6_3_Fig6_6and7.m
%
%                 Parameter estimation by Boot strap MC filter  
%                   of Van der Pol oscillator

%         Figure 6.6   - limit cycle with Np=100
%         Figure 6.7   - Parameter estimation with Np=100 & 1000
%
%                    In Figure 6.7, two results on parameter estimation
%                           (from the same Bootstrap filter) are obtained 
%                           one with Np=100 & the second with Np=1000
%                           (i.e., the same programme is repeated twice 
%                               independently -
%                           one for Np=100 and the second for Np=1000)%                               
%__________________________________________________________________________
%         
clc;
clear all;
close all;
%
lll=1;
T=30;
dt=0.01;
t=0:dt:T;
n_sub_in=10;
%__________________________________________________________________________
%               1st result with Bootstrap filter for Np=100
Np=100;
N=Np;  
%__________________________________________________________________________
%
D=2;    % dimension of the problem -  two states  x and dx
%
Nm=1;   % observation of one state only i.e., either x or dx
%
%
%
Index=1;%  Index for measurement  -  1 for displacement and 2 for velocity;
%
%
% Parameters of vanderpol (Equation 6.77)
c=4; %mu
k=10; %al
%
%
X_mess(1:2,1)=[1.4,0];  % initial conditions for x and dx
%
%%% Generating measurements at finer time step = dt/n_sun_in
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
    X_mess(:,ijk+1)=X_mess(:,ijk)+[X_mess(2,ijk);c*(1-X_mess(1,ijk)^2)*X_mess(2,ijk)-k*X_mess(1,ijk)]*dt/n_sub_in;
   end
%
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
%
X_ref=X_mes;   % True solution (obtained above with no noise)
%
for ijk=1:numel(t)
    X_mes(1:D,ijk)=X_mes(1:D,ijk)+0.05*max(X_mes(1:D,:)')'*randn(1);   % measurement with noise
end
%
z(1:Nm,:)=X_mes(Index,:); % Measurement variable of x or dx depending on Index =1 or 2 respectively
%
% Noise variance
sigma_meas=0.1*diag(max(z'))/sqrt(dt);
sig_pros=0.001;      %%process noise
sigma_pross=zeros(D);
sigma_pross(D,D)=sig_pros;  
sigma_c=0.5;       % intensity of BM for parameter c  - Equation 6.78c
sigma_k=0.5;       % intensity of BM for parameter k  - Equation 6.78d
   Randn=randn(D,Np);
%------------------------------------------------------------------------
% Initial displacement and velocity ensemble 
X(1:D,:)=[0*ones(1,Np);0*ones(1,Np)]+1e2*sigma_pross(D,D)*sqrt(dt)*Randn;
%
%  
X(3,:)=2*ones(1,Np)+0.1*sigma_c*sqrt(dt)*rand;     % for parameter c - Equation 6.78c
X(4,:)=8*ones(1,Np)+0.1*sigma_k*sqrt(dt)*rand;     % for parameter k - Equation 6.78d
% 
X_s(:,1)=mean(X,2);     % initial mean values for x, dx and parameters c and k (for plotting purposes)
%
%
%
% Starting computation
 tic
for i=1:numel(t)-1          % time integration starts
    i   % time counter
 %          
       for j=1:Np        
        b=[X(2,j);X(3,j)*(1-X(1,j)^2)*X(2,j)-X(4,j)*X(1,j)];
        X1(1:2,j)=X(1:D,j)+b*dt+sigma_pross*sqrt(dt)*randn(D,1);
        X1(3,j)=X(3,j)+ sigma_c*sqrt(dt)*randn;
        X1(4,j)=X(4,j)+ sigma_k*sqrt(dt)*randn;
       end
 %       
        for j=1:Np         
          wei(j)=multinorm_pdf(z(:,i+1),X1(Index,j),sigma_meas'*sigma_meas*(dt));
        end
        wei=wei/sum(wei);
        new_wei=resample(wei);   % resampling
        X1(1:4,:)=X1(1:4,new_wei);
        X2=X1;  
%  
    X(1:4,:)=X2(1:4,:);
    X_s(:,i+1)=mean(X,2);   % mean of the states
%
%
%  
%  
end
%
toc       
 %   
        figure(1);hold on;     % limit cycle - Figure 6.6
        plot(X_ref(1,:),X_ref(2,:),'black'); hold on   % true solution
        plot(X_s(1,:),X_s(2,:),'--black'); hold on     % estimate of limit cycle by BS filter
        
        figure(2),hold on      % Figure 6.7 with Np=100
        plot(t,c*ones(1,size(t,2)),'black'); hold on   % reference value for c
        plot(t,X_s(3,:),'--black'); hold on              % estimate of c by BS filter
       
        plot(t,k*ones(1,size(t,2)),'black'); hold on   % reference value for k
        plot(t,X_s(4,:),'--black'); hold on              % estimate of k by BS filter 
 %   
%__________________________________________________________________________
clear all
%               2nd  result with Bootstrap for Np=1000  
Np=1000;N=Np;    
%__________________________________________________________________________

T=30;
dt=0.01;
t=0:dt:T;
n_sub_in=10;
%

%
D=2;    % dimension of the problem -  two states  x and dx
% %
Nm=1;   % observation of one state only i.e., either x or dx
% %
% %
% %
Index=1;%  Index for measurement  -  1 for displacement and 2 for velocity;
%
%
% % Parameters of vanderpol (Equation 6.77)
c=4; %mu
k=10; %al
%
% %
X_mess(1:2,1)=[1.4,0];  % initial conditions for x and dx
%
%%% Generating measurements at finer time step = dt/n_sun_in
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
    X_mess(:,ijk+1)=X_mess(:,ijk)+[X_mess(2,ijk);c*(1-X_mess(1,ijk)^2)*X_mess(2,ijk)-k*X_mess(1,ijk)]*dt/n_sub_in;
   end
%
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
%
X_ref=X_mes;   % True solution (obtained above with no noise)
%
for ijk=1:numel(t)
    X_mes(1:D,ijk)=X_mes(1:D,ijk)+0.05*max(X_mes(1:D,:)')'*randn(1);   % measurement with noise
end
%
z(1:Nm,:)=X_mes(Index,:); % Measurement variable of x or dx depending on Index =1 or 2 respectively
%
% Noise variance
sigma_meas=0.1*diag(max(z'))/sqrt(dt);
sig_pros=0.001;      %%process noise
sigma_pross=zeros(D);
sigma_pross(D,D)=sig_pros; %  process noise  
sigma_c=0.5;
sigma_k=0.5;
   Randn=randn(D,Np);
%------------------------------------------------------------------------
% Initial displacement and velocity ensemble 
X(1:D,:)=[0*ones(1,Np);0*ones(1,Np)]+1e2*sigma_pross(D,D)*sqrt(dt)*Randn;
%
%  
X(3,:)=2*ones(1,Np)+0.1*sigma_c*sqrt(dt)*rand;     % for parameter c - Equation 6.78c
X(4,:)=8*ones(1,Np)+0.1*sigma_k*sqrt(dt)*rand;     % for parameter k - Equation 6.78d
% 
X_s(:,1)=mean(X,2);     % initial mean values for x, dx and parameters c and k (for plotting purposes)
%
%
%
tic
for i=1:numel(t)-1          % time integration starts
    i  % time counter
 %          
       for j=1:Np        
        b=[X(2,j);X(3,j)*(1-X(1,j)^2)*X(2,j)-X(4,j)*X(1,j)];
        X1(1:2,j)=X(1:D,j)+b*dt+sigma_pross*sqrt(dt)*randn(D,1);
        X1(3,j)=X(3,j)+ sigma_c*sqrt(dt)*randn;
        X1(4,j)=X(4,j)+ sigma_k*sqrt(dt)*randn;
       end
 %       
        for j=1:Np         
          wei(j)=multinorm_pdf(z(:,i+1),X1(Index,j),sigma_meas'*sigma_meas*(dt));
        end
        wei=wei/sum(wei);
        new_wei=resample(wei);   % resampling
        X1(1:4,:)=X1(1:4,new_wei);
        X2=X1;  
%  
    X(1:4,:)=X2(1:4,:);
    X_s(:,i+1)=mean(X,2);   % mean of the states
%
%
%  
%  
end
%
toc       
 %     
        figure(2),hold on    % Figure 6.7 with Np=1000
        plot(t,X_s(3,:),'black'); hold on     % estimate of parameter c by BS filter
        plot(t,X_s(4,:),'black'); hold on     % estimate of parameter k by BS filter 
