%                        Example6_3_Fig6_8.m
%
%                     Parameter estimation by Boot strap MC filter  
%                   of Van der Pol oscillator 
%
%                  Degeneracy wrt no. of particles
%                     ( Np=100, 500, 1000, 2000)
%
%             The starting values are taken as c=20  &  k=20
%    ( much away from the reference values - to get the degeneracy with low Np)
%                  (reference (true) values : c=4 and k=10)
%
%  NOTE:  change the value of Np for each run
%_____________________________________________________________________________
clc
clear all
close all
%
T=50;
dt=0.01;
t=0:dt:T;
n_sub_in=10;
MM=numel(t);
%
D=2;  % dimension of the problem -  two states  x and dx
Nm=1;  % observation of one state only i.e., either x or dx
%
%
Index=1;%  Index for measurement  -  1 for displacement and 2 for velocity;
%
% Parameters of vanderpol oscillator
c=4; 
k=10; 
%
%______________________________________________________________________
%   no. of particles;     Change for other values one by one
  Np=100;              %        Figure 6.8a
%   Np=500;            %        Figure 6.8b 
%   Np=1000;           %        Figure 6.8c
%   Np=2000;           %        Figure 6.8d
%______________________________________________________________________
%
%
%  start of computations
%
%     
X_mess(1:2,1)=[1.4,0]; % initial conditions for x and dx
%
%
% Generating measurements at finer time step = dt/n_sun_in
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
    X_mess(:,ijk+1)=X_mess(:,ijk)+[X_mess(2,ijk);c*(1-X_mess(1,ijk)^2)*X_mess(2,ijk)-k*X_mess(1,ijk)]*dt/n_sub_in;
   end
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
%
%
X_ref=X_mes;% True solution (obtained above with no noise)
%
%
for ijk=1:numel(t)
    X_mes(1:D,ijk)=X_mes(1:D,ijk)+0.05*max(X_mes(1:D,:)')'*randn(1); % measurement with noise
end

z(1:Nm,:)=X_mes(Index,:); %% Measurement variable of x or dx depending on Index =1 or 2 respectively

%% Noise variance
sigma_meas=0.1*diag(max(z'))/sqrt(dt);   % measurement noise variance
%
sig_pros=0.001; %%process noise intensity
sigma_pross=zeros(D);
sigma_pross(D,D)=sig_pros;
sigma_c=0.5;   % noise intensity for parameter equation of c - Equation 6.78c
sigma_k=0.5;   % noise intensity for parameter equation of k - Equation 6.78d
%
   Randn=randn(D,Np);
%------------------------------------------------------------------------
% Initial displacement and velocity ensemble
 X=zeros(4,Np);
X(1:D,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(D,D)*sqrt(dt)*Randn;%-.1; 

X(3,:)=20*ones(1,Np)+0.1*sigma_c*sqrt(dt)*rand;   % for parameter c - Equation 6.78c
X(4,:)=20*ones(1,Np)+0.1*sigma_k*sqrt(dt)*rand;   % for parameter k - Equation 6.78d
 
X_s(:,1)=mean(X,2);  % initial mean values for x, dx and parameters c and k (for plotting purposes)
%------------------------------------------------------------------------
%
%    
tic
for i=1:numel(t)-1
  i
    %     bootstrap filter         
       for j=1:Np        
         b=[X(2,j);X(3,j)*(1-X(1,j)^2)*X(2,j)-X(4,j)*X(1,j)];
         X1(1:2,j)=X(1:D,j)+b*dt+sigma_pross*sqrt(dt)*randn(D,1);
         X1(3,j)=X(3,j)+ sigma_c*sqrt(dt)*randn;
         X1(4,j)=X(4,j)+ sigma_k*sqrt(dt)*randn;
       end

        
        for j=1:Np
           wei(j)=multinorm_pdf(z(:,i+1),X1(Index,j),sigma_meas'*sigma_meas*(dt));
        end
        wei=wei/sum(wei);
        new_wei=resample(wei);
        X1(1:4,:)=X1(1:4,new_wei);
        X2=X1;
        
        X(1:4,:)=X2(1:4,:);
        X_s(:,i+1)=mean(X,2);   

end 
%        
        figure(1)    % Figure 6.8a - Np=100,  Figure 6.8b - Np=500,  Figure 6.8c - Np=1000,  Figure 6.8d - Np=2000
        
        plot(t,c*ones(1,size(t,2)),'black'); hold on  % refenece value of parameter c  
        plot(t,X_s(3,:),'black'); hold on      % estimate of parameter c by BS filter 

        plot(t,k*ones(1,size(t,2)),'black'); hold on   % refenece value of parameter k 
        plot(t,X_s(4,:),'black'); hold on      % estimate of parameter k by BS filter 
  
