%                           Example6_4_Fig6_10.m
%         
%
%            EnKF MC filter  Vs.   ABS & BS filters 
%
%                
%      Figure 6.10   - Parameter estimation of Van der Pol
%                                          oscillator
%
%
%      Three runs are executed in the program one after another
%                   - First is EnKF, second is ABS & third is BS
%__________________________________________________________________________                               
clc
clear all
close all
%
%__________________________________________________________________________
%                       1st Run
%
%                   start of compuation by EnKF 
%
A=diag(1,5);
%
T=50;
dt=0.01;
t=0:dt:T;
n_sub_in=10;
%
Np=500;
N=Np;
%
D=2;     % dimensionality of the oscillator - two states x and dx
%
Nm=1;% observation of one state only i.e., either x or dx
%
%
Index=1;%  Index for measurement  -  1 for displacement and 2 for velocity;
%
%
% true parameters of vanderpol
c=4; 
k=10; 
%
%
X_mess(1:2,1)=[1.4,0];% initial conditions for x and dx
%
%%% Generating measurements at finer time step = dt/n_sun_in
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
     X_mess(:,ijk+1)=X_mess(:,ijk)+[X_mess(2,ijk);c*(1-X_mess(1,ijk)^2)*X_mess(2,ijk)-k*X_mess(1,ijk)]*dt/n_sub_in;
   end
%
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
%
X_ref=X_mes;  % True solution (obtained above with no noise)
%
for ijk=1:numel(t)
    X_mes(1:D,ijk)=X_mes(1:D,ijk)+0.05*max(X_mes(1:D,:)')'*randn(1);
end
%
z(1:Nm,:)=X_mes(Index,:); % Measurement variable of x or dx depending on Index =1 or 2 respectively
%
% Noise variance
sigma_meas=0.1*diag(max(z'))/sqrt(dt);
%
sig_pros=0.001; %%process noise
sigma_pross=zeros(D);
sigma_pross(D,D)=sig_pros;
sigma_c=0.5;  % intensity of BM for parameter c - Equation 6.78c
sigma_k=0.5;  % intensity of BM for parameter k - Equation 6.78d
   Randn=randn(D,Np);
%------------------------------------------------------------------------
% Initial displacement and velocity ensemble 
X(1:D,:)=[0*ones(1,Np);0*ones(1,Np)]+1e2*sigma_pross(D,D)*sqrt(dt)*Randn;%-.1; 
%
X(3,:)=2*ones(1,Np)+0.1*sigma_c*sqrt(dt)*rand;
X(4,:)=8*ones(1,Np)+0.1*sigma_k*sqrt(dt)*rand;
% 
X_s(:,1)=mean(X,2);% initial mean values for x, dx and parameters c and k (for plotting purposes)
%
%--------------------------------------------------------------------------
tic
for i=1:numel(t)-1
    i     % time counter
          %    
      for j=1:Np        
        b=[X(2,j);X(3,j)*(1-X(1,j)^2)*X(2,j)-X(4,j)*X(1,j)];
        X1(1:2,j)=X(1:D,j)+b*dt+sigma_pross*sqrt(dt)*randn(D,1);
        X1(3,j)=X(3,j)+ sigma_c*sqrt(dt)*randn;
        X1(4,j)=X(4,j)+ sigma_k*sqrt(dt)*randn;      
      end       
 %      
        mean_X1=mean(X1,2);
        for u=1:Np
            no_mean_X1(:,u)=(1/sqrt(Np-1))*(X1(:,u)-mean_X1); % Equation 6.91
            new_z_un(:,u)=(1/sqrt(Np-1))*X1(Index,u);         % measurement error anamoly matrix
        end
%        
        mean_new_z_un=mean(new_z_un,2);
        for j=1:Np
            no_mean_new_z_un(:,j)=(new_z_un(:,j)-mean_new_z_un);
        end
        K_EnKF= no_mean_X1*no_mean_new_z_un'/(no_mean_new_z_un*no_mean_new_z_un'+sigma_meas'*sigma_meas*dt);
                                                  % Equation 6.93
%                                                  
        X2_mean=mean(X1,2)+K_EnKF*((z(:,i+1)-mean(X1(Index,:))));  % Equation 6.94
%        
        Cov_X=(no_mean_X1-K_EnKF* no_mean_new_z_un)*no_mean_X1';    % Equation 6.95
%        
        X2=gen_gauss_rnd(X2_mean,Cov_X,Np);     
%
      X=X2;  
      X_s(:,i+1)=X2_mean;  
%    
end
%
toc
%
        figure(1)     % Figure 6.10a - result by EnKF
        plot(t,c*ones(1,size(t,2)),'black'); hold on % reference value of parameter c 
        plot(t,X_s(3,:),'black'); hold on            % estimate of parameter c by EnKF filter
%        
        figure(2)    % Figure 6.10b - result by EnKF
        plot(t,k*ones(1,size(t,2)),'black'); hold on % reference value of parameter k 
        plot(t,X_s(4,:),'black'); hold on            % estimate of parameter k by EnKF filter
% 
%     End of ENKF computation
%_______________________________________________________________________________________________
%                                   2nd Run
%
%
%                               Start of computation by ABS filter
clear all;
%
T=50;
dt=0.01;
t=0:dt:T;
n_sub_in=10;
%
D=2;     % dimensionality of the oscillator - two states x and dx
%
Nm=1;% observation of one state only i.e., either x or dx
%
%
Index=1;%  Index for measurement  -  1 for displacement and 2 for velocity;
%
Np=500;N=Np;
%
aux_it=1;
%
%%% true parameters of Vanderpol oscillator
c=4;
k=10; 
%
X_mess(1:2,1)=[1.4,0];% initial conditions for x and dx
%
%%% Generating measurements at finer time step = dt/n_sun_in
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
    X_mess(:,ijk+1)=X_mess(:,ijk)+[X_mess(2,ijk);c*(1-X_mess(1,ijk)^2)*X_mess(2,ijk)-k*X_mess(1,ijk)]*dt/n_sub_in;
   end
%
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
%
X_ref=X_mes;% True solution (obtained above with no noise)
%
for ijk=1:numel(t)
    X_mes(1:D,ijk)=X_mes(1:D,ijk)+0.05*max(X_mes(1:D,:)')'*randn(1);
end
%
z(1:Nm,:)=X_mes(Index,:);  % Measurement variable of x or dx depending on Index =1 or 2 respectively
%
% Noise variance
sigma_meas=0.1*diag(max(z'))/sqrt(dt);
%
sig_pros=0.001; %%process noise
sigma_pross=zeros(D);
sigma_pross(D,D)=sig_pros;
sigma_c=0.5;% intensity of BM for parameter c - Equation 6.78c
sigma_k=0.5;  % intensity of BM for parameter k - Equation 6.78d
   Randn=randn(D,Np);
%------------------------------------------------------------------------
% Initial displacement and velocity ensemble 
X(1:D,:)=[0*ones(1,Np);0*ones(1,Np)]+1e2*sigma_pross(D,D)*sqrt(dt)*Randn;%-.1; 
%
X(3,:)=2*ones(1,Np)+0.1*sigma_c*sqrt(dt)*rand;
X(4,:)=8*ones(1,Np)+0.1*sigma_k*sqrt(dt)*rand;
%
X_s(:,1)=mean(X,2);% initial mean values for x, dx and parameters c and k (for plotting purposes)
%------------------------------------------------------------------------
%
tic
for i=1:numel(t)-1
    i         % time counter
 %        
     for aux=1:1
         for j=1:Np
           b=[X(2,j);X(3,j)*(1-X(1,j)^2)*X(2,j)-X(4,j)*X(1,j)];
           X1(1:2,j)=X(1:D,j)+b*dt+sigma_pross*sqrt(dt)*randn(D,1);
           X1(3,j)=X(3,j)+ sigma_c*sqrt(dt)*randn;
           X1(4,j)=X(4,j)+ sigma_k*sqrt(dt)*randn;             
         end
         beta_hat=zeros(1,N);
         for u=1:Np             
           beta_hat(u)=beta_hat(u)+1/aux_it*normpdf(z(i+1),X1(Index,u),sigma_meas*sqrt(dt));
         end
     end
     beta_hat1=beta_hat/sum(beta_hat);%%normalized
     beta_hat_new(:,1)=beta_hat(1,resample(beta_hat1));%%resampled beta hat
     X=X(:,resample(beta_hat1));
     X1_zeta=X1(:,resample(beta_hat1));
  %   
     for j=1:Np
        b=[X(2,j);X(3,j)*(1-X(1,j)^2)*X(2,j)-X(4,j)*X(1,j)];
        X1(1:2,j)=X(1:D,j)+b*dt+sigma_pross*sqrt(dt)*randn(D,1);
        X1(3,j)=X(3,j)+ sigma_c*sqrt(dt)*randn;
        X1(4,j)=X(4,j)+ sigma_k*sqrt(dt)*randn;
      end
%
     weight_sum=0;
     for vi=1:N
         weight_m(1,vi)=normpdf(z(i+1),X1(Index,vi),sigma_meas*sqrt(dt));
         New_wt(1,vi)=normpdf(z(i+1),X1_zeta(Index,vi),sigma_meas*sqrt(dt));
         weight(1,vi)=weight_m(1,vi)/New_wt(1,vi);
         weight_sum=weight_sum+weight(1,vi);
     end
     weight(1,:)=weight(1,:)/weight_sum(1);
     new_weight = resample(weight);
     X2= X1(:,new_weight);%% resampling 
 %
    X(1:4,:)=X2(1:4,:);
    X_s(:,i+1)=mean(X,2);     
 %  
end
%
toc
%        
        figure(1)    % Figure 6.10a - result by ABS filter
        plot(t,X_s(3,:),'--black'); hold on   % estimate of parameter c by ABS filter
        
        figure(2)    % Figure 6.10b - result by ABS filter
        plot(t,X_s(4,:),'--black'), hold on   % estimate of parameter k by ABS filter        
 %       
 %------------------------------------------------------------------- 
%                             3rd Run   
%
%                   Start of computaion by BS filter
%
T=50;
dt=0.01;
t=0:dt:T;
n_sub_in=10;
%
D=2;     % dimensionality of the oscillator - two states x and dx
%
Nm=1;% observation of one state only i.e., either x or dx
%
%
Index=1;%  Index for measurement  -  1 for displacement and 2 for velocity;
%
Np=500;N=Np;
%
aux_it=1;
%
%%% true parameters of Vanderpol oscillator
c=4;
k=10; 
%
X_mess(1:2,1)=[1.4,0];% initial conditions for x and dx
%
%%% Generating measurements at finer time step = dt/n_sun_in
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
    X_mess(:,ijk+1)=X_mess(:,ijk)+[X_mess(2,ijk);c*(1-X_mess(1,ijk)^2)*X_mess(2,ijk)-k*X_mess(1,ijk)]*dt/n_sub_in;
   end
%
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
%
X_ref=X_mes;  % True solution (obtained above with no noise)
%
for ijk=1:numel(t)
    X_mes(1:D,ijk)=X_mes(1:D,ijk)+0.05*max(X_mes(1:D,:)')'*randn(1);
end
%
z(1:Nm,:)=X_mes(Index,:);  % Measurement variable of x or dx depending on Index =1 or 2 respectively
%
% Noise variance
sigma_meas=0.1*diag(max(z'))/sqrt(dt);
%
sig_pros=0.001; %%process noise
sigma_pross=zeros(D);
sigma_pross(D,D)=sig_pros;
sigma_c=0.5;% intensity of BM for parameter c - Equation 6.78c
sigma_k=0.5;  % intensity of BM for parameter k - Equation 6.78d
   Randn=randn(D,Np);
%------------------------------------------------------------------------
% Initial displacement and velocity ensemble 
X(1:D,:)=[0*ones(1,Np);0*ones(1,Np)]+1e2*sigma_pross(D,D)*sqrt(dt)*Randn;%-.1; 
%
X(3,:)=2*ones(1,Np)+0.1*sigma_c*sqrt(dt)*rand;
X(4,:)=8*ones(1,Np)+0.1*sigma_k*sqrt(dt)*rand;
% 
X_s(:,1)=mean(X,2);% initial mean values for x, dx and parameters c and k (for plotting purposes)
%
%
tic
for i=1:numel(t)-1
    i           % time counter
    %                      
       for j=1:Np        
         b=[X(2,j);X(3,j)*(1-X(1,j)^2)*X(2,j)-X(4,j)*X(1,j)];
         X1(1:2,j)=X(1:D,j)+b*dt+sigma_pross*sqrt(dt)*randn(D,1);
         X1(3,j)=X(3,j)+ sigma_c*sqrt(dt)*randn;
         X1(4,j)=X(4,j)+ sigma_k*sqrt(dt)*randn; 
       end
  %      
        for j=1:Np
          wei(j)=multinorm_pdf(z(:,i+1),X1(Index,j),sigma_meas'*sigma_meas*(dt));
        end
        wei=wei/sum(wei);
        new_wei=resample(wei);
        X1(1:4,:)=X1(1:4,new_wei);
        X2=X1;     
%   
    X(1:4,:)=X2(1:4,:);
    X_s(:,i+1)=mean(X,2); 
 %   
end
%
toc
%
        figure(1),hold on  % Figure 6.10a - result by BS filter
        plot(t,X_s(3,:),'-.black'); hold on              % estimate of parameter c by BS filter  
        
        figure(2), hold on  % Figure 6.10b - result by BS filter
        plot(t,X_s(4,:),'-.black'); hold on              % estimate of parameter k by BS filter
    
        


