%                       Example7_2_Fig7_2c_and_d.m 
%
%
%                        Stochastic filtering
%
%              parameter estimation by IGSF bank with ADP 
%
%               20-dof shear frame - see Figure 6.11 of Chapter 6
%                           (under support excitation)
%
%     no. of system parameters - 40 (20 displacements + 20 velocities
%     no. of parameters to be estimated - 40 (20 stiffness values 
%                                             +  20 damping values)
%
%   Calls shear.m,  after_rev_to_gen_meas.m, gen_gauss_rnd.m
%                   after_rev_only_one_EnKF.m,  scaled_multinorm_pdf
%__________________________________________________________________________
close all
clear all;
clc;
%
dof=20;   % dimension of the MDOF system (see Figure 6.11 of Chapter 6)
%
T=5;   % total time of integration
hl=0.01; % time step
delt=hl;
NL=T/delt;% no. of time steps for integration
M=NL;
%
tic
for lll=1:1
    
    iterate=8;   % no. of inner iterations for gain update (of IGSF scheme)
%    
    ADP=3; %      alpha parameter in Equation (7.22)
%   
G=10; % No. of Gaussian terms in the mixture
%
n_particles=500; 
Np=n_particles;
N=n_particles;
% -------------------------------------------------------------------------
% deterministic excitation
Amp_F=0.1; %% % amplitude of the harmonic excitation on the shear frame
f_amp=1; % frequency of the harmonic excitation
%
omeg=1;
%--------------------------------------------------------------------------
%  Shear frame parameters
Mm=ones(1,dof);
m_rev=12;%
[K_new, C_new,phi_T_M,refk,refc,massref] =shear(m_rev,dof);% matrices for the shear frame generated

k_rev=K_new(1,1)/2;
c_rev=C_new(1,1)/2;
k_all=k_rev*ones(1,dof);
c_all=c_rev*ones(1,dof);
k=k_all(1:dof);
c=c_all(1:dof);
%--------------------------------------------------------------------------
%  measurement noise
sigma_vel2=0.005; 
% 
sigma_vel_all=sigma_vel2*ones(1,dof);


% 
sigma_meas2=0.5;
% 
sig_meas_disp_all=sigma_meas2*ones(1,dof);
sigma_vel=sigma_vel_all(1:dof);
sig_meas_disp=sig_meas_disp_all(1:dof);
%--------------------------------------------------------------------------
% process noise
sigma_k1=8;sigma_c1=0.1;
sigma_k=sigma_k1*ones(1,dof);
sigma_c=sigma_c1*ones(1,dof);
%
%--------------------------------------------------------------------------
%  generating measurements
[meas]=after_rev_to_gen_meas(dof,zeros(1,2*dof),hl,f_amp,omeg,M,Mm,c,k,phi_T_M,m_rev,Amp_F);
disp_ref=meas(1:dof,:);
vel_ref=meas(dof:2*dof,:);
%
%
for ii=1:dof
measure(ii,:)=disp_ref((ii),:)+0.03*max(disp_ref(ii))*randn(1,length(disp_ref));
%
end
%--------------------------------------------------------------------------
% Initial conditions for process equation
X_init_l1=zeros(4*dof,N);
k1=k(1,1);c1=c(1,1);
for ii=1:dof
X_init_l1(2*dof+ii,:)=0.4*k1+(1*k1-0.4*k1)*rand(1,N);
end
for ii=1:dof
X_init_l1(3*dof+ii,:)=0.4*c1+(1*c1-0.4*c1)*rand(1,N);
end
%--------------------------------------------------------------------------
% 
wt_g=(1/G)*ones(1,G);% Initial weights
%
unsc_mean_x_ana=mean(X_init_l1,2);
Error_cov_ana=diag(1e-8*ones(1,length(unsc_mean_x_ana)));
%
for jj=1:G
  G_unsc_mean_x_ana(:,jj)=unsc_mean_x_ana;
  G_Error_cov_ana(:,:,jj)= Error_cov_ana;
end
%
       for jj=1:G          
        unsc_mean_x_ana=G_unsc_mean_x_ana(:,jj);
        Error_cov_ana=G_Error_cov_ana(:,:,jj);
        X_init_l(:,:,jj) = gen_gauss_rnd(G_unsc_mean_x_ana(:,jj),G_Error_cov_ana(:,:,jj),n_particles);
       end
%

for i=1:M     % start of integration
    i
    T0_l=(i-1)*hl;
    T_l=i*hl;
    sigma_k=sigma_k*exp(-0.0025*(i-1));
    sigma_c=sigma_c*exp(-0.0025*(i-1));     
%   
    % prediction 
    for jj=1:G        
        [X_mean Y_mean K_EnKF Error_cov_ana Cov_ana,X_prev,meas_un]=after_rev_only_one_EnKF(dof,i,X_init_l(:,:,jj),hl,f_amp,omeg,N,Mm,....
            sigma_vel,sigma_k,sigma_c,sig_meas_disp,phi_T_M,m_rev,Amp_F);
%        
        for jjjj=1:n_particles
            unsc_mean_x_ana(:,jjjj) =X_prev(:,jjjj)+K_EnKF*(measure(:,i)-meas_un(:,jjjj));
        end
        Inter_update=unsc_mean_x_ana;
%        
    % Iterations
          for u=1:N
             meas_un(:,u)=unsc_mean_x_ana(1:dof,u);
          end
     for mmm=1:iterate
         if mmm<iterate
%
             del_N=((measure(:,i)*ones(1,n_particles)-meas_un));
             N_hat=del_N*del_N';
             K_new=(Inter_update-X_prev)*del_N'*inv(N_hat);
             X_last=X_prev+(1+ADP)*K_new*((measure(:,i)*(ones(1,length(meas_un)))-meas_un));
%
             Inter_update=X_last;
             for u=1:N
                 meas_un(:,u)=X_last(1:dof,u);
             end
%
         else
%
             del_N=((measure(:,i)*ones(1,n_particles)-meas_un));
             N_hat=del_N*del_N';
             K_new=(Inter_update-X_prev)*del_N'*inv(N_hat);
             X_last=X_prev+K_new*((measure(:,i)*(ones(1,length(meas_un)))-meas_un));
%
             Inter_update=X_last;
             for u=1:N
                 meas_un(:,u)=X_last(1:dof,u);
             end
%
         end
     end
% 
        G_unsc_mean_x_ana(:,jj)=mean(X_last,2);
        X_init_l(:,:,jj)=X_last;
        mean_Xlast=mean(X_last,2);
        for u=1:n_particles
            meas_un_latest(:,u)=X_last(1:dof,u);
        end
        Y_mean_latest=mean(meas_un_latest,2);
%        
        for u=1:n_particles
            no_mean_Xlast(:,u)=(1/sqrt(Np-1))*(X_last(:,u)-mean_Xlast);
            new_z_un_Xlast(:,u)=(1/sqrt(Np-1))*(meas_un_latest(:,u)-Y_mean_latest);
        end
%        
        P_err_ana=(no_mean_Xlast-K_new*new_z_un_Xlast)*no_mean_Xlast';
        G_Error_cov_ana(:,:,jj)=P_err_ana;
        Cov_ana=new_z_un_Xlast*new_z_un_Xlast'+diag(sig_meas_disp(1:dof).^2*hl);
%        
        px=scaled_multinorm_pdf(measure(:,i)- Y_mean_latest,zeros(1,length(Y_mean_latest))',Cov_ana);
        wt_y(jj)=px;
%    
    end
%   
    norm_wt_y=wt_y/sum(wt_y);
    fin_wt=norm_wt_y.*wt_g;
    norm_fin_wt=fin_wt/sum(fin_wt);
    wt_g=norm_fin_wt;
    clear wt_y
%    
    sum_mean=0;
    sum_P=0;
    for jj=1:G
        sum_mean=sum_mean+norm_fin_wt(jj)* G_unsc_mean_x_ana(:,jj);
    end
    for jj=1:G
        sum_P=sum_P+norm_fin_wt(jj)*(G_Error_cov_ana(:,:,jj)+(sum_mean-G_unsc_mean_x_ana(:,jj))*(sum_mean-G_unsc_mean_x_ana(:,jj))');
    end
%    
    stor_unsc_mean_x_ana(:,i)=sum_mean;   
%    
    save Bankdof5all_k_same_19_8_2014 ;
%    
end
%
 collect_9_10_12(:,:,lll)=stor_unsc_mean_x_ana;
% 
 save collect_Bankdof5; 
%  
close all;
load collect_Bankdof5; 
lll=1;
stor_unsc_mean_x_ana= collect_9_10_12(:,:,lll);
%
ttt=hl:hl:M*hl;
%
%--------------------------------------------------------------------------
%Plotting the results
figure(1)          % Figure 7.2c  -  estimated stiffness parameters 
ref1=refk;
plot(ttt,ref1*ones(1,length(ttt)),'black');hold on;% reference value for stiffness
for iii=2*dof+1:3*dof
plot(ttt,massref*stor_unsc_mean_x_ana(iii,:),'black');hold on;% plotting the estimates for stiffness
end
%
figure(2);     % Figure 7.2d  -  estimated damping parameters 
plot(ttt,refc*ones(1,length(ttt)),'black');hold on; % reference value for damping
for iii=3*dof+1:4*dof
  plot(ttt,massref*stor_unsc_mean_x_ana(iii,:),'black');hold on;% plotting the  estimates for damping
end
%  
end
toc

