%                        Example7_3_Fig7_3.m
%
%               Parameter estimation by Kushner (KS) Filter 
%
%           20-dof shear frame - (see Figure 6.11 of Chapter 6)
%                       (under support excitation) 
%
%     no. of system parameters - 40 (20 displacements + 20 velocities
%     no. of parameters to be estimated - 40 (20 stiffness values 
%                                             +  20 damping values)
%
%_________________________________________________________________________
clc
clear all
close all
%
Np=400;   % number of particles
%
%
T=5;
%
dt =0.01;   
t=0:dt:T;
n_sub_in=1;
%
plot_no=600;
%
D=20;   % dofs the shear frame
Nm=D;
%
%
%--------------------------------------------------------------------------
 Index=[D:-1:D-Nm+1];     % displacement measurements
%Index=[2*D:-1:2*D-Nm+1];   %  uncomment for run with velocity measurements 
%--------------------------------------------------------------------------
%  
 mref=2.1012*10^6
 kk=2.6125*10^8/mref
 cc=5.225*10^6/mref
 k=linspace(kk,kk,D);
 c=linspace(cc,cc,D);
%  ------------------------------------------------------------------------
%
%  generating measurements
X_mess(:,1)=0.1*ones(2*D,1);
%
t1=0:dt/n_sub_in:T;

% 
for ijk=1:numel(t1)-1
     force(ijk)=0.1*sin(2*pi*t1(ijk));
    %
    b=[X_mess(D+1:2*D,ijk); -c_mat(c)*X_mess(D+1:2*D,ijk)-k_mat(k)*X_mess(1:D,ijk)+force(ijk)];
    
    X_mess1=X_mess(1:2*D,ijk)+b*dt;
    %
    force(ijk+1)=0.1*sin(2*pi*t1(ijk+1));
    b1=[X_mess1(D+1:2*D,1); -c_mat(c)*X_mess1(D+1:2*D,1)-k_mat(k)*X_mess1(1:D,1)+force(ijk+1)];
    X_mess(1:2*D,ijk+1)=X_mess(1:2*D,ijk)+0.5*(b+b1)*dt;
 %   
end
%
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
%
for ijk=1:numel(t)
    X_mes(1:2*D,ijk)=X_mes(1:2*D,ijk)+0.05*max(X_mes(1:2*D,:)')'*randn(1);
end
%
z(1:Nm,:)=X_mes(Index,:);
%--------------------------------------------------------------------------
% reference solution
X_ref(1:2*D,:)=X_mes;
%
for i=1:numel(t)
    X_ref(2*D+1:4*D,i)=[k';c'];
end
% -------------------------------------------------------------------------
%
sigma_meas=0.05*diag(max(z'))/sqrt(dt);
%
Z(:,1)=zeros(Nm,1);
for i=1:numel(t)-1
    % 
    Z(:,i+1)=Z(:,i)+0.5*(sigma_meas\(z(:,i)+z(:,i+1)))*dt;
end
%--------------------------------------------------------------------------
% process noise
sig_pros=0.01*ones(1,D);
sig_k=.5*ones(D,1);
sig_c=0.05*ones(D,1);

sigma_vel=diag(sig_pros);
sigma_k=diag(sig_k);
sigma_c=diag(sig_c);


sigma_pross=zeros(4*D,4*D);

sigma_pross(D+1:2*D,D+1:2*D)=sigma_vel;
sigma_pross(2*D+1:3*D,2*D+1:3*D)=sigma_k;
sigma_pross(3*D+1:4*D,3*D+1:4*D)=0*sigma_c;
%--------------------------------------------------------------------------
%
Randn=rand(2*D,Np);
save ('rand_save.mat', 'Randn');
load rand_save.mat
%
for run=1:1   % independent MC runs
%
for j=1:Np
    X(1:D,j)=X_mess(1:D,1)+sigma_vel*sqrt(dt)*Randn(1:D,j); %%%%%%% Initial displacement ensemble
    X(D+1:2*D,j)=zeros(D,1)+1e3*sigma_vel*sqrt(dt)*Randn(D+1:2*D,j);%-.1; %%%%% Initial velocity ensemble
end
%
for i=1:D
    X(2*D+i,1:Np)=0.5*k(i)+(1*k(i)-0.5*k(i))*rand(1,Np);
    X(3*D+i,1:Np)=0.5*c(i)+(1*c(i)-0.5*c(i))*rand(1,Np);
end
X_s(:,1)=mean(X,2);
%    
    for i=1:numel(t)-1   % time loop
        [run,i]   % counter
%        
        for j=1:Np
           % 
            b=[X(D+1:2*D,j); -c_mat(X(3*D+1:4*D,j))*X(D+1:2*D,j)-k_mat(X(2*D+1:3*D,j))*X(1:D,j)+force(i);zeros(2*D,1)];
            Randn1=randn(4*D,1);
            X1(1:4*D,j)=X(1:4*D,j)+b*dt+sigma_pross(1:4*D,1:4*D)*sqrt(dt)*Randn1;
           %
            b1=[X1(D+1:2*D,j); -c_mat(X1(3*D+1:4*D,j))*X1(D+1:2*D,j)-k_mat(X1(2*D+1:3*D,j))*X1(1:D,j)+force(i+1);zeros(2*D,1)];
            X1(1:4*D,j)=X(1:4*D,j)+0.5*(b+b1)*dt+sigma_pross(1:4*D,1:4*D)*sqrt(dt)*Randn1;
 %           
        end
 %       
        error_norm=1;
        it_X=0;
        while error_norm>1e-3 
            it_X=it_X+1;
            for it_j=1:Np
                h(:,it_j)=sigma_meas\(X1(Index,it_j));
            end
            for h1=1:Nm
                dummy_sum=zeros(4*D,1);
                for j=1:Np
                    dummy_sum=dummy_sum+(h(h1,j).*X1(1:4*D,j));
                end
                gain1(:,h1)=dummy_sum/Np;
                gain2(:,h1)=mean(h(h1,:))*mean(X1(1:4*D,:),2);
                
            end
            kushner_gain=gain1-gain2;
 %           
 %               
                 if it_X==1
                    mean1=mean(X1(1:4*D,:),2);
                    mean2=kushner_gain*(Z(:,i+1)-Z(:,i) - mean(h,2))*dt;
                    alfa1=max(1,abs(goldensection(z(:,i+1),mean1,mean2,Index)));
                 else
                    alfa1=1+alfa1/exp(it_X);
                 end
%
%            
            for j=1:Np
                X2(1:4*D,j)=X1(1:4*D,j)+alfa1*kushner_gain*((Z(:,i+1)-Z(:,i)) - h(:,j)*dt  );
            end
            error_norm=norm(mean(X2(1:4*D,:),2)-mean(X1(1:4*D,:),2));
            X1=X2;
        end
%        
        X(1:4*D,:)=X2(1:4*D,:);
        X_s(:,i+1)=mean(X,2);
%        
    end
    for p1=1:4*D
        for time1=1:numel(t)
            path_X(p1,time1,run)= X_s(p1,time1);
        end     
    end
%  
% -------------------------------------------------------------------------
% plotting results
    for i=1:4*D
        %
        if i<=D
             figure(1+plot_no)   
            plot(t,X_s(i,:),'black'), hold on  % displacements - Not plotted
             plot(t,X_ref(i,:),'--black') ;
           % 
        elseif(i>D && i<=2*D)
              figure(2+plot_no)
           plot(t,X_s(i,:),'black'), hold on  % velocities - Not plotted
           plot(t,X_ref(i,:),'--black') ;
          %  
        elseif(i>2*D && i<=3*D)
              figure(3+plot_no)
           plot(t,mref*X_s(i,:),'black'), hold on  % Figure 7.3a - stiffness estimate
           plot(t,mref*X_ref(i,:),'--black') ;  
         elseif(i>3*D && i<=4*D)                   % Figure 7.3b - damping estimate
               figure(4+plot_no)
           plot(t,mref*X_s(i,:),'black'), hold on
           plot(t,mref*X_ref(i,:),'--black') ; 
        end
        saveas(gcf, [int2str(i+plot_no) '.fig']);
    end
    
end








