%                               Example7_4_Fig7_4.m
%
%               Parameter estimation by EnKS filter
%
%           Figure 7.4a and 7.4b -  Estimates by non-iterative version
%           Figure 7.4c and 7.4d -  Estimates by iterative version
%
%        20 dof Shear frame - A 80 dimensional filtering problem
%
%       parameters to be etimated : 20 stiffnesses
%                                   20 damping values 
%    
%
%  NOTE:  for running the non-iterative version, 
%                          uncomment the two lines 132 & 133 and comment 128 & 129
%         for running the iterative version, 
%                          uncomment the two lines 128 & 129 and comment 132 & 133
%
%__________________________________________________________________________
clear all
clc
close all
%
%
Np=800;
dt=0.01;
T=10;
t=0:dt:T;
% 
    plot_no=20;
%
%
D=20;            % dof of the shear frame
Nm=D;           % number of of sets of measurements
%--------------------------------------------------------------------------
% % % Generating random force
    force_rand=500*repmat(exp(-t)*abs(randn).*cos(5*t),D,1);%+0.001*randn(D,numel(t));
    save ('force_mat.mat', 'force_rand');
%
% load force_mat.mat;    % for repeated execution after the 1st
%--------------------------------------------------------------------------
 Index=[D:-1:D-Nm+1];  % UNCOMMENT this line for a run with measurements on displacements only
% Index=[2*D:-1:2*D-Nm+1];  % UNCOMMENT this line for a run with measurements on velocities only
%
%-------------------------------------------------------------------------
 % Shear frame mass, stiffenss and damping parametes
mref=2.1012*10^6;
kk=2.6125*10^8/mref;
cc=5.225*10^6/mref;
k=linspace(kk,kk,D);
c=linspace(cc,cc,D);
%
%--------------------------------------------------------------------------
% Initializing the displacements and velocities
X_mess(:,1)=0.1*ones(2*D,1);
t1=0:dt:T;

% Applying Heun's method to find the states at all times
for ijk=1:numel(t1)-1
    b=[X_mess(D+1:2*D,ijk); -c_mat(c)*X_mess(D+1:2*D,ijk)-k_mat(k)*X_mess(1:D,ijk)+force(ijk)];
    X_mess1=X_mess(1:2*D,ijk)+b*dt;
    b1=[X_mess1(D+1:2*D,1); -c_mat(c)*X_mess1(D+1:2*D,1)-k_mat(k)*X_mess1(1:D,1)+force(ijk+1)];
    X_mess(1:2*D,ijk+1)=X_mess(1:2*D,ijk)+0.5*(b+b1)*dt;
end
%
X_mes=X_mess(:,[1:numel(t1)]);
%--------------------------------------------------------------------------
% Creating reference values to compare
X_ref(1:2*D,:)=X_mes;
for i=1:numel(t)
    X_ref(2*D+1:4*D,i)=[k';c'];
end
%-------------------------------------------------------------------------- 
%  Generating measurements -  measurement vector  Y(t) - (D=20- dimensional)
z(1:Nm,:)=(X_mes(Index,:))+0.00005*diag(max(X_mes(Index,:)'))*randn(Nm,numel(t1));
sigma_meas=0.005*diag(max(z'))/sqrt(dt);
%--------------------------------------------------------------------------
% Process noise covariance matrix
sig_pros=0.01*ones(1,D);
sig_k=.5*ones(D,1);
sig_c=0.05*ones(D,1);
%
sigma_vel=diag(sig_pros);
sigma_k=diag(sig_k);
sigma_c=diag(sig_c);
%
sigma_pross=zeros(4*D,4*D);  %  process noise RXX (4Dx4D matrix)
%
sigma_pross(D+1:2*D,D+1:2*D)=sigma_vel;
sigma_pross(2*D+1:3*D,2*D+1:3*D)=sigma_k;
sigma_pross(3*D+1:4*D,3*D+1:4*D)=sigma_c;
%
X1 = zeros(4*D,Np);
X2 = zeros(4*D,Np);
X_s = zeros(4*D,numel(t));
%
Randn=rand(2*D,Np);
save ('rand_save.mat', 'Randn');
load rand_save.mat
%
for run=1:1  % Independent MC runs
 %   
    for j=1:Np
        X(1:D,j)=X_mess(1:D,1)+sigma_vel*sqrt(dt)*Randn(1:D,j);                 %%%%%%% Initial displacement ensemble
        X(D+1:2*D,j)=zeros(D,1)+1e3*sigma_vel*sqrt(dt)*Randn(D+1:2*D,j);%-.1;   %%%%%%% Initial velocity ensemble
    end
%    
    for i=1:D
        X(2*D+i,1:Np)=0.5*k(i)+(1*k(i)-0.5*k(i))*rand(1,Np);                    %%%%%%% Initial k ensemble
        X(3*D+i,1:Np)=0.5*c(i)+(1*c(i)-0.5*c(i))*rand(1,Np);                    %%%%%%% Initial c ensemble
    end
    X_s(:,1)=mean(X,2);
%    
    for i=1:numel(t)-1
        [run,i]
        hX=(X(Index,:));
        % Prediction Stage (Heun's Method)
        for j=1:Np
            b=[X(D+1:2*D,j); -c_mat(X(3*D+1:4*D,j))*X(D+1:2*D,j)-k_mat(X(2*D+1:3*D,j))*X(1:D,j)+force(i);zeros(2*D,1)];
            Randn1=randn(4*D,1);
            X1(1:4*D,j)=X(1:4*D,j)+b*dt+sigma_pross(1:4*D,1:4*D)*sqrt(dt)*Randn1;
            b1=[X1(D+1:2*D,j); -c_mat(X1(3*D+1:4*D,j))*X1(D+1:2*D,j)-k_mat(X1(2*D+1:3*D,j))*X1(1:D,j)+force(i+1);zeros(2*D,1)];
            X1(1:4*D,j)=X(1:4*D,j)+0.5*(b+b1)*dt+sigma_pross(1:4*D,1:4*D)*sqrt(dt)*Randn1;
        end        
%
% -------------------------------------       
%       parameters for Iterative EnKS   - Figure 7.4 (c) and (d)
%             tao=10;
%             CNT=10;
%--------------------------------------
%  parameters for Non-iterative EnKS    - Figure 7.4 (a) and (b)   
            tao=0;
            CNT=1;
%--------------------------------------------------------------------------
        X1_prd=X1;
        for cnt=1:CNT
            h=(X1(Index,:));
            X_prt=X1-repmat(mean(X1,2),1,Np);
            h_prt=h-repmat(mean(h,2),1,Np);            
%
                R1=sigma_meas^2*dt;R2=sigma_meas^2*dt;
                
                X_prt_til=repmat(mean(X1,2),1,Np)-repmat(mean(X,2),1,Np);
                h_prt_til=h-repmat(mean(hX,2),1,Np);
%--------------------------------------------------------------------------
                    a1=0.8;
%--------------------------------------------------------------------------
                G1=1/Np*X_prt*h_prt_til'/(a1*(h_prt*h_prt')*1/(Np-1)+(1-a1)*R1*eye(size(h_prt,1)));
                G2=1/Np*X_prt_til*h_prt'/(a1*(h_prt*h_prt')*1/(Np-1)+(1-a1)*R2*eye(size(h_prt,1)));
%   
                X2=X1+((1+tao)*(G1+G2)*(repmat(z(:,i+1),1,Np)- h ));
                tao=tao/exp(cnt);
%
            X1=X2;
        end
        X=X2;
        X(1:4*D,:)=X2(1:4*D,:);
        X_s(:,i+1)=mean(X,2);
    end
    for p1=1:4*D
        for time1=1:numel(t)
            sample_path(p1,time1,run)= X_s(p1,time1);
        end
    end
    for i=3:4
        for j=1:D
            figure(i+plot_no);hold on;   % Figures 7.5(a)&(b)for non-iterative EnKS
                                         % Figures 7.5(c)&(d)for iterative EnKS
                  plot(t,mref*X_s((i-1)*D+j,:),'black',t,mref*X_ref((i-1)*D+j,:),'--black') ;  
%
        end
    end    
%    
end
