%                               Example8_3_Fig8_10.m
%
%          GCEM -  Girsanov correction to EM method 
%
%                 solution to Hardening Duffing oscillator
%
%           Figure 8.10 contains a) history of mean displacement and
%                                b) history of mean velocity            
%__________________________________________________________________________
clc
clear all
close all
%
D=2; % dimension of  state space (displacement and velocity states)
Nm=D;     
%
Np=200;    % ensemble size
% 
%
%---------------------------------------------------------------------
% parameters of the oscillator ( Equation 8.52)
%   (note that w is lambda in Equation 8.52)
w=2*pi; c=4;k=100;alpha=100; m=1;
sigma=1e-2; % % noise strength - sigma
r=4;   % load amplitude, P
%
%initial conditions
y_init=[0;0];
y(:,1)=y_init;
%
%
%
%--------------------------------------------------------------------------
% To obtain the EM solution  ( which is diverging with the large time step))
dt1=0.05;   % time step
T=4;        % total time of integration
t_y=0:dt1:T;    % time variable
nT_meas=numel(t_y);
%
%
for i=1:nT_meas-1
    [y(:,i+1)]=dynamics_euler(y(:,i),dt1,i,r,w,c,k,alpha,m);
end
%
figure(2)
plot(t_y,y(1,:),'-.black'),hold on   % for displacement history - Figure 8.10a
figure(3)
plot(t_y,y(2,:),'-.black'),hold on   % for velocity history  - Figure 8.10b
%--------------------------------------------------------------------------
%
%
%   To obtain a true solution by EM method with a smal time step,dt=0.0001
%
dt_meas=0.0001;   % small time step
T=4;
t_meas=0:dt_meas:T;
nT_meas=numel(t_meas);
%
%
%
% initila conditions
x_init=[0;0];
%
%   Time integration starts
x(:,1)=x_init;
for i=1:nT_meas-1
    [x(:,i+1)]=dynamics_euler(x(:,i),dt_meas,i,r,w,c,k,alpha,m);
end
%
figure(2)
plot(t_meas,x(1,:),'--black'),hold on   % for displacement history - Figure 8.10a
figure(3)
plot(t_meas,x(2,:),'--black'),hold on   % for velocity  history - Figure 8.10b
%--------------------------------------------------------------------------
%
%
%
%  To obtain the corrected solution by GCEM scheme using filtering
%                                                   strategy)
dt=0.05;  % large time step
t=t_meas(1:dt/dt_meas:end);  % new time variable
% 
% 
% 
%
x_init=[0;0];
X=x_init*ones(1,Np);
X_s(:,1) = mean(X,2);
for i = 1:numel(t)-1
    i  % time step counter
     for j = 1:Np
        [tmp]=drift1(X(:,j),dt,i,r,w,c,k,alpha,m);
        b_pre(:,j)=tmp;
        X1(:,j) = X(:,j) + b_pre(:,j)*dt + diag([0;sigma])*randn(D,1);  % prediction (EM solution)
    end
%        
    error_norm = 1;
    it_X = 0;
    while error_norm>1e-4 && it_X<2
        it_X = it_X+1;
        for j = 1:Np
            [tmp]=drift2(X1(:,j),dt,i+1,r,w,c,k,alpha,m);
            b(:,j)=tmp;
            e(:,j) = (b(:,j) - b_pre(:,j));   %  error e(t)
        end
%        
            h=(1/.05)*e;  
            for h1=1:Nm   
                dummy_sum=zeros(D,1);
                for j=1:Np
                    dummy_sum=dummy_sum+(h(h1,j).*X1(:,j));
                end
                gain1(:,h1)=dummy_sum/Np;
                gain2(:,h1)=mean(h(h1,:))*mean(X1,2);
            end
            kushner_gain=gain1-gain2;
            G_lin=kushner_gain;
%
            for u = 1:Np
                X2(:,u) = X1(:,u) + G_lin*(0 - h(:,u)*dt);  % update
            end      
%       
        error_norm = norm(mean(X2,2) - mean(X1,2));
        X1 = X2;
    end
%
    X = X2;
    X_s(:,i+1) = mean(X,2);
%
end
%
plot_no=2;
%
    figure(plot_no);
    plot(t,X_s(1,:),'black');hold on;  % for displacement history - Figure 8.10a
    
    figure(plot_no+1);
    plot(t,X_s(2,:),'black');hold on;  % for velocity history - Figure 8.10b
% 

