%                               Example8_4_Fig8_12.m
%
%          GCEM -     Girsanov correction to EM method  
%
%                  A non-smooth oscillator (Figure 8.11)
%
%           Figure 8.12 contains a) history of mean displacement and
%                                b) history of mean velocity            
%__________________________________________________________________________
clc
clear all
close all
%
%
D=2; % dimension of  state space (displacement and velocity states)
Nm=D;
%
Np=100; % ensemble size
% 
%
%
%parmeters in the oscilltor(Equation 8.58)
%   (note that w is lambda in Equation 8.58)
 w=2*pi; c=4; k=100; m=1; g=0.0045; cc=5; kc=200;
sigma=0.01;    % assumed forward equation noise strength -  sigma
r=4;   % load amplitude
%
%
%--------------------------------------------------------------------------
%  unstable solution  by EM method with dt = 0.03 
dt_meas=0.03;   % time step
T=5;            % total time of integration
t_meas=0:dt_meas:T;   % time variable
nT_meas=numel(t_meas);
%
% initial conditions
x_init=[0.1;0.1];
x(:,1)=x_init;
for i=1:nT_meas-1
    [x(:,i+1)]=dynamics_euler(x(:,i),dt_meas,i,r,w,c,k,m,g,cc,kc);
%
end
 figure(1)
 plot(t_meas,x(1,:),'-.black'),hold on    % mean displacement - Figure 8.12a
 figure(2)
 plot(t_meas,x(2,:),'-.black'),hold on   % mean velocity - Figure 8.12b
%
%__________________________________________________________________________
%
%
%   To obtain the reference (true) solution  by EM method with fine time
%                                  step, dt=0.0001
%           
dt_meas=0.0001;
T=5;
t_meas=0:dt_meas:T;
nT_meas=numel(t_meas);
%
%
% initial conditions
x_init=[0.1;0.1];
x(:,1)=x_init;
%
for i=1:nT_meas-1
    [x(:,i+1)]=dynamics_euler(x(:,i),dt_meas,i,r,w,c,k,m,g,cc,kc);
  %
end
figure(1)
 plot(t_meas,x(1,:),'--black '),hold on  % mean displacement - Figure 8.12a
 figure(2)
 plot(t_meas,x(2,:),'--black'),hold on   % mean velocity - Figure 8.12b
%--------------------------------------------------------------------------
%
%   To obtain the corrected solution by GCEM method with dt=0.03
%
dt=0.03;
t=0:dt:T;
%
%
%
% initial conditions
x_init=[0.1;0.1];
X=x_init*ones(1,Np);
%
%
X_s(:,1) = mean(X,2);
for i = 1:numel(t)-1
    i   % time step counter
    for j = 1:Np
        [tmp]=drift1(X(:,j),dt,i,r,w,c,k,m,g,cc,kc);
        b_pre(:,j)=tmp;
        X1(:,j) = X(:,j) + b_pre(:,j)*dt + diag([0;sigma])*randn(D,1);  % prediction
    end
% 
    error_norm = 1;
    it_X = 0;
    while error_norm>1e-4 && it_X<2
        it_X = it_X+1;
        for j = 1:Np
            [tmp]=drift2(X1(:,j),dt,i+1,r,w,c,k,m,g,cc,kc);
            b(:,j)=tmp;
            e(:,j) = (b(:,j) - b_pre(:,j));
        end        
            h=(1/.077)*e; 
            for h1=1:Nm   
                dummy_sum=zeros(D,1);
                for j=1:Np
                    dummy_sum=dummy_sum+(h(h1,j).*X1(:,j));
                end
                gain1(:,h1)=dummy_sum/Np;
                gain2(:,h1)=mean(h(h1,:))*mean(X1,2);
            end
            kushner_gain=gain1-gain2;
            G_lin=kushner_gain;
 %
            for u = 1:Np
                X2(:,u) = X1(:,u) + G_lin*(0 - h(:,u)*dt);% Update
            end     
        %       
        error_norm = norm(mean(X2,2) - mean(X1,2));
        X1 = X2;
    end
% 
    X = X2;
    X_s(:,i+1) = mean(X,2);
%
end
%
    figure(1)
    plot(t,X_s(1,:),'black');hold on;% mean displacement -  Figure 8.12a
    figure(2) 
    plot(t,X_s(2,:),'black');hold on;%  mean velocity - Figure 8.12b

