%                       Example8_1_Fig8_5ab.m
%
%                         GCLM  correction 
%
%                   
%       Figure 8.5a  - E[X1^2] - stationary result + solution with and 
%                                        without correction
%       Figure 8.5b  - E[X2^2] - stationary result + solution with and 
%                                        without correction
%__________________________________________________________________________
clc; clear all; close all
%
ll=1;
%
% Input parameters of the SDOF (HD) oscillator
c=5.0; k=100;alpha=100;sigma=5.0;ome_ga=1;
%
NN=5000; % No of particles(Ensemble size)
%
NL=1000;% Time span
hh=0.01;
%
P=2;   % no. of dof (displacement and velocity)
% 
%For finding the stationary limits 
syms x_disp 
syms x_vel
syms q
B11=int(c*((k*q+alpha*q^3)/(0.5*sigma^2)),q,0,x_disp);
A_prev=int(exp(((-c*x_vel^2)/sigma^2)-B11),x_disp,-inf,inf);
A11=int(A_prev,x_vel,-inf ,inf);
R=double(1/A11);
%
%
%  Integration for - E[X1^2]
Z=int(int(x_disp^2*R*exp(((-c*x_vel^2)/sigma^2)-int(c*((k*q+alpha*q^3)/(0.5*sigma^2)),q,0,x_disp)),x_disp,-inf,inf),x_vel,-inf ,inf)
%
%
% Integration for  - E[X2^2]
Z1=int(int(x_vel^2*R*exp(((-c*x_vel^2)/sigma^2)-int(c*((k*q+alpha*q^3)/(0.5*sigma^2)),q,0,x_disp)),x_disp,-inf,inf),x_vel,-inf ,inf)
%
%__________________________________________________________________________
%            Start of computation for solution  without  correction
%            -----------------------------------------------------
% 
for j=1:NN                 % loop over samples
 j    % sample counter
A=[0 1;-k -c];
X=zeros(2,10);
xi(1)=0.0;
xi(2)=0.0;
% 
displ=zeros(1,NL);
vel=zeros(1,NL);
displ(1)=xi(1);
vel(1)=xi(2);
%
final_X=zeros(3,10);
XX=zeros(3,3);
%
Omega_init=0;
% 
% 
for l=0:NL-2
      
        
        T0=l*hh;
        T=T0+hh;
%
%
ta=0.5*hh*(1-1/sqrt(3));
tb=0.5*hh*(1+1/sqrt(3));
%
  TRa=0;TRb=0;
  TRa=(((expm(A*(hh-ta)))*[0;sigma]).^2);
  TRb=(((expm(A*(hh-tb)))*[0;sigma]).^2);
 TR= 0.5*hh*(TRa+TRb);
  Fa=(expm(A*(hh-ta)))* [0; -alpha*(displ(l+1)+vel(l+1)*(hh-ta))^3];
  Fb=(expm(A*(hh-tb)))* [0; -alpha*(displ(l+1)+vel(l+1)*(hh-tb))^3];
 Fab= 0.5*hh*(Fa+Fb);
Randn1=randn(1);
F11=sqrt(TR)*Randn1;
X=(expm(A*(T-T0)))*xi'+F11+Fab;
%
xi(1)=X(1);
xi(2)=X(2);
%
displ(l+2)=X(1);
vel(l+2)=X(2);
%
displ_sqr(l+2)=X(1)^2;
vel_sqr(l+2)=X(2)^2;
end
%    
Final_disp1(j,:)=displ_sqr;
Final_vel1(j,:)=vel_sqr;
%
end
%
MM1=mean(Final_disp1(1:j,:));
MV1=mean(Final_vel1(1:j,:));  % not used
%
 t=hh:hh:NL*hh;
% 
   figure(ll)    % Figure 8.5a  - E[X1^2]
   plot(t,MM1(:),'--black');hold on   
%   
   figure(10*ll)   % Figure 8.5b  - E[X2^2]
   plot(t,MV1(:),'--black');hold on
%
 toc;
 %      End of computation for the solution without correction
%__________________________________________________________________________
%
%
%
%
%
%   Start of computation for solution by lower order linearization
%   --------------------------------------------------------------
%
tic
hl=hh;
M=NL;% %Time span
N=NN;%  ensemble size
%
Pl=P;
% 
A=[0 1;-k -c];
%
X_init_l=zeros(2,N);
FinalX_disp_tiplus1_l=mean(X_init_l(1));
FinalX_vel_tiplus1_l=mean(X_init_l(2));
%
Phi_next_l=zeros(1,N);
Phi_max_l(1)=1;
 nn=0;
 Brow_l=zeros(1,Pl+2);
 Brow_l(1)=0;
%
acc_l=zeros(1,M);
rej_l=zeros(1,M);
total_l=zeros(1,M);
%
for i=1:M 
   %
   il=i               % time step counter
    T0_l=(i-1)*hl;
    T_l=i*hl; 
%
    for j=1:N    % loop over samples 
%
        xi_l(1)=X_init_l(1,j);% Initial values of disp and velocity
        xi_l(2)=X_init_l(2,j);
        Phi_1_l(1)=Phi_next_l(j);
%        
        I=0;
      while (I==0)
        Sol_x_l=zeros(2,Pl+2);
        Phi_1_l=zeros(1,Pl+2);
        New_Phi_l=zeros(1,Pl+2);      
%
        Sol_x_l(1,1)=xi_l(1);
        Sol_x_l(2,1)=xi_l(2);
%     
                T_random_l=T0_l+(T_l-T0_l)*rand(Pl,1);
                T_ascend_l=sort(T_random_l);
                T_final_l=cat(1,T0_l,T_ascend_l,T_l);              
%
        for p=2:Pl+2
            T_ini=T_final_l(p-1);
            T_f=T_final_l(p);
            h_inter=T_f-T_ini;
%            
ta=0.5*h_inter*(1-1/sqrt(3));
tb=0.5*h_inter*(1+1/sqrt(3));
%
 TRa=0;TRb=0;
 TRa=(((expm(A*(h_inter-ta)))*[0;sigma]).^2);
 TRb=(((expm(A*(h_inter-tb)))*[0;sigma]).^2);
 TR= 0.5*h_inter*(TRa+TRb);
 Fa=(expm(A*(h_inter-ta)))* [0; -alpha*(xi_l(1))^3];
 Fb=(expm(A*(h_inter-tb)))* [0; -alpha*(xi_l(1))^3];
 Fab= 0.5*h_inter*(Fa+Fb);
 Randn1=randn(1);
 F11=sqrt(TR)*Randn1;
%
            X_l=(expm(A*(T_final_l(p)-T_final_l(p-1))))*xi_l'+F11+Fab;
%            
            Biplus1_l=randn(1)*sqrt(T_final_l(p)-T_final_l(p-1));
            Brow_l(p)=Biplus1_l;
            Phi_1_l(p)=(-1/sigma)*alpha*3*X_l(1)^2*X_l(2)*Brow_l(p) + (1/2)*(1/sigma^2)*alpha^2*((X_l(1)^3)-(xi_l(1)^3))^2;
            xi_l(1)=X_l(1);
            xi_l(2)=X_l(2);
            Sol_x_l(1,p)=X_l(1);
            Sol_x_l(2,p)=X_l(2);
        end
        for ii=1:Pl+2
            if Phi_1_l(ii)<0
                New_Phi_l(ii)=Phi_1_l(ii);
            else New_Phi_l(ii)=0;
            end
        end
%
        New_Phi_sort_l=sort(New_Phi_l);
        C=abs(New_Phi_sort_l(1));
        Phi_1_l=Phi_1_l+C;
        U_l=rand(1);
% 
        for jj=2:Pl+1
            for kk=1:100
                W_l=(1/hl)*rand(1);

                if ((Phi_1_l(jj)<W_l)||(U_l>1/factorial(kk)))
                    if mod(kk,2)~=0;
                        if jj<Pl+1
%
                            NNN=0;
                            break;
                        else I=1;
                            NNN=0;acc_l(i)=acc_l(i)+1;
%     
                            X_inter_init1_l(:,j)=Sol_x_l(1,Pl+2);
                            X_inter_init2_l(:,j)=Sol_x_l(2,Pl+2);
                                                      
                            Weight_l(j)=exp(((-1/sigma)*alpha*(Sol_x_l(1,Pl+2)^3*Brow_l(Pl+2)-Sol_x_l(1,Pl+1)^3*Brow_l(Pl+1)-Sol_x_l(1,Pl+1)^3*(Brow_l(Pl+2)-Brow_l(Pl+1))))-C*(T_final_l(Pl+2)-T_final_l(Pl+1)));
%    
                             Brow_l(1)=Brow_l(Pl+2);
                            New_Phi_l(j)=Phi_1_l(Pl+2);
                            Time_dur_l(j,:)=T_final_l;
                        %disp('Accepted');
                            break;
                        end
                    else I=0;NNN=1;
                        rej_l(i)=rej_l(i)+1;
                        %disp('Not Accepted');
                        break;
                    end
                end
            end
           if NNN~=0 
               break;
           end
        end
      end
    end
%
total_l(i)=acc_l(i)+rej_l(i);
prob_l(i)=acc_l(i)/total_l(i);
X_init_prev_l=[X_inter_init1_l; X_inter_init2_l];
X_init_l=X_init_prev_l(:,resample(Weight_l));
%
Phi_next_l=New_Phi_l;
%
 if i<M
 Phi_max_l(i+1)=max(New_Phi_l);   
 FinalX_disp_tiplus1_l(i+1)=mean(X_init_l(1,:).^2);
 FinalX_vel_tiplus1_l(i+1)=mean(X_init_l(2,:).^2);
 end
%
end
%
ttt=hl:hl:M*hl;
figure(ll);    %  Figure 8.5a  - E[X1^2]
 plot(ttt,FinalX_disp_tiplus1_l,'black');hold on
 plot(ttt,double(Z),'--black');   % stationary value - E[X1^2]
 %
 %
 figure(10*ll);    %  Figure 8.5b  - E[X2^2]
 plot(ttt,FinalX_vel_tiplus1_l,'black');hold on
%
 plot(ttt,double(Z1),'--black');   % stationary value - E[X2^2]
%
% 
toc;
%
