%                        Fig9_9ace.m 
%
%               Lorenz oscillator (Equation 9.43)
%
%        Pseudo code 2  (ENKS-based optimization)  - Table 9.7
%
%             Uses coalescence and scrambling strategies in Equation (9.40)
%
%  This program contains only one run with Np=30
%
%         Figure 9.9a,c,e - contains results 
%                           with Np=30
%
%
% NOTE:  Since it is a stochastic search algorithm, 
%                   better convergence may be tried for a number of independent MC runs >1
%__________________________________________________________________________   
clc
clear all
close all
%
%
global sigma r b
%
% reference values for the three parameters
sigma = 10;
r = 28;
b = 8/3;
%
 gen_max=300;
%
% reference (true) values for plotting purposes
x1_ref(1:gen_max+1)=sigma;
x2_ref(1:gen_max+1)=r;
x3_ref(1:gen_max+1)=b;
%
%
[z]=lorenz_states();
Np=30; D=3;
%
 xl=-[10;10;0];xu=[51;60;40];
 x1=repmat(xl,1,Np)+diag(xu-xl)*rand(D,Np);
%
%  
 x1_init=x1;
% 
x1=x1_init;
%
% cost values with initial particles
for i=1:Np
    cost(i)=fun_chao(z,x1(:,i));
end
%
min_cost=min(cost);
cost_plot(1)=mean(cost);
%
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
x3_plot(1)=mean(x1(3,:));
count=0;
%
tic
while count<gen_max
    count   % counter
 %
    for  ii=1:Np
 %
        r1=randi(Np);
%       
        r1_st(ii)=r1;
        r2=ii;
        while (r2==r1 || r2==ii)
            r2=randi(Np);
        end
        r2_st(ii)=r2;
        r3=ii;
        while (r3==r1 || r3==r2 || r3==ii)
            r3=randi(Np);
        end
        r3_st(ii)=r3;
%
    end
%    
    X=x1;
    X_pr=x1; 
    hh=(x1(:,r2_st)-x1(:,r3_st));% innovation matrix  (m x Np) - (coalescence + scrambling)
% 
  
    h=[cost(r1_st)-min_cost; hh]  ;  % % total innovation matrix including the objective function (m+1 x Np)  
    h_pr=h;   
%
    [gain]=new_KS_EnKF_gain_matrix(X,h,h_pr,X_pr,Np);
%--------------------------------------------------------------------------
%
         theta=1-1/exp(count-1);
         F=theta;   
    for i=1:Np
%  
            trial(:,1)=x1(:,i)-2*F*gain*h(:,i);% Equation (9.40)
%
%
        score=fun_chao(z,trial);
        if score<=cost(i)
            x2(:,i)=trial;
            cost(i)=score;
        else
            x2(:,i)=x1(:,i);
        end
    end
%    
    min_cost=min(cost);
    x1=x2;
    count=count+1;
    cost_plot(count+1)=mean(cost);
    x1_plot(count+1)=mean(x1(1,:));
    x2_plot(count+1)=mean(x1(2,:));
    x3_plot(count+1)=mean(x1(3,:)); 
%   
end
%  
toc
%
iteration=1:(gen_max+1);
%
figure(1)
plot(iteration,x1_plot,'black'),hold on    %  Figure 9.9a
plot(iteration,x1_ref,'--black')
figure(2)
plot(iteration,x2_plot,'black'),hold on     % Figure 9.9c
plot(iteration,x2_ref,'--black')
figure(3)
plot(iteration,x3_plot,'black'),hold on     % Figure 9.9e
plot(iteration,x3_ref,'--black')
%__________________________________________________________________________
