%                        Fig9_131415.m 
%
%               Rotor bearing system (Figure 9.12 & Table 9.8)
%
%               Parameter estimation in frequency domain
%
%              by Pseudo code 2  (ENKS-based optimization) - Table 9.7
%
%  This program contains only one run with Np=30
%
%         Figure 9.13, 9.14 & 9.15 - contains results with Np=30
%
%   NOTE:  Better convergence may be tried for a number of independent MC runs >1  
%__________________________________________________________________________   
clc
clear all
close all
%
gen_max=200;
%
% reference values of the rotor-bearing parameters
 byy=4.378*10^7;
 bzz=4.378*10^7;
 cyy=2.627e3;
 czz=2.627e3;
%
% reference values for plotting purposes
x1_ref(1:gen_max+1)=byy;
x2_ref(1:gen_max+1)=bzz;
x3_ref(1:gen_max+1)=cyy;
x4_ref(1:gen_max+1)=czz;
%
%
% frequency bandwidth
fr=(1500:1:1800);
% to obtain frequency (unbalance) response
[z]=rotor_states_full_model(byy,bzz,cyy,czz);
%
%
figure(1)   % plots at dof - 41 and 42  (node 11) - Figure 9.13a
plot(fr,z(1,:)),hold on
plot(fr,z(2,:))
figure(2)       % plots at dof - 57 and 58 (node 15)-   Figure 9.13b
plot(fr,z(3,:)),hold on
plot(fr,z(4,:))
% 
%
Np=30; 
D=4;   % dimension of X (= m)
% 
%
 xl=[1*1e7;1*1e7;1e3;1e3];xu=[6*1e7;6*1e7;4e3;4e3];
 x1=repmat(xl,1,Np)+diag(xu-xl)*rand(D,Np);
% 
%  
 x1_init=x1;
%
x1=x1_init;
%
% function evaluation at initial values of the parameters
for i=1:Np
    cost(i)=fun_rotor(z,x1(:,i));
end
%
min_cost=min(cost);
cost_plot(1)=mean(cost);
%
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
x3_plot(1)=mean(x1(3,:));
x4_plot(1)=mean(x1(4,:));
%
count=0;
% 
%
%
tic
while count<gen_max
    count       % counter
%
    for  ii=1:Np
%
        r1=randi(Np);
%         
        r1_st(ii)=r1;
        r2=ii;
        while (r2==r1 || r2==ii)
            r2=randi(Np);
        end
        r2_st(ii)=r2;
        r3=ii;
        while (r3==r1 || r3==r2 || r3==ii)
            r3=randi(Np);
        end
        r3_st(ii)=r3;
  end
%    
    X=x1;
    X_pr=x1; 
%    
    hh=(x1(:,r2_st)-x1(:,r3_st));  % innovation matrix  (m x Np) - (coalescence + scrambling)
%
    h=[cost(r2_st)-min_cost; hh];   % total innovation matrix including the objective function ( m+1 x Np)  
    h_pr=[cost(r2_st)-min_cost; hh];
%       
    [gain]=new_KS_EnKF_gain_matrix(X,h,h_pr,X_pr,Np);
%
    theta=1-1/exp(count-1);
    F=theta;   
%    
    for i=1:Np
%
%
        trial(:,1)=x1(:,i)-2*F*gain*h(:,i);  % Equation (9.40)
%                    factor 2 above is for a better convergence       
%
%
        score=fun_rotor(z,trial);
        if score<=cost(i)
            x2(:,i)=trial;
            cost(i)=score;
        else
            x2(:,i)=x1(:,i);
        end
    end
    min_cost=min(cost);
    x1=x2;
    count=count+1;
    cost_plot(count+1)=mean(cost);
    x1_plot(count+1)=mean(x1(1,:));
    x2_plot(count+1)=mean(x1(2,:));
    x3_plot(count+1)=mean(x1(3,:));
    x4_plot(count+1)=mean(x1(4,:));
    mean(cost)
end
toc
%
iteration=1:(gen_max+1);
figure(3)                   % Figure 9.15a
plot(iteration,x1_plot,'black'),hold on
plot(iteration,x1_ref,'--black')
figure(4)                   % Figure 9.15b
plot(iteration,x2_plot,'black'),hold on
plot(iteration,x2_ref,'--black')
figure(5)                       % Figure 9.15c
plot(iteration,x3_plot,'black'),hold on
plot(iteration,x3_ref,'--black')
figure(6)                          % Figure 9.15d 
plot(iteration,x4_plot,'black'),hold on
plot(iteration,x4_ref,'--black')
%
figure(7)                           % Figure 9.14 
plot(iteration,cost_plot,'black')
