%                             called by Fig9_3a.m    
%__________________________________________________________________________
function [minimum,fval] = anneal(loss, parent, options)
%
%
def = struct(...
        'CoolSched',@(T) (.9*T),...   %0.8T changed to 0.9T
        'Generator',@(x) (x+(randperm(length(x))==length(x))*randn/100),...
        'InitTemp',5,...
        'MaxConsRej',1000,...
        'MaxSuccess',20,...
        'MaxTries',300,...
        'StopTemp',1e-12,...
        'StopVal',-Inf,...
        'Verbosity',2);
%
% Check input
if ~nargin %default options, give it and stop
    minimum = def;
    return
elseif nargin<2, % only objective function given, 
    error('MATLAB:anneal:noParent','You need to input a first guess.');
elseif nargin<3, % no options structure, use default
    options=def;
else % all input, check if options is complete
    if ~isstruct(options)
        error('MATLAB:anneal:badOptions',...
            'Input argument ''options'' is not a structure')
    end
    fs = {'CoolSched','Generator','InitTemp','MaxConsRej',...
        'MaxSuccess','MaxTries','StopTemp','StopVal','Verbosity'};
    for nm=1:length(fs)
        if ~isfield(options,fs{nm}), options.(fs{nm}) = def.(fs{nm}); end
    end
end
%
% main settings
newsol = options.Generator;      % neighborhood space function
Tinit = options.InitTemp;        % initial temp
minT = options.StopTemp;         % stopping temp
cool = options.CoolSched;        % annealing schedule
minF = options.StopVal;
max_consec_rejections = options.MaxConsRej;
max_try = options.MaxTries;
max_success = options.MaxSuccess;
report = options.Verbosity;
k = 1;                           % boltzmann constant
%
% counters 
itry = 1;
success = 0;
finished = 0;
consec = 0;
T = Tinit;
initenergy = loss(parent);
oldenergy = initenergy
total = 0;
if report==2, fprintf(1,'\n  T = %7.5f, loss = %10.5f\n',T,oldenergy); end
%
%
isuc=1;
temp(isuc)=T;
opt(isuc)=oldenergy;

while ~finished;
    itry = itry+1; % just an iteration counter
    current = parent; 
%    
% Stop or decrement Temparature criteria
    if itry >= max_try || success >= max_success;
        if T < minT || consec >= max_consec_rejections;
            finished = 1;
            total = total + itry;
            break;
        else
            T = cool(T);  % decrease T according to cooling schedule
            if report==2, % output
                fprintf(1,'  T = %7.5f, loss = %10.5f\n',T,oldenergy);
                isuc=isuc+1
                opt(isuc)=oldenergy
                temp(isuc)=T;
            end
            total = total + itry;
            itry = 1;
            success = 1;
        end
    end
%    
    newparam = newsol(current);
    newenergy = loss(newparam);
%    
    if (newenergy < minF),
        parent = newparam; 
        oldenergy = newenergy;
        break
    end
%    
    if (oldenergy-newenergy > 1e-6)
        parent = newparam;
        oldenergy = newenergy;
        success = success+1;
        consec = 0;
    else
        if (rand < exp( (oldenergy-newenergy)/(k*T) ));
            parent = newparam;
            oldenergy = newenergy;
            success = success+1;
        else
            consec = consec+1;
        end
    end
%
end
%
minimum = parent;
fval = oldenergy;

if report;
    fprintf(1, '\n  Initial temperature:     \t%g\n', Tinit);
    fprintf(1, '  Final temperature:       \t%g\n', T);
    fprintf(1, '  Consecutive rejections:  \t%i\n', consec);
    fprintf(1, '  Number of function calls:\t%i\n', total);
    fprintf(1, '  Total final loss:        \t%g\n', fval);
end
%
 iters=(1:isuc)
%
%plotting
figure(1)
plot(iters,opt,'black')   % figure 9.3a
% figure(2)
% plot(iters,temp,'black')   % optional
% 
end










