%       Fig9_4_5a_9_6ab.m
%
%      parameter identification of Van der Pol oscillator  by PSO
%
%      Figure 9.4  - referenec (observed solution)
%      Figure 9.5(a) - evolution of the objective function with iterations
%      Figure 9.6 (a&b) - evolution of the damping (c) and stiffness (k)
%                    parameters with terations
%
%
%  routines called by this program: vanderpol_states.m 
%                                   fun_vanderpol.m
%__________________________________________________________________________
clc
clear all
close all
% 
global eta stif
%
% following are the reference values for c (variable 'eta') 
%         and k (variable 'stif') to get the reference (observed) solution
eta=4;
stif = 10;
%
T=5.0;     %  total time of integration
dtt=0.01;    % time step size
nTt=T/dtt+1;
tt=linspace(0,T,nTt)'; % time variable vector
%
%--------------------------------------------------------------------------
% call to obtain the reference (observed) solution 
%           for x1(t) % x2(t) (Figure 9.4)  
[z]=vanderpol_states();
figure(4)
plot(tt,z(1,:),'black'),hold on    % plotting Figure 9.4
plot(tt,z(2,:),'black')
%--------------------------------------------------------------------------
%
%
Np=50;   % number of particles
Nd=2;    % number of parameters
npar=Nd;
%
%
% bounds for the two parameters
varhi(1)=10;
varhi(2)=20;
varlo(1)=1;
varlo(2)=5;
%
% initial population (of the Np particles) - position vectors
for i=1:npar
    for j=1:Np
       par(i,j)=(varhi(i)-varlo(i))*rand+varlo(i); % uniform distribution 
    end
end
x1=par;
%
%
%
% Fitness values for the initial population
for i=1:Np
    cost(i)=fun_vanderpol(z,x1(:,i));
    P_best(:,i)=x1(:,i);
end
%
%
%
%
% initialization for plotting purposes
min_cost=min(cost);
cost_plot(1)=mean(cost);
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
%
% to get Pbest and gbest (see Table 9.4)
[cost_min,G_loc]=min(cost);
G_best=P_best(:,G_loc);
%
%
% initialization of the velocity vector
v=zeros(Nd,Np);
%
%
it=1; % set iteration =1
it_max=150; % Maximum iterations
%
%
iteration=1:(it_max+1);
%
% reference (true and target) values for damping and stiffness parameters
x1_ref(1:iteration)=4;
x2_ref(1:iteration)=10;
%
theta_max=0.9; theta_min=0.2;
%
% c1 and c2 are the cognitive and social parameters (see Table 9.4)
c1=2.0;
c2=2.0;
%
tic
while it<it_max
    it   % iteration counter
    theta=theta_max-(theta_max-theta_min)*(it/it_max);  % weight parameter (Table 9.4)
    r1=rand(1);
    r2=rand(1);    
    v_new=theta*v+c1*r1*(P_best-x1)+c2*r2*(repmat(G_best,1,Np)-x1); % Equation 9.1a (Table 9.4)
    x_new=x1+v_new;   % Equation 9.1b (Table 9.4)
%    
    for i=1:Np
        %  
        cost_new(i)=fun_vanderpol(z,x_new(:,i));  
%        
        % updating Pbest  
        if cost_new(i)<=cost(i)
            P_best(:,i)=x_new(:,i);
        end
    end
    %
    %
     [cost_new_min,G_loc]=min(cost_new);
%     
     % updating gbest 
     if (cost_new_min<=cost_min)
     G_best=x_new(:,G_loc);
     cost_min=cost_new_min;
     end
     %
    %
% 
    x1=x_new;
    v=v_new;
    cost=cost_new;
%    
    it=it+1;
    it
    mean(cost)
%
    cost_plot(it+1)=mean(cost);
    x1_plot(it+1)=mean(x1(1,:));
    x2_plot(it+1)=mean(x1(2,:));
%    
end
%
toc
%
iteration=1:(it_max+1);
%
figure(1); hold on;
plot(iteration,cost_plot,'black')   % cost function  (Figure 9.5a)
%
figure(2); hold on;
plot(iteration,x1_plot,'black')    % damping parameter (Figure 9.6a)
plot(iteration,x1_ref,'--black')
%
figure(3); hold on;
plot(iteration,x2_plot,'black')    % stiffness parameter (Figure 9.6b)
plot(iteration,x2_ref,'--black')

