%       Fig9_5b_9_6cd.m
%
%      parameter identification of Van der Pol oscillator  by GA
%
%      Figure 9.4  - referenec (observed solution)
%      Figure 9.5(b) - evolution of the objective function with iterations
%      Figure 9.6 (c&d) - evolution of the damping (c) and stiffness (k)
%                    parameters with terations
%
%  routines called : vanderpol_states.m 
%                    fun_vanderpol.m
%__________________________________________________________________________
%
%
clear all; clc;close all;
%
global eta stif
% following are the reference values for c (variable 'eta') 
%         and k (variable 'stif') to get the reference (observed) solution
eta = 4;
stif = 10;
% call to obtain the reference (observed) solution 
%           for x1(t) % x2(t) (Figure 9.4)  
[z]=vanderpol_states();
%
Nd=2;% number of parameters
npar=Nd;
%-------------------------------
% parameters of GA
%
popsize=200; % set population size
Np=popsize;
%
mutrate=.005; % set mutation rate
selection=0.5; % fraction of population kept
%
Nt=npar; % continuous parameter GA 
keep=floor(selection*popsize); % #population members that survive
nmut=ceil((popsize-1)*Nt*mutrate); % total number of mutations
M=ceil((popsize-keep)/2); % number of matings
%---------------------------------------
%
% bounds for the two parameters  c and k
varhi(1)=10;   % upper bound for c
varhi(2)=20;    % upper bound for k
varlo(1)=1;     % lower bound for c
varlo(2)=5;    % lower bound for k
%
% initial population (of the Np particles) 
for i=1:npar
    for j=1:popsize
       par(i,j)=(varhi(i)-varlo(i))*rand+varlo(i); % uniform distribution 
    end
end
%
%
x1=par;
%-----------------------------------------------------------%
% Fitness values for the initial population
for i=1:Np
    cost(i)=fun_vanderpol(z,x1(:,i));
     P_best(:,i)=x1(:,i);
end
%----------------------------------------------------------
[cost,ind]=sort(cost) ;
par=par(:,ind); % sort continuous
minc(1)=min(cost); % minc contains min of cost
meanc(1)=mean(cost); % meanc contains mean of cost
%
% initialization of the plotting
min_cost=min(cost);
cost_plot(1)=mean(cost);
x1_plot(1)=mean(x1(1,:));
x2_plot(1)=mean(x1(2,:));
%
itr=1; 
itr_max=200;  % maximum number of iterations
iteration=1:(itr_max+1);
x1_ref(1:iteration)=4;
x2_ref(1:iteration)=10;
%
tic
%
while itr<itr_max
itr=itr+1; % increment of iteration
itr
%
% Pair and mate
M=ceil((popsize-keep)/2); % number of matings
prob=flipud([1:keep]'/sum([1:keep])); % weights
%
% chromosomes
odds=[0 cumsum(prob(1:keep))']; % probability
% distribution
% function
pick1=rand(1,M); % mate #1
pick2=rand(1,M); % mate #2
%
% ma and pa contain the indicies of the chromosomes
%        that will mate
ic=1;
while ic<=M
for id=2:keep+1
if pick1(ic)<=odds(id) & pick1(ic)>odds(id-1)
ma(ic)=id-1;
end
if pick2(ic)<=odds(id) & pick2(ic)>odds(id-1)
pa(ic)=id-1;
end
end
ic=ic+1;
end
%
% Single point crossover
ix=1:2:keep; % index of mate #1
xp=ceil(rand(1,M)*Nt) ;% crossover point
r=rand(1,M); % mixing parameter
%
for ic=1:M
    ic;
%     
xy=par(xp(ic),ma(ic))-par(xp(ic),pa(ic)); 
%
par(:,keep+ix(ic))=par(:,ma(ic)); % 1st offspring
par(:,keep+ix(ic)+1)=par(:,pa(ic)); % 2nd offspring
par(xp(ic),keep+ix(ic))=par(xp(ic),ma(ic))-r(ic).*xy;
%
par(xp(ic),keep+ix(ic)+1)=par(xp(ic),pa(ic))+r(ic).*xy;
%
if xp(ic)<npar 
par(:,keep+ix(ic))=[par(1:xp(ic),keep+ix(ic)); par(xp(ic)+1:npar,keep+ix(ic)+1)];
par(:,keep+ix(ic)+1)=[par(1:xp(ic),keep+ix(ic)+1); par(xp(ic)+1:npar,keep+ix(ic))];
end 
end
%
mrow=sort(ceil(rand(1,nmut)*(popsize-1))+1);
mcol=ceil(rand(1,nmut)*Nt);
%
for ii=1:nmut
% 
par(mcol(ii),mrow(ii))=(varhi(mcol(ii))-varlo(mcol(ii)))*rand+varlo(mcol(ii));
%
end 
%
%-----------------------------------------------
%
x1=par;
for i=1:Np
    cost(i)=fun_vanderpol(z,x1(:,i));
     P_best(:,i)=x1(:,i);
end
%
% Sorting the fitness 
[cost,ind]=sort(cost);
par=par(:,ind);
%
%
minc(itr+1)=min(cost);
meanc(itr+1)=mean(cost);
%-------------------------------
%    
    cost_plot(itr+1)=mean(cost);
    x1_plot(itr+1)=mean(x1(1,:));
    x2_plot(itr+1)=mean(x1(2,:)) ;
% Stopping criterion
if itr>itr_max | cost(1)<min(cost)
break
end
[itr cost(1)]
end 
toc
% figures
figure(1); hold on;
plot(iteration,x1_plot,'black')  % Figure 9.6c  - damping
plot(iteration,x1_ref,'--black')
%
figure(2); hold on;
plot(iteration,x2_plot,'black')   % Figure 9.6d  - stiffness
plot(iteration,x2_ref,'--black')
%
figure(4); hold on;               % Figiure 9.5b   - cost 
plot(iteration,cost_plot,'black')



