%       Fig9_5c_9_6ef.m
%
%      parameter identification of Van der Pol oscillator  by SA
%
%      Figure 9.4  - referenec (observed solution)
%      Figure 9.5(c) - evolution of the objective function with iterations
%      Figure 9.6 (e&f) - evolution of the damping (c) and stiffness (k)
%                    parameters with terations
%

%  routines called : vanderpol_states.m 
%                    fun_vanderpol.m
%__________________________________________________________________________
%
clc;
close all;
clear all;
%
global z eta stif
%
% following are the reference values for c (variable 'eta') 
%         and k (variable 'stif') to get the reference (observed) solution
eta=4;
stif = 10;
%
Time=5.0;   % Total time of integration
dtt=0.01;   % time step size
nTt=Time/dtt+1;
tt=linspace(0,Time,nTt)'; % time variable vector
%
% call to obtain the reference (observed) solution 
%           for x1(t) % x2(t) (Figure 6.4)  
[z]=vanderpol_states();
%
%
%
% bounds for the two parameters
varhi(1)=10;
varhi(2)=20;
varlo(1)=1;
varlo(2)=5;
%
% 
k = 1;                         
%
Tinit=5;   % initial temparature
iter = 1;
isuccess = 0;  % number of successes
ifinished = 0;
iconsec = 0;   % Number of consecutive rejections
T = Tinit;
%
% initial particle
param(1)=1;    
param(2)=1;
%
% Fitness values for the initial population
initenergy = fun_vanderpol(param);
oldenergy = initenergy;
%
total = 0;
 fprintf(1,'\n  T = %7.5f, cost = %10.5f\n',T,oldenergy);
%
isuc=1;
temp(isuc)=T;
opt(isuc)=oldenergy;
x1(isuc)=param(1);
x2(isuc)=param(2);
minT=1e-15;
minF=1e-5;
max_itr=200;   % maximum iterations
max_success=1000;
max_consec_rejections=50000;
tic
%
while iter<max_itr;
    iter = iter+1 ;    % increase the iteration counter
  [iter max_itr T]  
    
    current = param; 
%    
  % decrease T according to the cooling (annealing) schedule
    if iter >= max_itr || isuccess >= max_success;
        if T < minT || iconsec >= max_consec_rejections;
            ifinished = 1;
            total = total + iter;
            break;
        else
%   
             T = 0.8*T;  % decrease T according to cooling schedule
%  
%             
                isuc=isuc+1;
                opt(isuc)=oldenergy;
                x1(isuc)=param(1);
                x2(isuc)=param(2);
                temp(isuc)=T;
%             end
            total = total + iter;
            iter = 1;
            isuccess = 1;
        end
    end
%    
%    
    newparam(1)=(varhi(1)-varlo(1))*rand+varlo(1);
     newparam(2)=(varhi(2)-varlo(2))*rand+varlo(2);
%  
     newenergy = fun_vanderpol(newparam);
    newparam;
    if (newenergy < minF),
        param = newparam; 
        oldenergy = newenergy;
        break
    end
%    
    if (oldenergy-newenergy > 1e-6)
        param = newparam;
        oldenergy = newenergy;
        isuccess = isuccess+1;
        iconsec = 0;
    else
        if (rand < exp( (oldenergy-newenergy)/(k*T) ));
            param = newparam;
            oldenergy = newenergy;
            isuccess = isuccess+1;
        else
            iconsec = iconsec+1;
        end
    end
% 
end
toc
minimum = param;
fval = oldenergy;
%
% 
% 
%
 iters=(1:isuc);
%
%
figure(1)
plot(iters,opt)    % Figure 9.5c
figure(3)
plot(iters,x1)     % Figure 9.6e
figure(4)
plot(iters,x2)     % Figure 9.6f
%
% figure(2)
% plot(iters,temp)   % temparature










